/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2007 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


/**	<p>Classes that implement this interface provide Qwicap with instances of {@link Auth2Scheme}, which supply 
 * 	authentication &amp; authorization (Auth<sup><small>2</small></sup>) functionality. An authentication &amp; 
 * 	authorization scheme is associated with a Qwicap web application by an entry in its "web.xml" file, for example:
 * 	</p>
 *	
 *<pre>&lt;web-app&gt;
 *    ....
 *    &lt;servlet&gt;
 *        ....
 *        &lt;init-param&gt;
 *            &lt;param-name&gt;QwicapAuth2SchemeFactoryClassName&lt;/param-name&gt;
 *            &lt;param-value&gt;edu.utexas.its.eis.tools.fatcookie.qwicap.FatCookieAuth2SchemeFactory&lt;/param-value&gt;
 *        &lt;/init-param&gt;
 *        ....
 *    &lt;/servlet&gt;
 *    ....
 *&lt;/web-app&gt;</pre>
 *	
 *	<p>In other words, include within the "servlet" element an "init-param" element containing a "param-name" element 
 *	of "QwicapAuth2SchemeFactoryClassName" and a "param-value" element that specifies the fully-qualified name of the 
 * 	class that implements the <code>{@link Auth2SchemeFactory}</code> interface.
 * 	</p>
 *
 *	<p>If the web application doesn't need to acquire any information about the user from its associated 
 *	{@link Auth2Scheme}, then that's all there is to it; the web app doesn't even need to know that it has an 
 *	<code>Auth2Scheme</code>.
 * 	</p>
 *
 * 	@see Auth2Scheme
 * 	@see Auth2SchemeReply
 * 
 *	@author Chris W. Johnson
 */
public interface Auth2SchemeFactory {

/**	Invoked whenever Qwicap needs an {@link Auth2Scheme} instance in order to authenticate &amp; authorize 
 * 	(auth<sup><small>2</small></sup>) a client request.
 * 
 * @return					An {@link Auth2Scheme} instance. 
 * @throws Exception
 */
	public Auth2Scheme getAuth2SchemeInstance() throws Exception;
}
