/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2011 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


/**	<p>Instances of classes implementing this interface inform Qwicap of the outcome of an authentication &amp; 
 * 	authorization (Auth<sup><small>2</small></sup>) operation, as conducted by the 
 * 	{@link Auth2Scheme#qwicapAuth2SchemeHit Auth2Scheme.qwicapAuth2SchemeHit} method. It tells Qwicap whether a user 
 * 	can access a web application's static content and/or its dynamic content. If the user is not be to permitted access 
 * 	to either type of content, Qwicap prevents all access to the web application, and does so in such a way that no 
 * 	server-side state is created for the user.
 * 	</p>	
 * 
 * 	<p>An authenication &amp; authorization scheme that wishes to provide its client application with additional 
 * 	information about users should add the necessary retrieval methods to the class that implements this 
 * 	<code>Auth2SchemeReply</code> interface. The client application can retrieve the instance of that class which 
 * 	concerns the most recent, valid hit by invoking the {@link Qwicap#getAuth2SchemeReply()}
 * 	method, and can then typecast the returned object to whatever class it knows that its {@link Auth2Scheme} is 
 * 	supplying. It can then use whatever methods that class supplies for retrieving information about the user. 
 * 	(Since it seems impossible to indentify a truly universal subset of authentication &amp; authorization information, 
 * 	and its representation, the <code>Auth2SchemeReply</code> interface makes no effort to specify such methods.)
 * 	</p>
 * 
 * 	@see Auth2Scheme
 * 	@see Auth2SchemeFactory
 * 
 * 	@author Chris W. Johnson
 */
public interface Auth2SchemeReply {
		
/**	Returns <code>true</code> to inform Qwicap that requests for static files in the web application should be 
 * 	fulfilled, or <code>false</code> if they should be refused.
 * 
 * @return	<code>true</code> to inform Qwicap that requests for static files in the web application should be 
 * 			fulfilled, or <code>false</code> if they should be refused.
 */
	public boolean qwicapCanSendStaticFiles();

/**	Returns <code>true</code> to inform Qwicap that requests intended for the web application should be passed along to
 * 	it, or <code>false</code> if they should be discarded. This also applies to requests for {@link Downloadable} 
 * 	content.
 * 
 * @return	<code>true</code> to inform Qwicap that requests intended for the web application should be passed along to
 * 			it, or <code>false</code> if they should be discarded.
 */
	public boolean qwicapCanInvokeWebApplication();

/**	Returns <code>true</code> to inform Qwicap to halt and discard the current session.
 * 
 * @return	<code>true</code> to inform Qwicap to halt and discard the current session, <code>false</code> otherwise.
 */
	public boolean qwicapShouldDiscardThisSession();
}
