/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2009 Chris W. Johnson. All rights reserved.
	
	Chris W. Johnson's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Oct 31, 2009
 * Time: 2:23:43 AM
 * To change this template use File | Settings | File Templates.
 */
final class Bits {
	
	static byte[] fromBytes
	(
		final byte...			Bytes	//	Every byte is value between 0x00 and 0xFF, inclusive.
	)
	{
		final byte[]			Bits     = new byte[Bytes.length << 3];
		int						BitIndex = 0;

		for (final byte Byte : Bytes) {
			
			for (int BitMask = 0x80; BitMask != 0; BitMask >>>= 1)
				Bits[BitIndex++] = (Byte & BitMask) != 0 ? (byte) 1 : (byte) 0;
		}
		
		return Bits;	//	Every byte represents a bit, and therefore has a value of 1 or 0.
	}

	static byte[] toBytes
	(
		final byte...			Bits	//	Every byte represents a bit, and therefore has a value of 1 or 0.
	)
	{
		final int				BitCount   = Bits.length;
		int						BitIndex   = 0;
		final byte[]			Bytes      = new byte[BitCount + 7 >>> 3];
		int						BytesIndex = 0;
		final int				ExtraBits  = BitCount % 8;
		
	//	If the number of bits isn't a multiple of 8, the first byte will be incomplete. Handle that byte now.
		
		if (ExtraBits != 0) {
			int						Byte = 0;
			
		//	The missing bits will be the highest-order bits, and will be set to zero.
			
			while (BitIndex < ExtraBits)
				Byte = Byte << 1 | Bits[BitIndex++];
				
			Bytes[BytesIndex++] = (byte) Byte;
		}
		
	//	Handle all of the remaining bits (the number of remaining bits is now a multiple of 8).
		
		while (BytesIndex < Bytes.length) {
				
			Bytes[BytesIndex++] = (byte) (
				Bits[BitIndex++] << 7 |
				Bits[BitIndex++] << 6 |
				Bits[BitIndex++] << 5 |
				Bits[BitIndex++] << 4 |
				Bits[BitIndex++] << 3 |
				Bits[BitIndex++] << 2 |
				Bits[BitIndex++] << 1 |
				Bits[BitIndex++]
			);
		}
		
		return Bytes;	//	 Every byte represents a value between 0x00 and 0xFF, inclusive.	
	}
	
	static String toBinaryString
	(
		final byte...			Bits	//	Every byte represents a bit, and therefore has a value of 1 or 0.
	) 
	{
		final char[]			Chars = new char[Bits.length];
		
		for (int Index = 0; Index < Bits.length; Index++)
			Chars[Index] = (char) ('0' + Bits[Index]);
			
		return new String(Chars);
	}
	
	static byte[] fromBinaryString
	(
		final String			BinStr
	)
	{
		final char[]			Chars = BinStr.toCharArray();
		final byte[]			Bits  = new byte[Chars.length];
		
		for (int Index = 0; Index < Chars.length; Index++)
			Bits[Index] = (byte) (Chars[Index] - '0');
		
		return Bits;	//	Every byte represents a bit, and therefore has a value of 1 or 0.
	}
}
