/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2007 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


/**	Describes a Qwicap blocking operation. This base class provides basic identifying information (the type of the 
 * 	operation). Operation-specific information is provided by methods in operation-specific subclasses. A "prompt"
 * 	operation ({@link #getOperation} returns {@link BlockingOperation#PROMPT}) is described by the subclass 
 * 	{@link PromptDescription}. A "redirect" operation ({@link #getOperation} returns {@link BlockingOperation#REDIRECT}) 
 * 	is described by the subclass {@link RedirectDescription}.
 * 
 * 	@author Chris W. Johnson
 */
public abstract class BlockingDescription {
		
	private final Qwicap				Q;
	private final BlockingOperation		Op;
	
	
	BlockingDescription
	(
		final Qwicap			Q,
		final BlockingOperation	Op
	)
	{
		this.Q  = Q;
		this.Op = Op;
	}

/**	Returns a reference to the instance of <code>Qwicap</code> that is blocking or unblocking.
 * 
 * @return							A reference to the instance of <code>Qwicap</code> that is blocking or unblocking.
 */
	public Qwicap getCurrentInstance() {
		return Q;
	}

/**	Returns an {@link BlockingOperation} object that identifies the operation which caused blocking to occur.
 * 
 * @return							The kind of operation which caused blocking to occur.
 */
	public BlockingOperation getOperation() {
		return Op;
	}
}
