/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import edu.utexas.ots.chrisj.util.Hexadecimal;

import java.io.UnsupportedEncodingException;
import java.nio.ByteBuffer;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CharacterCodingException;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Jul 6, 2007
 * Time: 1:39:56 AM
 * To change this template use File | Settings | File Templates.
 */
final class ByteArray {

	private final byte[]		Array;
	private final int			Offset;
	private final int			Length;

	private boolean				HashCodeComputed;
	private int					HashCode;

	private ByteBuffer			BBuff;

	private String				CachedStringCharSet;
	private String				CachedString;


	ByteArray
	(
		final byte[]			Array,
		final int				Offset,
		final int				Length
	)
	{
		this.Array  = Array;
		this.Offset = Offset;
		this.Length = Length;
	}

	ByteArray
	(
		final byte[]			Array
	)
	{
		this(Array, 0, Array.length);
	}

	int size() {
		return Length;
	}

	int getOffset() {
		return Offset;
	}

	byte[] getArray() {
		return Array;
	}

	ByteBuffer getByteBuffer() {

		if (BBuff == null) {
			
			BBuff = ByteBuffer.wrap(Array, Offset, Length);
			BBuff.mark();
			
		} else
			BBuff.reset();
		
		return BBuff;
	}

	String toString
	(
		final CharsetDecoder	Decoder
	)
	throws CharacterCodingException
	{
		final String			NewStringCharSetName = Decoder.charset().name();

		if (CachedString != null && CachedStringCharSet.equalsIgnoreCase(NewStringCharSetName))
			return CachedString;
		
		CachedString = Decoder.decode(getByteBuffer()).toString();
		CachedStringCharSet = NewStringCharSetName;
		
		return CachedString;
	}

	String toString
	(
		final String			CharSetName
	)
	throws UnsupportedEncodingException
	{
		if (CachedString != null && CachedStringCharSet.equalsIgnoreCase(CharSetName))
			return CachedString;
		
		CachedString = new String(Array, Offset, Length, CharSetName);
		CachedStringCharSet = CharSetName;

		return CachedString;
	}

	@Override public String toString() {
		return Hexadecimal.fromBytes(Array, Offset, Length);
	}

	@Override public int hashCode() {

		if (!HashCodeComputed) {
			int						HashCode = 0;

			for (int Index = Offset, Count = Length >= 4 ? 4 : Length; Index < Count; Index++)
				HashCode = HashCode << 8 | Array[Index] & 0xFF;

			this.HashCode = HashCode;
			this.HashCodeComputed = true;
		}

		return HashCode;
	}

	@Override public boolean equals
	(
		final Object			Other
	)
	{
		if (Other instanceof ByteArray == false)
			return false;

		final ByteArray			O  = (ByteArray) Other;
		final byte[]			OA = O.Array;
		final byte[]			A  = Array;

		if (Length != O.Length)
			return false;

		for (int AIndex = Offset, OAIndex = O.Offset, ByteCount = Length; --ByteCount >= 0; ) {

			if (A[AIndex++] != OA[OAIndex++])
				return false;
		}

		return true;
	}

	boolean startsWith
	(
		final ByteArray			RefBytes
	)
	{
		return startsWith(RefBytes.getArray());
	}

	boolean startsWith
	(
		final byte[]			RefBytes
	)
	{
		int						RefBytesCount = RefBytes.length;
		final byte[]			Bytes = Array;

		if (RefBytesCount > Length)
			return false;

		for (int Index = Offset, RefIndex = 0; --RefBytesCount >= 0; ) {

			if (RefBytes[RefIndex++] != Bytes[Index++])
				return false;
		}

		return true;
	}

/*	int indexOf
	(
		final ByteArray			RefBytes
	)
	{
		return indexOf(RefBytes.getArray(), 0);
	}
	
	int indexOf
	(
		final ByteArray			RefBytes,
		final int				ByteIndex
	)
	{
		return indexOf(RefBytes.getArray(), Offset + ByteIndex);
	}	
	
	int indexOf
	(
		final byte[]			RefBytes
	)
	{
		return indexOf(RefBytes, 0);
	}

	int indexOf
	(
		final byte[]			RefBytes,
		int						ByteIndex
	)
	{
		final byte[]			Bytes         = Array;
		final int				RefBytesCount = RefBytes.length;
		final byte				RefByteZero   = RefBytes[0];
		final int				BytesCount    = Bytes.length;
		final int				BytesLimit    = BytesCount - RefBytesCount;

		while (ByteIndex < BytesLimit) {

			if (Bytes[ByteIndex] == RefByteZero) {
				final int				OrigByteIndex = ByteIndex++;
				int						RefByteIndex  = 1;

				while (RefByteIndex < RefBytesCount && Bytes[ByteIndex] == RefBytes[RefByteIndex]) {

					ByteIndex++;
					RefByteIndex++;
				}

				if (RefByteIndex == RefBytesCount)
					return OrigByteIndex;

				ByteIndex = OrigByteIndex + 1;

			} else
				ByteIndex++;
		}

		return -1;
	}
*/
}
