/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import java.io.UnsupportedEncodingException;
import java.util.Set;
import java.util.TreeSet;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Jul 5, 2007
 * Time: 11:13:44 PM
 * To change this template use File | Settings | File Templates.
 */
class CharacterSetFamily {

	private static final Logger	Log = Logger.getLogger(CharacterSetFamily.class.getName());

	private final String		SeriesName;
	private final String		FirstCharSetName;
	private final ByteArray		ReferenceBytes;
	private final Set<String> 	CharSetNames = new TreeSet<String>();


	CharacterSetFamily
	(
		final String			FirstCharSetName,
		final String			SeriesName
	)
	throws UnsupportedEncodingException
	{
		this.SeriesName       = SeriesName;
		this.FirstCharSetName = FirstCharSetName;
		this.ReferenceBytes   = new ByteArray(CharacterSetRecognizer.kReferenceString.getBytes(FirstCharSetName));
		add(FirstCharSetName);
	}

	ByteArray getReferenceBytes() {
		return ReferenceBytes;
	}

	void add
	(
		final String			CharSetName
	)
	{
		CharSetNames.add(CharSetName.toUpperCase());
	}

	boolean includes
	(
		final String			CharSetName
	)
	{
		return CharSetNames.contains(CharSetName.toUpperCase());
	}

	String getDefault() {
		return FirstCharSetName;
	}

	String getName() {
		return SeriesName;
	}

	boolean includesCharSet
	(
		final String			CharSet
	)
	{
		return CharSetNames.contains(CharSet.toUpperCase());
	}
	
	String identifyCharSet
	(
		final FormControlBytes[]	FCBs
	)
	throws UnsupportedEncodingException
	{
		FormControlBytes		CharSetFCB = null;
		
		for (final FormControlBytes FCB : FCBs) {
			
			if (FCB.nameMatches(ReferenceBytes)) {
				
				CharSetFCB = FCB;
				break;
			}
		}
		
		if (CharSetFCB == null)
			return null;
		
		String					CharSetName = CharSetFCB.Value.toString(getDefault());
		
		if (includes(CharSetName) == false) {
			final String			BestGuessCharSet = getDefault();

			Log.log(Level.WARNING, "The data set claims to be encoded in the \"{0}\" character set, but is actually encoded in {1}. It will be decoded as \"{2}\".", new Object[] { CharSetName, getName(), BestGuessCharSet });
			CharSetName = BestGuessCharSet;
		}

		return CharSetName;
	}
	
	@Override public String toString() {
		final StringBuilder		Buff = new StringBuilder(256);

		Buff.append("Each character set listed below is ").append(getName()).append(". The default is ").append(getDefault()).append(". (Ref. Bytes: ").append(ReferenceBytes).append(")\n");

		for (final String CharSetName : CharSetNames)
			Buff.append('\t').append(CharSetName).append('\n');

		return Buff.toString();
	}
}
