/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import java.io.UnsupportedEncodingException;
import java.util.logging.Logger;
import java.util.logging.Level;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Jul 5, 2007
 * Time: 11:15:46 PM
 * To change this template use File | Settings | File Templates.
 */
final class CharacterSetFamilyASCII extends CharacterSetFamily {

	private static final Logger	Log = Logger.getLogger(CharacterSetFamilyASCII.class.getName());


	CharacterSetFamilyASCII
	(
		final String			SeriesName,
		final String			FirstCharSetName
	)
	throws UnsupportedEncodingException
	{
		super(SeriesName, FirstCharSetName);
	}

	@Override String identifyCharSet
	(
		final FormControlBytes[]	FCBs
	)
	throws UnsupportedEncodingException
	{
		final ByteArray			ReferenceBytes = getReferenceBytes();
		FormControlBytes		CharSetFCB     = null;

		for (final FormControlBytes FCB : FCBs) {

			if (FCB.nameMatches(ReferenceBytes)) {

				CharSetFCB = FCB;
				break;
			}
		}

		if (CharSetFCB == null)
			return null;

		String					CharSetName = CharSetFCB.Value.toString(getDefault());

		if (includes(CharSetName)) {
			
			if (UTF8Analysis.getUnified(CharSetName, FCBs).ProbablyIsUTF8)
				CharSetName = "UTF-8";

		} else {
			final String			ImpossibleCharSetName = CharSetName;
			
			if (UTF8Analysis.getUnified(null, FCBs).ProbablyIsUTF8)
				CharSetName = "UTF-8";
			else
				CharSetName = getDefault();
			
			Log.log(Level.WARNING, "The data set claims to be encoded in the \"{0}\" character set, but is actually encoded in {1}. It will be decoded as \"{2}\".", new Object[] { ImpossibleCharSetName, getName(), CharSetName });
		}

		return CharSetName;
	}
}
