/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2011 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import java.io.UnsupportedEncodingException;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Jul 6, 2007
 * Time: 4:41:25 PM
 * To change this template use File | Settings | File Templates.
 */
abstract class CharacterSetFamilyUTF16 extends CharacterSetFamilyUTFMultiByte {

	protected static final int		BadStringLengthMask = 0x01;
	protected static final byte[]	BE_BOM = { (byte) 0xFE, (byte) 0xFF };
	protected static final byte[]	LE_BOM = { (byte) 0xFF, (byte) 0xFE };


	protected CharacterSetFamilyUTF16
	(
		final String			SeriesName,
		final String			FirstCharSetName
	)
	throws UnsupportedEncodingException
	{
		super(SeriesName, FirstCharSetName);
		add("UTF-16");
	}
}


/*	From RFC 2781, at <ftp://ftp.rfc-editor.org/in-notes/rfc2781.txt>:

	4.1 Interpreting text labelled as UTF-16BE
	
	   Text labelled "UTF-16BE" can always be interpreted as being big-
	   endian.  The detection of an initial BOM does not affect de-
	   serialization of text labelled as UTF-16BE. Finding 0xFF followed by
	   0xFE is an error since there is no Unicode character 0xFFFE.
	
	4.2 Interpreting text labelled as UTF-16LE
	
	   Text labelled "UTF-16LE" can always be interpreted as being little-
	   endian. The detection of an initial BOM does not affect de-
	   serialization of text labelled as UTF-16LE. Finding 0xFE followed by
	   0xFF is an error since there is no Unicode character 0xFFFE, which
	   would be the interpretation of those octets under little-endian
	   order.
	
	4.3 Interpreting text labelled as UTF-16
	
	   Text labelled with the "UTF-16" charset might be serialized in either
	   big-endian or little-endian order. If the first two octets of the
	   text is 0xFE followed by 0xFF, then the text can be interpreted as
	   being big-endian. If the first two octets of the text is 0xFF
	   followed by 0xFE, then the text can be interpreted as being little-
	   endian. If the first two octets of the text is not 0xFE followed by
	   0xFF, and is not 0xFF followed by 0xFE, then the text SHOULD be
	   interpreted as being big-endian.
	
	   All applications that process text with the "UTF-16" charset label
	   MUST be able to read at least the first two octets of the text and be
	   able to process those octets in order to determine the serialization
	   order of the text. Applications that process text with the "UTF-16"
	   charset label MUST NOT assume the serialization without first
	   checking the first two octets to see if they are a big-endian BOM, a
	   little-endian BOM, or not a BOM. All applications that process text
	   with the "UTF-16" charset label MUST be able to interpret both big-
	   endian and little-endian text.
*/
