/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2009 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import java.io.UnsupportedEncodingException;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Oct 22, 2009
 * Time: 5:48:51 PM
 * To change this template use File | Settings | File Templates.
 */
class CharacterSetFamilyUTFMultiByte extends CharacterSetFamily {
	
	private static final Logger		Log = Logger.getLogger(CharacterSetFamilyUTFMultiByte.class.getName());
	
	
	CharacterSetFamilyUTFMultiByte
	(
		final String			FirstCharSetName,
		final String			SeriesName
	)
	throws UnsupportedEncodingException
	{
		super(FirstCharSetName, SeriesName);
	}	
	
	protected String identifyCharSet
	(
		final FormControlBytes[]	FCBs,
		final byte[]				GoodBOM,
		final byte[]				BadBOM,
		final int					BadStringLengthMask
	)
	throws UnsupportedEncodingException
	{
		if (FCBs.length == 0)
			return null;
		
		final FormControlBytes	FCB0 = FCBs[0];
		final boolean			FCB0Replaced;
		
		{
			final ByteArray			FCB0Name = FCB0.Name;
			
		//	It just might be valid to begin a UTF-32 data set with a byte-order-mark (BOM). If so, it would be 
		//	interpreted as a part of the first control's name.
			
			if (FCB0Name.startsWith(BadBOM))
				return null;	//	This definitely is *not* whatever character set the "good" BOM represents, because it starts with the "bad" BOM.
				
		//	Replace the first element in FCBs with a new element that excludes the BOM from the control's name.
			
			if (FCB0Name.startsWith(GoodBOM)) {
				
				FCBs[0]      = new FormControlBytes(new ByteArray(FCB0Name.getArray(), FCB0Name.getOffset() + GoodBOM.length, FCB0Name.size() - GoodBOM.length), FCB0.Value);
				FCB0Replaced = true;
			
			} else
				FCB0Replaced = false;	
		}
		
	//	Check all of the form control names and values to make sure that they could be UTF-32.
		
		final ByteArray			ReferenceBytes = getReferenceBytes();
		FormControlBytes		CharSetFCB     = null;
		
		for (final FormControlBytes FCB : FCBs) {
			final ByteArray			Name = FCB.Name;
						
		//	In UTF-32 all strings must be multiples of 4 bytes in length, so for UTF-32 the BadStringLengthMask is 0x03.
		//	In UTF-16 all strings must be multiples of either 2 or 4 bytes in length, so for UTF-16 the BadStringLengthMask is 0x01.
			
			if ((Name.size() & BadStringLengthMask) != 0 || (FCB.Value.size() & BadStringLengthMask) != 0) {
				
				if (FCB0Replaced)
					FCBs[0] = FCB0;	//	Restore the original object in FCBs[0], since that wasn't a BOM we removed from it, after all.
				
				return null;
			}
			
			if (Name.equals(ReferenceBytes)) {
				
				CharSetFCB = FCB;
				break;
			}
		}

		if (CharSetFCB == null) {
			
			if (FCB0Replaced)
				FCBs[0] = FCB0;	//	Restore the original object in FCBs[0], since that wasn't a BOM we removed from it, after all.
			
			return null;
		}
		
		String					CharSetName = CharSetFCB.Value.toString(getDefault());

		if (includes(CharSetName) == false) {
			final String			BadCharSetName = CharSetName;
			
			CharSetName = getDefault();
			
			Log.log(Level.WARNING, "The data set claims to be encoded in the \"{0}\" character set, but is actually encoded in {1}. It will be decoded as \"{2}\".", new Object[] { BadCharSetName, getName(), CharSetName });
		}
		
		return CharSetName;
	}
}