/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2010 Chris W. Johnson. All rights reserved.
	
	Chris W. Johnson's copyright notice in this evolving version reflects original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import junit.framework.TestCase;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Feb 7, 2010
 * Time: 10:37:39 PM
 * To change this template use File | Settings | File Templates.
 */
public class ContentTypeJUnit extends TestCase {
	
	public void test1() {
		final ContentType CT = new ContentType("text/plain", new DefaultCharSet("us-ascii"));
		
		assertTrue("text".equals(CT.getType()));
		assertTrue("plain".equals(CT.getSubtype()));
		assertTrue("US-ASCII".equals(CT.getCharacterSet()));
	}
	
	public void test1_1() {
		final ContentType CT = new ContentType("text/plain (Plain Text)", new DefaultCharSet("us-ascii"));
		
		assertTrue("text".equals(CT.getType()));
		assertTrue("plain".equals(CT.getSubtype()));
		assertTrue("US-ASCII".equals(CT.getCharacterSet()));
		
	//	Note: The API does not currently provide access to comments in this position occupied by the comment above;
	//	      it only provides for access to comments associated with parameters, of which the content type above has
	//	      none.
	}
	
	public void test2() {
		final ContentType CT = new ContentType("text/plain; charset=utf-8", null);
		
		assertTrue("text".equals(CT.getType()));
		assertTrue("plain".equals(CT.getSubtype()));
		assertTrue("UTF-8".equals(CT.getCharacterSet()));
		
		final ContentTypeParameter[]	CPs = CT.getParameters();
		
		assertTrue(CPs.length == 1);
		assertTrue("charset".equals(CPs[0].Name));
		assertTrue("utf-8".equals(CPs[0].Value));
		assertTrue(null == CPs[0].Comment);
	}
	
	public void test2_1() {
		
	//	Note: A space is normally placed after the semicolon (';'), but the relevant standards do not appear to require
	//	      that space, so we should cope with its absence.
		
		final ContentType CT = new ContentType("text/plain;charset=utf-8", null);
		
		assertTrue("text".equals(CT.getType()));
		assertTrue("plain".equals(CT.getSubtype()));
		assertTrue("UTF-8".equals(CT.getCharacterSet()));
		
		final ContentTypeParameter[]	CPs = CT.getParameters();
		
		assertTrue(CPs.length == 1);
		assertTrue("charset".equals(CPs[0].Name));
		assertTrue("utf-8".equals(CPs[0].Value));
		assertTrue(null == CPs[0].Comment);
	}
	
	public void test2_2() {
		
	//	Note: Make sure we can cope with a semicolon (';') surrounded by whitespace, even though that doesn't seem to 
	//	      be something encouraged or permitted(?) by the relevant standards.
		
		final ContentType CT = new ContentType("text/plain ; charset=utf-8", null);
		
		assertTrue("text".equals(CT.getType()));
		assertTrue("plain".equals(CT.getSubtype()));
		assertTrue("UTF-8".equals(CT.getCharacterSet()));
		
		final ContentTypeParameter[]	CPs = CT.getParameters();
		
		assertTrue(CPs.length == 1);
		assertTrue("charset".equals(CPs[0].Name));
		assertTrue("utf-8".equals(CPs[0].Value));
		assertTrue(null == CPs[0].Comment);
	}
	
	public void test2_3() {
		
	//	Note: Spaces between a parameter name and the trailing equal sign ('='), or between the equal sign and the 
	//	      value appears to be illegal according to the relevant standards, but we should cope with it, just to be
	//	      safe.
		
		final ContentType CT = new ContentType("text/plain; charset = utf-8", null);
		
		assertTrue("text".equals(CT.getType()));
		assertTrue("plain".equals(CT.getSubtype()));
		assertTrue("UTF-8".equals(CT.getCharacterSet()));
		
		final ContentTypeParameter[]	CPs = CT.getParameters();
		
		assertTrue(CPs.length == 1);
		assertTrue("charset".equals(CPs[0].Name));
		assertTrue("utf-8".equals(CPs[0].Value));
		assertTrue(null == CPs[0].Comment);
	}
	
	public void test2_4() {
		
	//	Note: Spaces between a parameter name and the trailing equal sign ('='), or between the equal sign and the 
	//	      value appears to be illegal according to the relevant standards, but we should cope with it, just to be
	//	      safe.
		
	//	Note: Trailing whitespace should not be present, but we should cope with it, just to be safe.
		
		final ContentType CT = new ContentType("text/plain; charset = utf-8 ", null);
		
		assertTrue("text".equals(CT.getType()));
		assertTrue("plain".equals(CT.getSubtype()));
		assertTrue("UTF-8".equals(CT.getCharacterSet()));
		
		final ContentTypeParameter[]	CPs = CT.getParameters();
		
		assertTrue(CPs.length == 1);
		assertTrue("charset".equals(CPs[0].Name));
		assertTrue("utf-8".equals(CPs[0].Value));
		assertTrue(null == CPs[0].Comment);
	}
	
	public void test2_5() {
		
	//	Note: Spaces between a parameter name and the trailing equal sign ('='), or between the equal sign and the 
	//	      value appears to be illegal according to the relevant standards, but we should cope with it, just to be
	//	      safe.
		
		final ContentType CT = new ContentType("text/plain; charset = \"utf-8\"", null);
		
		assertTrue("text".equals(CT.getType()));
		assertTrue("plain".equals(CT.getSubtype()));
		assertTrue("UTF-8".equals(CT.getCharacterSet()));
		
		final ContentTypeParameter[]	CPs = CT.getParameters();
		
		assertTrue(CPs.length == 1);
		assertTrue("charset".equals(CPs[0].Name));
		assertTrue("utf-8".equals(CPs[0].Value));
		assertTrue(null == CPs[0].Comment);
	}
	
	
	public void test2_6() {
		
	//	Note: Spaces between a parameter name and the trailing equal sign ('='), or between the equal sign and the 
	//	      value appears to be illegal according to the relevant standards, but we should cope with it, just to be
	//	      safe.
		
	//	Note: Trailing whitespace should not be present, but we should cope with it, just to be safe.
		
		final ContentType CT = new ContentType("text/plain; charset = \"utf-8\" ", null);
		
		assertTrue("text".equals(CT.getType()));
		assertTrue("plain".equals(CT.getSubtype()));
		assertTrue("UTF-8".equals(CT.getCharacterSet()));
		
		final ContentTypeParameter[]	CPs = CT.getParameters();
		
		assertTrue(CPs.length == 1);
		assertTrue("charset".equals(CPs[0].Name));
		assertTrue("utf-8".equals(CPs[0].Value));
		assertTrue(null == CPs[0].Comment);
	}
	
	public void test3_1() {
	
	//	Test an example from RFC 2045, <http://www.ietf.org/rfc/rfc2045.txt>, section 5.1, "Syntax of the Content-Type 
	//	Header Field".

		final ContentType CT = new ContentType("text/plain; charset=us-ascii (Plain text)", null);
		
		assertTrue("text".equals(CT.getType()));
		assertTrue("plain".equals(CT.getSubtype()));
		assertTrue("US-ASCII".equals(CT.getCharacterSet()));
		
		final ContentTypeParameter[]	CPs = CT.getParameters();
		
		assertTrue(CPs.length == 1);
		assertTrue("charset".equals(CPs[0].Name));
		assertTrue("us-ascii".equals(CPs[0].Value));
		assertTrue("Plain text".equals(CPs[0].Comment));
	}
	
	public void test3_2() {
		
	//	Test an example from RFC 2045, <http://www.ietf.org/rfc/rfc2045.txt>, section 5.1, "Syntax of the Content-Type 
	//	Header Field".
		
		final ContentType CT = new ContentType("text/plain; charset=\"us-ascii\"", null);
		
		assertTrue("text".equals(CT.getType()));
		assertTrue("plain".equals(CT.getSubtype()));
		assertTrue("US-ASCII".equals(CT.getCharacterSet()));
		
		final ContentTypeParameter[]	CPs = CT.getParameters();
		
		assertTrue(CPs.length == 1);
		assertTrue("charset".equals(CPs[0].Name));
		assertTrue("us-ascii".equals(CPs[0].Value));
		assertTrue(null == CPs[0].Comment);
	}
	
	public void test5() {
		final ContentType CT = new ContentType("text/plain; charset=\"us-ascii\" (Plain text)", null);
		
		assertTrue("text".equals(CT.getType()));
		assertTrue("plain".equals(CT.getSubtype()));
		assertTrue("US-ASCII".equals(CT.getCharacterSet()));
		
		final ContentTypeParameter[]	CPs = CT.getParameters();
		
		assertTrue(CPs.length == 1);
		assertTrue("charset".equals(CPs[0].Name));
		assertTrue("us-ascii".equals(CPs[0].Value));
		assertTrue("Plain text".equals(CPs[0].Comment));
	}
	
	public void test5_1() {
	
	//	Note: Trailing whitespace should not be present, but we should cope with it, just to be safe.

		final ContentType CT = new ContentType("text/plain; charset=\"us-ascii\" (Plain text)   ", null);
		
		assertTrue("text".equals(CT.getType()));
		assertTrue("plain".equals(CT.getSubtype()));
		assertTrue("US-ASCII".equals(CT.getCharacterSet()));
		
		final ContentTypeParameter[]	CPs = CT.getParameters();
		
		assertTrue(CPs.length == 1);
		assertTrue("charset".equals(CPs[0].Name));
		assertTrue("us-ascii".equals(CPs[0].Value));
		assertTrue("Plain text".equals(CPs[0].Comment));
	}
	
	public void test6() {
		final ContentType CT = new ContentType("text/plain; thing=\"Test \\\"quoted\\\" string\"; charset=us-ascii (Plain text)", null);
		
		assertTrue("text".equals(CT.getType()));
		assertTrue("plain".equals(CT.getSubtype()));
		assertTrue("US-ASCII".equals(CT.getCharacterSet()));
		
		final ContentTypeParameter[]	CPs = CT.getParameters();
		
		assertTrue(CPs.length == 2);
		
		assertTrue("thing".equals(CPs[0].Name));
		assertTrue("Test \"quoted\" string".equals(CPs[0].Value));
		assertTrue(null == CPs[0].Comment);
		
		assertTrue("charset".equals(CPs[1].Name));
		assertTrue("us-ascii".equals(CPs[1].Value));
		assertTrue("Plain text".equals(CPs[1].Comment));
	}
	
	public void test7() {
		
	//	Note: RFC 2045 does not allow for the MIME type to be missing, as it is here. Neither does HTTP 1.1 (RFC 2616), 
	//	      which uses the the RFC 2045 content type format for its "Content-Type" header, but arbitrarily stupid 
	//	      things happen in the HTTP world, so maybe we should be able to cope with the absence of a MIME type. The 
	//	      previous Qwicap class that parsed content types, the HTTPContentType class, could cope with a missing
	//	      MIME type. So, the question becomes: Did it cope with missing MIME types because there was some real-world
	//	      case in which that was necessary, or did it cope just because that seemed like a good idea at the time?
		
		final ContentType CT = new ContentType("charset=us-ascii", null);
		
		assertTrue(CT.getType()    == null);
		assertTrue(CT.getSubtype() == null);
		assertTrue("US-ASCII".equals(CT.getCharacterSet()));
	}
	
	public void test8_1() {
	
	//	Test some "Content-type" header values from RFC 2046, <http://www.ietf.org/rfc/rfc2046.txt>, section 5.1.1.
		
		final ContentType CT = new ContentType("multipart/mixed; boundary=gc0p4Jq0M2Yt08j34c0p", null);
		
		assertTrue("multipart".equals(CT.getType()));
		assertTrue("mixed".equals(CT.getSubtype()));
		assertTrue(null == CT.getCharacterSet());
		
		final ContentTypeParameter[]	CPs = CT.getParameters();
		
		assertTrue(CPs.length == 1);
		
		assertTrue("boundary".equals(CPs[0].Name));
		assertTrue("gc0p4Jq0M2Yt08j34c0p".equals(CPs[0].Value));
		assertTrue(null == CPs[0].Comment);
	}
	
	public void test8_2() {
	
	//	Test some "Content-type" header values from RFC 2046, <http://www.ietf.org/rfc/rfc2046.txt>, section 5.1.1.
		
		final ContentType CT = new ContentType("multipart/mixed; boundary=\"gc0pJq0M:08jU534c0p\"", null);
		
		assertTrue("multipart".equals(CT.getType()));
		assertTrue("mixed".equals(CT.getSubtype()));
		assertTrue(null == CT.getCharacterSet());
		
		final ContentTypeParameter[]	CPs = CT.getParameters();
		
		assertTrue(CPs.length == 1);
		
		assertTrue("boundary".equals(CPs[0].Name));
		assertTrue("gc0pJq0M:08jU534c0p".equals(CPs[0].Value));
		assertTrue(null == CPs[0].Comment);
	}
	
	private static class DefaultCharSet implements ContentTypeDefaultCharSet {
		
		private final String	CharSet;
		
		DefaultCharSet
		(
			final String		CharSet
		)
		{
			this.CharSet = CharSet;
		}
		
		@Override public String get(final String Type, final String Subtype) {
			return CharSet;
		}
	}
}
