/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2007-2008 Chris W. Johnson. All rights reserved.
	
	Chris W. Johnson's copyright notice in this evolving version reflects his original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import javax.servlet.ServletContext;
import java.util.Set;
import java.util.HashSet;
import java.util.logging.Logger;
import java.util.logging.Level;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Oct 22, 2007
 * Time: 10:58:32 PM
 * To change this template use File | Settings | File Templates.
 */
final class DocumentCensus {
	
	private static final Logger Log = Logger.getLogger(DocumentCensus.class.getName());
	
/**	Number of documents in this web application. */
	
	private final int			DocumentCount;
	
/**	Number of unique document names in this web application. */
	
	private final int			NameCount;

	
	DocumentCensus
	(
		final int			DocumentCount,
		final int			NameCount
	)
	{
		this.DocumentCount = DocumentCount;
		this.NameCount     = NameCount;
	}
	
	DocumentCensus
	(
		final String			ContextPath,
		final ServletContext	SCtxt	
	)
	{
	//	todo Note that we're not currently doing anything to allow for resources like "qwicap.css" that will be loaded.
		
		final HashSet<String>	Names = new HashSet<String>(300);
		
		DocumentCount = performCensus(SCtxt, Names, "/");
		NameCount     = Names.size();
		
		Log.log(Level.FINE, "Document census of application \"{0}\" performed. There are {1,number,integer} documents, and {2,number,integer} unique document names.", new Object[] { ContextPath, DocumentCount, NameCount });
	}
	
	private static int performCensus
	(
		final ServletContext	SCtxt,
		final HashSet<String>	Names,
		final String			DirName
	)
	{
		if ("/WEB-INF/".equals(DirName) || "/META-INF/".equals(DirName))
			return 0;
		
		int						DocCount = 0;
		final Set<String>		Items    = SCtxt.getResourcePaths(DirName);
				
		if (Items != null) {
			
			for (final String Item : Items) {
				
				if (Item.endsWith("/")) {
					
					DocCount += performCensus(SCtxt, Names, Item);
					
				} else {
					
					Names.add(Item.substring(Item.lastIndexOf('/')));
					DocCount++;
				}
			}
		}
		
		return DocCount;
	}
	
	int getDocumentCount() {
		return DocumentCount;	
	}
	
	int getDocumentNameCount() {
		return NameCount;	
	}
}
