/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2007 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Apr 29, 2006
 * Time: 5:44:58 PM
 * To change this template use File | Settings | File Templates.
 */
abstract class FilterAction {

	private static final Logger	Log = Logger.getLogger(FilterAction.class.getName());

	FilterAction				Next;


	abstract boolean execute() throws Exception;


	void executeAll() throws QwicapAbandonmentException {

		try {

		//	Execute the actions in our Actions list, removing each action as it is executed.
		//	This loop may be terminated by actions throwing exceptions like QwicapAbandonmentException. In such cases,
		//	it is the responsibility of that exception to continue execution of the actions at an appropriate time.

			FilterAction			Action = this;

			while (Action != null) {

			//	Execute the current action, ceasing execution if the action returns false.

				if (Action.execute() == false)
					break;		//	Execute no further actions.

				Action = Action.Next;
			}

		} catch (QwicapAbandonmentException e) {
			throw e;
		} catch (QwicapSessionDeathException e) {
			throw e;
		} catch (Exception e) {
			Log.log(Level.SEVERE, "An exception other than QwicapAbandonmentException, or QwicapSessionDeathException, was produced while executing a sequence of filter actions created by the filter \"" + Qwicap.getCurrentInstance().getInputFilter().getClass().getName() + "\".", e);
		}
	}
}
