/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2007 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import junit.framework.TestCase;
import edu.utexas.its.eis.tools.qwicap.template.xml.immutable.XMLString;
import edu.utexas.its.eis.tools.qwicap.template.xml.Match;
import edu.utexas.its.eis.tools.qwicap.template.xml.TagException;
import edu.utexas.its.eis.tools.qwicap.template.xml.mutable.MutableMarkup;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Apr 10, 2007
 * Time: 5:26:08 PM
 * To change this template use File | Settings | File Templates.
 */
public class FormControlInfoJUnit extends TestCase {
	
	public static final XMLString TestDoc;
	
	static {
		try {
			TestDoc = new XMLString(
				"<body>\n" +
				"\t<div class='content'>" +
				"\t\t<form class='upload' method='post' action='' enctype='multipart/form-data'>\n" +
				"\t\t\t<table>\n" +
				"\t\t\t\t<tr>\n" +
				"\t\t\t\t\t<td>\n" +
				"\t\t\t\t\t\t<label for='upload-button'>\n" +
				"\t\t\t\t\t\t\t<span class='text-bold'>Certificate Signed Request (CSR):</span>\n" +
				"\t\t\t\t\t\t</label>\n" +
				"\t\t\t\t\t</td>\n" +
				"\t\t\t\t\t<td>\n" +
				"\t\t\t\t\t\t<input id='upload-button' type='file' name='csr'/>\n" +
				"\t\t\t\t\t</td>\n" +
				"\t\t\t\t\t<td>\n" +
				"\t\t\t\t\t\t<button class='standard-big' name='submit' value=\"proceed\" type='submit'>Upload CSR File</button>\n" +
				"\t\t\t\t\t</td>\n" +
				"\t\t\t\t</tr>\n" +
				"\t\t\t</table>\n" +
				"\t\t</form>\n" +
				"\t</div>\n" + 
				"</body>"
			);
		} catch (TagException e) {
			throw new RuntimeException(e);
		}
	}

	
	public void testGetEncosingElement() throws QwicapException {

		final MutableMarkup Doc = TestDoc.getMutable();

		final Form				F = Doc.get("form").toForm();
		final FormControlInfo	CInfo = new FormControlInfo(F, "csr", "blah, blah, blah", F.getControlsByName("csr").get(0));
		final Match				Elem = CInfo.getEnclosingElement();

		assertTrue("Enclosing element is null, but should not be.", Elem != null);

	//	System.out.println(Elem.getType());
	}
	
	public void testGetLabelForSelectControl1() throws QwicapException {
		final MutableMarkup TestDoc = new XMLString(
			"<form method='post' action=''>\t\t\n" +
			"<select name='who' id='0'>\n" +
			"<option>Fred</option>\n" +
			"<option>Barney</option>\n" +
			"<option>Wilma</option>\n" +
			"<option>Betty</option>\n" +
			"</select>\n" +
			"</form>"
		).getMutable();
		
		final String			TestStr = TestDoc.get("form").toForm().getControl("who", "Fred").toString();
		final String			MasterStr = 
		"+--------------------------------+--------------------------------+-----------------+--------------+------------------+-------------------------+------------+\n" +
		"| Enclosing Element              | Form Element                   | Control Element | Control Name | Control Value(s) | Label Element(s)        | Label Text |\n" +
		"+--------------------------------+--------------------------------+-----------------+--------------+------------------+-------------------------+------------+\n" +
		"| <form method='post' action=''> | <form method='post' action=''> | <option>        | who          | \"Fred\"           | \"<option>Fred</option>\" | Fred       |\n" +
		"+--------------------------------+--------------------------------+-----------------+--------------+------------------+-------------------------+------------+\n";

		System.out.println(TestStr);
		
		assertTrue(TestStr.equals(MasterStr));
	}
	
	public void testGetLabelForSelectControl2() throws QwicapException {
		final MutableMarkup TestDoc = new XMLString(
			"<form method='post' action=''>\t\t\n" +
			"<select name='who' id='0'>\n" +
			"<option value='fred'>Fred</option>\n" +
			"<option value='barney'>Barney</option>\n" +
			"<option value='wilma'>Wilma</option>\n" +
			"<option value='betty'>Betty</option>\n" +
			"</select>\n" +
			"</form>"
		).getMutable();
		
		final String			TestStr = TestDoc.get("form").toForm().getControl("who", "fred").toString();
		final String			MasterStr = 
		"+--------------------------------+--------------------------------+-----------------------+--------------+------------------+--------------------------------------+------------+\n" +
		"| Enclosing Element              | Form Element                   | Control Element       | Control Name | Control Value(s) | Label Element(s)                     | Label Text |\n" +
		"+--------------------------------+--------------------------------+-----------------------+--------------+------------------+--------------------------------------+------------+\n" +
		"| <form method='post' action=''> | <form method='post' action=''> | <option value='fred'> | who          | \"fred\"           | \"<option value='fred'>Fred</option>\" | Fred       |\n" +
		"+--------------------------------+--------------------------------+-----------------------+--------------+------------------+--------------------------------------+------------+\n";

		System.out.println(TestStr);
		
		assertTrue(TestStr.equals(MasterStr));
	}
	
	public void testGetLabelForSelectControl3() throws QwicapException {
		final MutableMarkup TestDoc = new XMLString(
			"<form method='post' action=''>\t\t\n" +
			"<select name='who' id='0'>\n" +
			"<option label='Fred' value='fred'/>\n" +
			"<option label='Barney' value='barney'/>\n" +
			"<option label='Wilma' value='wilma'/>\n" +
			"<option label='Betty' value='betty'/>\n" +
			"</select>\n" +
			"</form>"
		).getMutable();
		
		final String			TestStr = TestDoc.get("form").toForm().getControl("who", "fred").toString();
		final String			MasterStr = 
		"+--------------------------------+--------------------------------+-------------------------------------+--------------+------------------+---------------------------------------+------------+\n" +
		"| Enclosing Element              | Form Element                   | Control Element                     | Control Name | Control Value(s) | Label Element(s)                      | Label Text |\n" +
		"+--------------------------------+--------------------------------+-------------------------------------+--------------+------------------+---------------------------------------+------------+\n" +
		"| <form method='post' action=''> | <form method='post' action=''> | <option label='Fred' value='fred'/> | who          | \"fred\"           | \"<option label='Fred' value='fred'/>\" | Fred       |\n" +
		"+--------------------------------+--------------------------------+-------------------------------------+--------------+------------------+---------------------------------------+------------+\n";

		System.out.println(TestStr);
		
		assertTrue(TestStr.equals(MasterStr));
	}
}
