/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2007-2008 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import junit.framework.TestCase;

import java.io.OutputStream;
import java.io.IOException;
import java.io.ByteArrayOutputStream;
import java.net.URLEncoder;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Jul 9, 2007
 * Time: 11:10:48 PM
 * To change this template use File | Settings | File Templates.
 */
public class FormDataSetDecoderJUnit extends TestCase {

	public void test1() throws Exception {
		final byte[]				InputBytes = "qwicap-charset=UTF-8&qwicap-page-id=0FBBCEF6DBF5E64B&qwicap-form-id=8C11510B25E8CF4A&guess=%E0%B9%94%E0%B9%92".getBytes("US-ASCII");
		final FormDataSetDecoder	FDSD       = new FormDataSetDecoder(InputBytes, null);
		final FormDataSet			FDS        = new FormDataSet();

		FDSD.populateFormDataSet(FDS);

		if ("UTF-8".equals(FDS.getDatum("qwicap-charset")) == false)
			fail();

		if ("0FBBCEF6DBF5E64B".equals(FDS.getDatum("qwicap-page-id")) == false)
			fail();

		if ("8C11510B25E8CF4A".equals(FDS.getDatum("qwicap-form-id")) == false)
			fail();

		if ("๔๒".equals(FDS.getDatum("guess")) == false)
			fail();
	}
	
	public void testUSASCII() throws Exception {
		testCharSet("US-ASCII", "US-ASCII", "guess", "42");
	}
	
	public void testISO88591() throws Exception {
		testCharSet("ISO-8859-1", "ISO-8859-1", "guess", "42");
	}
	
	public void testUTF8() throws Exception {
		testCharSet("UTF-8", "UTF-8", "guess", "๔๒");
	}
	
	public void testUTF16BE() throws Exception {
		testCharSet("UTF-16",   "UTF-16BE", "guess", "๔๒");
		testCharSet("UTF-16BE", "UTF-16BE", "guess", "๔๒");
	}
	
	public void testUTF16LE() throws Exception {
		testCharSet("UTF-16LE", "UTF-16LE", "guess", "๔๒");
	}
	
	public void testEBCDIC() throws Exception {
		testCharSet("IBM1047", "IBM1047", "guess", "42");
	}
	
	public void test_UTF8_Claiming_To_Be_USASCII() throws Exception {
		testCharSet("US-ASCII", "UTF-8", "guess", "๔๒");
	}
	
	public void test_UTF8_Claiming_To_Be_EBCDIC() throws Exception {
		testCharSet("IBM1047", "UTF-8", "guess", "๔๒");
	}
	
	public void test_UTF8_Claiming_To_Be_UTF16() throws Exception {
		testCharSet("UTF-16", "UTF-8", "guess", "๔๒");
	}
	
	public void test_UTF16BE_Claiming_To_Be_EBCDIC() throws Exception {
		testCharSet("IBM1047", "UTF-16BE", "guess", "๔๒");
	}
	
	public void test_UTF16BE_Claiming_To_Be_USASCII() throws Exception {
		testCharSet("US-ASCII", "UTF-16BE", "guess", "๔๒");
	}
	
	public void test_UTF16BE_Claiming_To_Be_UTF8() throws Exception {
		testCharSet("UTF-8", "UTF-16BE", "guess", "๔๒");
	}
	
	private void testCharSet
	(
		final String			StatedCharSet,
		final String			ActualCharSet,
		final String			ExtraControlName,
		final String			ExtraControlValue
	)
	throws Exception 
	{
		final ByteArrayOutputStream	BOut = new ByteArrayOutputStream(4096);

		appendControl(BOut, "qwicap-charset", StatedCharSet,      ActualCharSet);
		append(BOut, "&", "US-ASCII");
		appendControl(BOut, "qwicap-page-id", "0FBBCEF6DBF5E64B", ActualCharSet);
		append(BOut, "&", "US-ASCII");
		appendControl(BOut, "qwicap-form-id", "8C11510B25E8CF4A", ActualCharSet);
		append(BOut, "&", "US-ASCII");
		appendControl(BOut, ExtraControlName, ExtraControlValue,  ActualCharSet);
		
		final byte[]				InputBytes = BOut.toByteArray();
		
		final FormDataSetDecoder	FDSD = new FormDataSetDecoder(InputBytes, null);
		final FormDataSet			FDS  = new FormDataSet();

		FDSD.populateFormDataSet(FDS);
		
		if (StatedCharSet.equals(FDS.getDatum("qwicap-charset")) == false)
			fail("The character set is expected to be \"" + StatedCharSet + "\" but is actually \"" + FDS.getDatum("qwicap-charset") + "\".");
		
		if ("0FBBCEF6DBF5E64B".equals(FDS.getDatum("qwicap-page-id")) == false)
			fail();

		if ("8C11510B25E8CF4A".equals(FDS.getDatum("qwicap-form-id")) == false)
			fail();

		if (ExtraControlValue.equals(FDS.getDatum(ExtraControlName)) == false)
			fail();
	}

	private static void appendControl
	(
		final OutputStream		Out,
		final String			Name,
		final String			Value,
		final String			CharSet
	)
	throws IOException
	{
	
	//	The use of ISO-8859-1 here is of no significance; any character set that had a single-byte encoding for every 
	//	octet (0-255) would do just as well. (Any other sort of character set would either omit characters for which it 
	//	had no encoding, would add characters if it used multi-byte encodings, or both. We just want the characters 
	//	treated like bytes, even though they're returned in Unicode strings. URLEncoder.encode should really return 
	//	bytes, since the whole point of using a character set encoding is to transform characters into bytes. 
	//	Unfortunately, it returns Unicode strings which confuses matters significantly.)
	
		append(Out, URLEncoder.encode(new String(Name.getBytes(CharSet), "ISO-8859-1"), "ISO-8859-1"), "US-ASCII");
		append(Out, "=", "US-ASCII");
		append(Out, URLEncoder.encode(new String(Value.getBytes(CharSet), "ISO-8859-1"), "ISO-8859-1"), "US-ASCII");
	}

	private static void append
	(
		final OutputStream		Out,
		final String			Str,
		final String			CharSet
	)
	throws IOException
	{
		Out.write(Str.getBytes(CharSet));
	}
}
