/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2007 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


/**	Defines the interface for a class that filters the form data set received by Qwicap before it is processed by 
 * 	Qwicap, or passed-along to the client application. The filter may directly modify the data set, and may instruct
 * 	Qwicap to take various actions. You instruct Qwicap to use a <code>FormDataSetFilter</code> object by passing that
 * 	object to {@link Qwicap#setInputFilter(FormDataSetFilter)}. Note that there can be only one filter at a time.
 * 	
 * @author Chris W. Johnson
 */
public interface FormDataSetFilter {

/**	Invoked whenever Qwicap receives a new form data set (or the data sets present in URLs). The <code>filter</code> 
 *	implemenetation may alter the contents of the <code>DataSet</code> parameter, and may instruct Qwicap to perform 
 * 	various actions by invoking methods on the {@link FormDataSetFilterResponse} instance passed as the 
 * 	<code>YourResponse</code> parameter. The implementation may even choose to write a complete reply to the client
 * 	through the <code>HttpServletResponse</code> instance passed as the <code>Resp</code> parameter - in this case,
 * 	the implementation should invoke the <code>ignoreDataSet</code> method on the <code>YourResponse</code> object.
 * 
 * @param DataSet				The data set received through this request. By altering the contests of this object 
 * 								you will change the data set that Qwicap delivers to your client application.
 * @param FilterContext			Information about the context in which the filter was invoked.
 * @param YourResponse			Invoking the public methods of this object allows you to instruct Qwicap to take
 * 								various actions after this <code>filter</code> method has exited.
 * @throws QwicapException		If any of the usual Qwicap exceptions occurred during filter execution. In the case of
 * 								Qwicap exceptions like abandonment exceptions (see {@link QwicapAbandonmentException}),
 * 								proper operation of the web application depends on those propagating in manner 
 * 								consistent with the behavior of the "prompt pattern", so filter implementations must 
 * 								not interfere with the propagation of Qwicap exceptions.
 */
	public void filter
	(
		final FormDataSet				DataSet,
		final FormDataSetFilterContext	FilterContext,
		final FormDataSetFilterResponse	YourResponse
	) throws QwicapException;
}
