/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2007 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletRequest;


/**	The <code>FormDataSetFilterContext</code> class provides information about the context in which a 
 * 	{@link FormDataSetFilter#filter FormDataSetFilter.filter} invocation took place.
 * 	
 * 	@author Chris W. Johnson
 */
public class FormDataSetFilterContext {
	
	private final HitContext		HitCtxt;
	private final StackTraceElement	PromptingMethodTrace;
	
	
	FormDataSetFilterContext
	(
		final HitContext		HitCtxt
	)
	{
		this.HitCtxt = HitCtxt;
		this.PromptingMethodTrace = new SimplifiedStackTrace().getApplicationTraceElement();
	}

/**	Returns the current {@link HttpServletRequest} object. Ordinarily, this is the request that is being filtered.
 * 	However, if your filter invokes any of the <code>Qwicap</code> methods <code>prompt</code>, <code>redirect</code> or 
 * 	<code>rejectInput</code> (directly or indirectly), this method will subsequently return the latest 
 * 	<code>HttpServletRequest</code> object.
 * 
 * @return					The current <code>HttpServletRequest</code> object.
 */
	public HttpServletRequest getRequest() {
		return HitCtxt.getRequest();
	}

/**	Returns the current {@link HttpServletResponse} object. Ordinarily, this is the response associated with the request 
 * 	that is being filtered. However, if your filter invokes any of the <code>Qwicap</code> methods <code>prompt</code>, 
 * 	<code>redirect</code> or <code>rejectInput</code> (directly or indirectly), this method will subsequently return the 
 * 	latest <code>HttpServletResponse</code> object.
 * 
 * @return					The current <code>HttpServletResponse</code> object.
 */
	public HttpServletResponse getResponse() {
		return HitCtxt.getResponse();
	}

/**	Returns the fully-qualified name of the web application's class that prompted for the input that is being filtered.
 * 
 * @return					The name of the web application's class that prompted for the input that is being filtered.
 */
	public String getPromptingMethodClassName() {
		return PromptingMethodTrace.getClassName();
	}
	
/**	Returns the name of the web application's method that prompted for the input that is being filtered.
 * 
 * @return					The name of the web application's method that prompted for the input that is being filtered.
 */
	public String getPromptingMethodName() {
		return PromptingMethodTrace.getMethodName();
	}
}
