/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import edu.utexas.its.eis.tools.qwicap.template.xml.immutable.XMLString;
import edu.utexas.its.eis.tools.qwicap.template.xml.mutable.MutableMarkup;
import edu.utexas.its.eis.tools.qwicap.template.xml.Markup;
import edu.utexas.its.eis.tools.qwicap.util.ArrayToString;
import edu.utexas.its.eis.tools.table.lltable.MutableLLTable;
import junit.framework.TestCase;

import java.util.Arrays;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Apr 22, 2006
 * Time: 5:40:50 PM
 * To change this template use File | Settings | File Templates.
 */
public class FormJUnit extends TestCase {
	
	public void testGetControl() throws QwicapException {
		final MutableMarkup TestDoc = new XMLString(
			"<form method='post' action=''>\n" +
			"<select name='who' id='0'>\n" +
			"<option>Fred</option>\n" +
			"<option>Barney</option>\n" +
			"<option>Wilma</option>\n" +
			"<option>Betty</option>\n" +
			"</select>\n" +
			"</form>"
		).getMutable();
		
		final String			TestStr   = TestDoc.get("form").toForm().getControl("who", "Fred").toString();
		final String			MasterStr = 
		"+--------------------------------+--------------------------------+-----------------+--------------+------------------+-------------------------+------------+\n" +
		"| Enclosing Element              | Form Element                   | Control Element | Control Name | Control Value(s) | Label Element(s)        | Label Text |\n" +
		"+--------------------------------+--------------------------------+-----------------+--------------+------------------+-------------------------+------------+\n" +
		"| <form method='post' action=''> | <form method='post' action=''> | <option>        | who          | \"Fred\"           | \"<option>Fred</option>\" | Fred       |\n" +
		"+--------------------------------+--------------------------------+-----------------+--------------+------------------+-------------------------+------------+\n";
		
		System.out.println(TestStr);
		assertTrue(MasterStr.equals(TestStr));
	}
	
	public void testGetControlWithSharedNameAndValueControls1() throws QwicapException {
		final MutableMarkup TestDoc = new XMLString(
			"<form method='post' action=''>\n" +
			"<select name='who'>\n" +
			"<option value='fred'>Fred 1</option>\n" +
			"<option value='barney'>Barney 1</option>\n" +
			"<option value='wilma'>Wilma 1</option>\n" +
			"<option value='betty'>Betty 1</option>\n" +
			"</select>\n" +
			"<select name='who'>\n" +
			"<option value='fred'>Fred 2</option>\n" +
			"<option value='barney'>Barney 2</option>\n" +
			"<option value='wilma'>Wilma 2</option>\n" +
			"<option value='betty'>Betty 2</option>\n" +
			"</select>\n" +
			"</form>"
		).getMutable();
		
		final String			TestStr   = TestDoc.get("form").toForm().getControlByIndexAndValue("who", 0, "fred").toString();
		final String			MasterStr =
		"+--------------------------------+--------------------------------+-----------------------+--------------+------------------+----------------------------------------+------------+\n" +
		"| Enclosing Element              | Form Element                   | Control Element       | Control Name | Control Value(s) | Label Element(s)                       | Label Text |\n" +
		"+--------------------------------+--------------------------------+-----------------------+--------------+------------------+----------------------------------------+------------+\n" +
		"| <form method='post' action=''> | <form method='post' action=''> | <option value='fred'> | who          | \"fred\"           | \"<option value='fred'>Fred 1</option>\" | Fred 1     |\n" +
		"+--------------------------------+--------------------------------+-----------------------+--------------+------------------+----------------------------------------+------------+\n";
		
		System.out.println(TestStr);
		assertTrue(MasterStr.equals(TestStr));
	}
	
	public void testGetControlByIndexAndValueWithUnmatchedValue() throws QwicapException {
		final MutableMarkup TestDoc = new XMLString(
			"<form method='post' action=''>\n" +
			"<select name='who' id='0'>\n" +
			"<option value='fred'>Fred</option>\n" +
			"<option value='barney'>Barney</option>\n" +
			"<option value='wilma'>Wilma</option>\n" +
			"<option value='betty'>Betty</option>\n" +
			"</select>\n" +
			"<select name='who' id='1'>\n" +
			"<option value='fred'>Fred</option>\n" +
			"<option value='barney'>Barney</option>\n" +
			"<option value='wilma'>Wilma</option>\n" +
			"<option value='betty'>Betty</option>\n" +
			"</select>\n" +
			"</form>"
		).getMutable();
		
		final String			TestStr   = TestDoc.get("form").toForm().getControlByIndexAndValue("who", 0, "Wombat").toString();
		final String			MasterStr =
		"+--------------------------------+--------------------------------+-----------------+--------------+------------------+------------------+------------+\n" +
		"| Enclosing Element              | Form Element                   | Control Element | Control Name | Control Value(s) | Label Element(s) | Label Text |\n" +
		"+--------------------------------+--------------------------------+-----------------+--------------+------------------+------------------+------------+\n" +
		"| <form method='post' action=''> | <form method='post' action=''> | null            | who          | \"Wombat\"         | \"\"               | null       |\n" +
		"+--------------------------------+--------------------------------+-----------------+--------------+------------------+------------------+------------+\n";
		
		System.out.println(TestStr);
		assertTrue(MasterStr.equals(TestStr));
	}
	
	public void testGetControlWithUnmatchedValue() throws QwicapException {
		final MutableMarkup TestDoc = new XMLString(
			"<form method='post' action=''>\n" +
			"<select name='who' id='0'>\n" +
			"<option value='fred'>Fred</option>\n" +
			"<option value='barney'>Barney</option>\n" +
			"<option value='wilma'>Wilma</option>\n" +
			"<option value='betty'>Betty</option>\n" +
			"</select>\n" +
			"</form>"
		).getMutable();
		
		final String			TestStr   = TestDoc.get("form").toForm().getControl("who", "Wombat").toString();
		final String			MasterStr =
		"+--------------------------------+--------------------------------+----------------------------+--------------+------------------+------------------+------------+\n" +
		"| Enclosing Element              | Form Element                   | Control Element            | Control Name | Control Value(s) | Label Element(s) | Label Text |\n" +
		"+--------------------------------+--------------------------------+----------------------------+--------------+------------------+------------------+------------+\n" +
		"| <form method='post' action=''> | <form method='post' action=''> | <select name='who' id='0'> | who          | \"Wombat\"         | \"\"               | null       |\n" +
		"+--------------------------------+--------------------------------+----------------------------+--------------+------------------+------------------+------------+\n";
		
		System.out.println(TestStr);
		assertTrue(MasterStr.equals(TestStr));
	}
	
	public void testGetControlValueOnSelectWithoutASelectedOption() throws QwicapException {
		final Markup TestDoc = new XMLString(
			"<form method='post' action=''>\n" +
			"<select name='who' id='0'>\n" +
			"<option value='fred'>Fred</option>\n" +
			"<option value='barney'>Barney</option>\n" +
			"<option value='wilma'>Wilma</option>\n" +
			"<option value='betty'>Betty</option>\n" +
			"</select>\n" +
			"</form>"
		);
		
		final String			TestStr = TestDoc.get("form").toForm().getControlValue("who");
		
		System.out.println(TestStr);
		
		assertTrue("fred".equals(TestStr));
	}
	
	public void testGetControlValueOnSelectWithASelectedOption() throws QwicapException {
		final Markup TestDoc = new XMLString(
			"<form method='post' action=''>\n" +
			"<select name='who' id='0'>\n" +
			"<option value='fred'>Fred</option>\n" +
			"<option value='barney'>Barney</option>\n" +
			"<option value='wilma' selected='selected'>Wilma</option>\n" +
			"<option value='betty'>Betty</option>\n" +
			"</select>\n" +
			"</form>"
		);
		
		final String			TestStr = TestDoc.get("form").toForm().getControlValue("who");
		
		System.out.println(TestStr);
		
		assertTrue("wilma".equals(TestStr));
	}
	
	public void testGetSubmits1() throws QwicapException {
		final MutableMarkup TestDoc = new XMLString(
			"<div class='guess'>\n" +
			"<p>A random number between 0 and 100 (inclusive) has been selected.</p>\n" +
			"<form method='post' action='' accept-charset='UTF-8'>\n" +
			"<p>\n" +
			"<input type='hidden' name='qwicap-charset' value='UTF-8' /><input type='hidden' name='qwicap-page-id' value='0192D3DBB6F7D747' /><input type='hidden' name='qwicap-form-id' value='219571CCF5F64125' /><label>Guess the number: <input type='text' size='6' name='guess' id='input-field'/></label>\n" +
			"<input type='submit' value='Guess'/>\n" +
			"</p>\n" +
			"</form>\n" +
			"</div>\n"
		).getMutable();
		
		final Form				Form    = TestDoc.get("form").toForm();
		final FormControlInfo[]	Submits = Form.getSubmitControls();
		final MutableLLTable	Table   = new MutableLLTable().setColNames("Name", "Value(s)", "Has Label", "Label Element", "Label Text");
		
		for (final FormControlInfo FCI : Submits)
			Table.addRow().addAll(FCI.getName(), new ArrayToString(FCI.getValues()).alwaysQuote(), FCI.hasLabelText(), FCI.getLabels(), FCI.getLabelText(true));
				
		String					ThisStr;
		String					MasterStr;
		
		ThisStr   = Table.toString();
		MasterStr = 
		"+------+----------+-----------+--------------------------------------+------------+\n" +
		"| Name | Value(s) | Has Label | Label Element                        | Label Text |\n" +
		"+------+----------+-----------+--------------------------------------+------------+\n" +
		"| null | \"null\"   | true      | <input type='submit' value='Guess'/> | Guess      |\n" +
		"+------+----------+-----------+--------------------------------------+------------+\n";
		
		System.out.println(ThisStr);
		assertTrue(MasterStr.equals(ThisStr));
		
		final FormDataSet		FDS = Form.getDataSet();
		
		ThisStr   = Form.getDataSet().toString();
		MasterStr = 
		"+---------------+-------------------------------------+\n" +
		"| Form Method   | post                                |\n" +
		"+---------------+-------------------------------------+\n" +
		"| Action URL    | null                                |\n" +
		"+---------------+-------------------------------------+\n" +
		"| Context Path  |                                     |\n" +
		"+---------------+-------------------------------------+\n" +
		"| Servlet Path  |                                     |\n" +
		"+---------------+-------------------------------------+\n" +
		"| Character Set | UTF-8                               |\n" +
		"+---------------+-------------------------------------+\n" +
		"| DataSet       | Control Name   | Control Value      |\n" +
		"|               |----------------+--------------------|\n" +
		"|               | guess          | \"\"                 |\n" +
		"|               | qwicap-charset | \"UTF-8\"            |\n" +
		"|               | qwicap-form-id | \"219571CCF5F64125\" |\n" +
		"|               | qwicap-page-id | \"0192D3DBB6F7D747\" |\n" +
		"+---------------+-------------------------------------+\n";
		
		System.out.println(ThisStr);
		assertTrue(MasterStr.equals(ThisStr));
		
		Form.setDataSet(FDS);
		ThisStr = Form.getDataSet().toString();
		
		System.out.println(ThisStr);
		assertTrue(MasterStr.equals(ThisStr));
	}
	
	public void testGetSubmits2() throws QwicapException {
		final MutableMarkup TestDoc = new XMLString(
			"<div class='guess'>\n" +
			"<p>A random number between 0 and 100 (inclusive) has been selected.</p>\n" +
			"<form method='post' action='' accept-charset='UTF-8'>\n" +
			"<p>\n" +
			"<input type='hidden' name='qwicap-charset' value='UTF-8' /><input type='hidden' name='qwicap-page-id' value='0192D3DBB6F7D747' /><input type='hidden' name='qwicap-form-id' value='219571CCF5F64125' /><label>Guess the number: <input type='text' size='6' name='guess' id='input-field'/></label>\n" +
			"<input type='submit' name='guess-button' value='Guess'/>\n" +
			"</p>\n" +
			"</form>\n" +
			"</div>\n"
		).getMutable();
		
		final Form				Form    = TestDoc.get("form").toForm();
		final FormControlInfo[]	Submits = Form.getSubmitControls();
		final MutableLLTable	Table   = new MutableLLTable().setColNames("Name", "Value(s)", "Has Label", "Label Element", "Label Text");
		
		for (final FormControlInfo FCI : Submits)
			Table.addRow().addAll(FCI.getName(), new ArrayToString(FCI.getValues()).alwaysQuote(), FCI.hasLabelText(), FCI.getLabels(), FCI.getLabelText(true));
				
		String					ThisStr;
		String					MasterStr;
		
		ThisStr   = Table.toString();
		MasterStr = 
		"+--------------+----------+-----------+----------------------------------------------------------+------------+\n" +
		"| Name         | Value(s) | Has Label | Label Element                                            | Label Text |\n" +
		"+--------------+----------+-----------+----------------------------------------------------------+------------+\n" +
		"| guess-button | \"Guess\"  | true      | <input type='submit' name='guess-button' value='Guess'/> | Guess      |\n" +
		"+--------------+----------+-----------+----------------------------------------------------------+------------+\n";
		
		System.out.println(ThisStr);
		assertTrue(MasterStr.equals(ThisStr));
		
		final FormDataSet		FDS = Form.getDataSet();
		
		ThisStr   = Form.getDataSet().toString();
		MasterStr = 
		"+---------------+-------------------------------------+\n" +
		"| Form Method   | post                                |\n" +
		"+---------------+-------------------------------------+\n" +
		"| Action URL    | null                                |\n" +
		"+---------------+-------------------------------------+\n" +
		"| Context Path  |                                     |\n" +
		"+---------------+-------------------------------------+\n" +
		"| Servlet Path  |                                     |\n" +
		"+---------------+-------------------------------------+\n" +
		"| Character Set | UTF-8                               |\n" +
		"+---------------+-------------------------------------+\n" +
		"| DataSet       | Control Name   | Control Value      |\n" +
		"|               |----------------+--------------------|\n" +
		"|               | guess          | \"\"                 |\n" +
		"|               | guess-button   | \"Guess\"            |\n" +
		"|               | qwicap-charset | \"UTF-8\"            |\n" +
		"|               | qwicap-form-id | \"219571CCF5F64125\" |\n" +
		"|               | qwicap-page-id | \"0192D3DBB6F7D747\" |\n" +
		"+---------------+-------------------------------------+\n";
		
		System.out.println(ThisStr);
		assertTrue(MasterStr.equals(ThisStr));
		
		Form.setDataSet(FDS);
		ThisStr = Form.getDataSet().toString();
		
		System.out.println(ThisStr);
		assertTrue(MasterStr.equals(ThisStr));
	}
	
	public void testGetSubmits3() throws QwicapException {
		final MutableMarkup TestDoc = new XMLString(
			"<div class='guess'>\n" +
			"<p>A random number between 0 and 100 (inclusive) has been selected.</p>\n" +
			"<form method='post' action='' accept-charset='UTF-8'>\n" +
			"<p>\n" +
			"<input type='hidden' name='qwicap-charset' value='UTF-8' /><input type='hidden' name='qwicap-page-id' value='0192D3DBB6F7D747' /><input type='hidden' name='qwicap-form-id' value='219571CCF5F64125' /><label>Guess the number: <input type='text' size='6' name='guess' id='input-field'/></label>\n" +
			"<button type='submit' name='guess-button' value='guess-submitted'>Guess</button>\n" +
			"</p>\n" +
			"</form>\n" +
			"</div>\n"
		).getMutable();
		
		final Form				Form    = TestDoc.get("form").toForm();
		final FormControlInfo[]	Submits = Form.getSubmitControls();
		final MutableLLTable	Table   = new MutableLLTable().setColNames("Name", "Value(s)", "Has Label", "Label Element", "Label Text");
		
		for (final FormControlInfo FCI : Submits)
			Table.addRow().addAll(FCI.getName(), new ArrayToString(FCI.getValues()).alwaysQuote(), FCI.hasLabelText(), FCI.getLabels(), FCI.getLabelText(true));
				
		String					ThisStr;
		String					MasterStr;
		
		ThisStr   = Table.toString();
		MasterStr = 
		"+--------------+-------------------+-----------+----------------------------------------------------------------------------------+------------+\n" +
		"| Name         | Value(s)          | Has Label | Label Element                                                                    | Label Text |\n" +
		"+--------------+-------------------+-----------+----------------------------------------------------------------------------------+------------+\n" +
		"| guess-button | \"guess-submitted\" | true      | <button type='submit' name='guess-button' value='guess-submitted'>Guess</button> | Guess      |\n" +
		"+--------------+-------------------+-----------+----------------------------------------------------------------------------------+------------+\n";
		
		System.out.println(ThisStr);
		assertTrue(MasterStr.equals(ThisStr));
		
		final FormDataSet		FDS = Form.getDataSet();
		
		ThisStr   = Form.getDataSet().toString();
		MasterStr = 
		"+---------------+-------------------------------------+\n" +
		"| Form Method   | post                                |\n" +
		"+---------------+-------------------------------------+\n" +
		"| Action URL    | null                                |\n" +
		"+---------------+-------------------------------------+\n" +
		"| Context Path  |                                     |\n" +
		"+---------------+-------------------------------------+\n" +
		"| Servlet Path  |                                     |\n" +
		"+---------------+-------------------------------------+\n" +
		"| Character Set | UTF-8                               |\n" +
		"+---------------+-------------------------------------+\n" +
		"| DataSet       | Control Name   | Control Value      |\n" +
		"|               |----------------+--------------------|\n" +
		"|               | guess          | \"\"                 |\n" +
		"|               | guess-button   | \"guess-submitted\"  |\n" +
		"|               | qwicap-charset | \"UTF-8\"            |\n" +
		"|               | qwicap-form-id | \"219571CCF5F64125\" |\n" +
		"|               | qwicap-page-id | \"0192D3DBB6F7D747\" |\n" +
		"+---------------+-------------------------------------+\n";
		
		System.out.println(ThisStr);
		assertTrue(MasterStr.equals(ThisStr));
		
		Form.setDataSet(FDS);
		ThisStr = Form.getDataSet().toString();
		
		System.out.println(ThisStr);
		assertTrue(MasterStr.equals(ThisStr));
	}

	public void testAddRedundantRadioButton() throws QwicapException {

		final MutableMarkup TestDoc = new XMLString(
			"<form method='post' action='.'>\n" +
				"<input type='radio' name='choice' value='yes'/>\n" +
				"<input type='radio' name='choice' value='no'/>\n" +
				"<input type='radio' name='choice' value='maybe'/>\n" +
			"</form>"
		).getMutable();

		final Form				F = TestDoc.get("form").toForm();

		TestDoc.get("form").toForm().addChoice(false, "choice", "maybe", false, "Maybe");

		assertTrue("Form.addChoice is allowing redundant choices to be added to groups of radio buttons.", F.getControlsByName("choice").size() == 3);
	}

	public void testAddRedundantCheckbox() throws QwicapException {

		final MutableMarkup TestDoc = new XMLString(
			"<form method='post' action='.'>\n" +
				"<input type='checkbox' name='choice' value='yes'/>\n" +
				"<input type='checkbox' name='choice' value='no'/>\n" +
				"<input type='checkbox' name='choice' value='maybe'/>\n" +
			"</form>"
		).getMutable();

		final Form				F = TestDoc.get("form").toForm();

		TestDoc.get("form").toForm().addChoice(false, "choice", "maybe", false, "Maybe");

		assertTrue("Form.addChoice is allowing redundant choices to be added to groups of checkboxes.", F.getControlsByName("choice").size() == 3);
	}

	public void testAddRedundantOption() throws QwicapException {

		final MutableMarkup TestDoc = new XMLString(
			"<form method='post' action='.'>\n" +
				"<select name='choice'>\n" +
					"<option value='yes'/>\n" +
					"<option value='no'/>\n" +
					"<option value='maybe'/>\n" +
				"</select>\n" +
			"</form>"
		).getMutable();

		final Form				F = TestDoc.get("form").toForm();

		TestDoc.get("form").toForm().addChoice(false, "choice", "maybe", false, "Maybe");

		assertTrue("Form.addChoice is allowing redundant options to be added to selects.", F.get("option").size() == 3);
	}

	public void testSetDataSetOnTextFieldsWithTheSameName1() throws QwicapException {

		final MutableMarkup TestDoc = new XMLString(
			"<form method='post' action='.'>\n" +
				"<input type='text' name='field1'/>\n" +
				"<input type='text' name='field1'/>\n" +
				"<input type='text' name='field1'/>\n" +
			"</form>"
		).getMutable();

		final FormDataSet DataSet = new FormDataSet().addData("field1", "value 1", "value 2", "value 3");

		TestDoc.get("form").toForm().setDataSet(DataSet);

		final String ExpectedResult =
			"<form method='post' action='.'>\n" +
				"<input type='text' name='field1' value='value 1' />\n" +
				"<input type='text' name='field1' value='value 2' />\n" +
				"<input type='text' name='field1' value='value 3' />\n" +
			"</form>";

		final String ActualResult = TestDoc.toString();

		if (ActualResult.equals(ExpectedResult) == false)
			fail("Expected the following result:\n\n" + ExpectedResult + "\n\n...but received this result:\n\n" + ActualResult);
	}

	public void testSetDataSetOnTextFieldsWithTheSameName2() throws QwicapException {

		final MutableMarkup TestDoc = new XMLString(
			"<form method='post' action='.'>\n" +
				"<input type='text' name='field1'/>\n" +
				"<input type='text' name='field1'/>\n" +
				"<input type='text' name='field1'/>\n" +
			"</form>"
		).getMutable();

		final FormDataSet DataSet = new FormDataSet().addData("field1", "value 1", "value 2");

		TestDoc.get("form").toForm().setDataSet(DataSet);

		final String ExpectedResult =
			"<form method='post' action='.'>\n" +
				"<input type='text' name='field1' value='value 1' />\n" +
				"<input type='text' name='field1' value='value 2' />\n" +
				"<input type='text' name='field1' />\n" +
			"</form>";

		final String ActualResult = TestDoc.toString();

		if (ActualResult.equals(ExpectedResult) == false)
			fail("Expected the following result:\n\n" + ExpectedResult + "\n\n...but received this result:\n\n" + ActualResult);
	}

	public void testSetDataSetOnTextFieldsWithTheSameName3() throws QwicapException {

		final MutableMarkup TestDoc = new XMLString(
			"<form method='post' action='.'>\n" +
				"<input type='text' name='field1'/>\n" +
				"<input type='text' name='field1'/>\n" +
				"<input type='text' name='field1'/>\n" +
			"</form>"
		).getMutable();

		final FormDataSet DataSet = new FormDataSet().addData("field1", "value 1", "value 2", "value 3", "value 4");

		TestDoc.get("form").toForm().setDataSet(DataSet);

		final String ExpectedResult =
			"<form method='post' action='.'>\n" +
				"<input type='text' name='field1' value='value 1' />\n" +
				"<input type='text' name='field1' value='value 2' />\n" +
				"<input type='text' name='field1' value='value 3' />\n" +
			"</form>";

		final String ActualResult = TestDoc.toString();

		if (ActualResult.equals(ExpectedResult) == false)
			fail("Expected the following result:\n\n" + ExpectedResult + "\n\n...but received this result:\n\n" + ActualResult);
	}

	public void testSetDataSetOnCheckBoxesWithTheSameName() throws QwicapException {

		final MutableMarkup TestDoc = new XMLString(
			"<form method='post' action='.'>\n" +
				"<input type='checkbox' name='field1' value='value 1'/>\n" +
				"<input type='checkbox' name='field1' value='value 2'/>\n" +
				"<input type='checkbox' name='field1' value='value 3'/>\n" +
			"</form>"
		).getMutable();

		final FormDataSet DataSet = new FormDataSet().addData("field1", "value 1", "value 2", "value 3");

		TestDoc.get("form").toForm().setDataSet(DataSet);

		final String ExpectedResult =
			"<form method='post' action='.'>\n" +
				"<input type='checkbox' name='field1' value='value 1' checked='checked' />\n" +
				"<input type='checkbox' name='field1' value='value 2' checked='checked' />\n" +
				"<input type='checkbox' name='field1' value='value 3' checked='checked' />\n" +
			"</form>";

		final String ActualResult = TestDoc.toString();

		if (ActualResult.equals(ExpectedResult) == false)
			fail("Expected the following result:\n\n" + ExpectedResult + "\n\n...but received this result:\n\n" + ActualResult);
	}

	public void testGetCharacterSets() throws QwicapException {

		final MutableMarkup TestDoc = new XMLString(
			"<form method='post' action='.'>\n" +
				"<input type='checkbox' name='field1' value='value 1'/>\n" +
				"<input type='checkbox' name='field1' value='value 2'/>\n" +
				"<input type='checkbox' name='field1' value='value 3'/>\n" +
			"</form>"
		).getMutable();

		final Form			TheForm  = TestDoc.get("form").toForm();
		String[]			CharSets = TheForm.getCharacterSets();

		assertTrue(CharSets.length == 1);
		assertTrue(Arrays.equals(CharSets, new String[] { "UTF-8" } ));

		TheForm.setCharacterSet("");
		CharSets = TheForm.getCharacterSets();
		
		assertTrue(CharSets.length == 1);
		assertTrue(Arrays.equals(CharSets, new String[] { "UTF-8" } ));

		TheForm.setCharacterSet("UTF-16");
		CharSets = TheForm.getCharacterSets();

		assertTrue(CharSets.length == 1);
		assertTrue(Arrays.equals(CharSets, new String[] { "UTF-16" } ));

		TheForm.setCharacterSet("UTF-16,UTF-8, US-ASCII ISO-8859-1");
		CharSets = TheForm.getCharacterSets();

		assertTrue(CharSets.length == 4);
		assertTrue(Arrays.equals(CharSets, new String[] { "UTF-16", "UTF-8", "US-ASCII", "ISO-8859-1" } ));
	}
	
	public void testSetControlValueOnSelectControlWithSelectedItem() throws QwicapException {
		
		final MutableMarkup TestDoc = new XMLString(
			"<form method='post' action='.'>\n" +
				"<select name='method'>\n" + 
					"<option value='1'>One</option>\n" +
					"<option value='2' selected='selected'>Two</option>\n" +
					"<option value='3'>Three</option>\n" +
				"</select>\n" + 
			"</form>"
		).getMutable();
		
		final Form				TheForm = TestDoc.get("form").toForm();
		
		TheForm.setControlValue("method", "1");
								
		final String CorrectOutput = 
			"<form method='post' action='.'>\n" +
			"<select name='method'>\n" +
			"<option value='1' selected='selected'>One</option>\n" +
			"<option value='2'>Two</option>\n" +
			"<option value='3'>Three</option>\n" +
			"</select>\n" +
			"</form>";
		
		assertTrue(TestDoc.toString().equals(CorrectOutput));
	}
	
	public void testSetControlValueOnRadioControlsWithSelectedItem() throws QwicapException {
		
		final MutableMarkup TestDoc = new XMLString(
			"<form method='post' action='.'>\n" +
				"<input type='radio' name='method' value='1'/> One\n" +
				"<input type='radio' name='method' value='2' checked='checked'/> Two\n" +
				"<input type='radio' name='method' value='3'/> Three\n" +
			"</form>"
		).getMutable();
		
		final Form				TheForm = TestDoc.get("form").toForm();
		
		TheForm.setControlValue("method", "1");
								
		final String CorrectOutput = 
			"<form method='post' action='.'>\n" +
			"<input type='radio' name='method' value='1' checked='checked' /> One\n" +
			"<input type='radio' name='method' value='2' /> Two\n" +
			"<input type='radio' name='method' value='3' /> Three\n" +
			"</form>";
		
		assertTrue(TestDoc.toString().equals(CorrectOutput));
	}
	
	public void testSetControlValueOnCheckboxControlsWithSelectedItem() throws QwicapException {
		
		final MutableMarkup TestDoc = new XMLString(
			"<form method='post' action='.'>\n" +
				"<input type='checkbox' name='method' value='1'/> One\n" +
				"<input type='checkbox' name='method' value='2' checked='checked'/> Two\n" +
				"<input type='checkbox' name='method' value='3'/> Three\n" +
			"</form>"
		).getMutable();
		
		final Form				TheForm = TestDoc.get("form").toForm();
		
		TheForm.setControlValue("method", "1");
								
		final String CorrectOutput = 
			"<form method='post' action='.'>\n" +
			"<input type='checkbox' name='method' value='1' checked='checked' /> One\n" +
			"<input type='checkbox' name='method' value='2' /> Two\n" +
			"<input type='checkbox' name='method' value='3' /> Three\n" +
			"</form>";
		
		assertTrue(TestDoc.toString().equals(CorrectOutput));
	}
}
