/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2011 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import edu.utexas.its.eis.tools.qwicap.template.xml.Markup;
import edu.utexas.its.eis.tools.qwicap.template.xml.TagException;


/**	<code>FormNotFoundException</code> is thrown, depending on the context, when: (1) A "<samp>form</samp>" element 
 * 	cannot be found. (2) Within a particular body of markup, a "<samp>form</samp>" element cannot be found that could 
 * 	have supplied a particular form data set, or unique form ID. (3) A reference to an element that should have been a 
 * 	"<samp>form</samp>" element was not.
 *	
 *	@author Chris W. Johnson
 */
public final class FormNotFoundException extends TagException {
	
	
	public FormNotFoundException
	(
		final String			ErrorMessage
	)
	{
		super(ErrorMessage);
	}
	
	FormNotFoundException
	(
		final Markup			ThePage,
		final String			FormIDStr
	)
	{
		super("Cannot find any form in page \"" + ThePage.getMarkupName() + "\" whose ID is \"" + FormIDStr + "\".");
	}
	
	FormNotFoundException
	(
		final Markup			ThePage,
		final FormDataSet		DataSet,
		final boolean			IgnoreQwicapParameters
	)
	{
		super(createMessage(ThePage, DataSet, IgnoreQwicapParameters));
	}	
	
	private static String createMessage
	(
		final Markup			ThePage,
		final FormDataSet		DataSet,
		final boolean			IgnoreQwicapControls
	)
	{
		final StringBuilder		Buff = new StringBuilder(192);
		
		Buff.append("Cannot find any one form in page \"").append(ThePage.getMarkupName()).append("\" that could have supplied all of the following control values: ");
		
		int						ParamCount = 0;
		
		for (final FormControlValue Param : DataSet) {
			
			if (IgnoreQwicapControls && Param.isQwicapControl())
				continue;
			
			if (ParamCount++ != 0)
				Buff.append(", ");
			
			Buff.append(Param.getName());
		}
		
		Buff.append('.');
		
		return Buff.toString();
	}
}
