/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2007 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import junit.framework.TestCase;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Sep 17, 2007
 * Time: 8:28:34 PM
 * To change this template use File | Settings | File Templates.
 */
public class HTTPAcceptListJUnit extends TestCase {

	public void testDescendingList() {
		final HTTPAcceptLanguageList	HAL  = new HTTPAcceptLanguageList("en, ru;q=.1, es;q=0.8, en-gb;q=.8");
		final HTTPLanguageRange[]		List = HAL.getDescendingIdentifierList();

	//	System.out.println("Sorted list (descending quality): " + new ArrayToString(List).alwaysQuote());

		assertTrue("Element 0 should be 'en', but is: "    + List[0], "en".equals(List[0].toString()));
		assertTrue("Element 1 should be 'es', but is: "    + List[1], "es".equals(List[1].toString()));
		assertTrue("Element 2 should be 'en-gb', but is: " + List[2], "en-gb".equals(List[2].toString()));
		assertTrue("Element 3 should be 'ru', but is: "    + List[3], "ru".equals(List[3].toString()));
	}

	public void testDescendingListEqualWeights() {
		final HTTPAcceptLanguageList	HAL  = new HTTPAcceptLanguageList("en, ru, es, en-gb");
		final HTTPLanguageRange[]		List = HAL.getDescendingIdentifierList();

	//	System.out.println("Sorted list (descending quality): " + new ArrayToString(List).alwaysQuote());

		assertTrue("Element 0 should be 'en', but is: "    + List[0], "en".equals(List[0].toString()));
		assertTrue("Element 1 should be 'ru', but is: "    + List[1], "ru".equals(List[1].toString()));
		assertTrue("Element 2 should be 'es', but is: "    + List[2], "es".equals(List[2].toString()));
		assertTrue("Element 3 should be 'en-gb', but is: " + List[3], "en-gb".equals(List[3].toString()));
	}

	public void testAccepts1() {
		final HTTPAcceptLanguageList	HAL  = new HTTPAcceptLanguageList("en, ru;q=.1, es;q=0.8, en-gb;q=.8, da;q=0");

		assertTrue(HAL.accepts("en"));
		assertTrue(HAL.accepts("EN"));
		assertTrue(HAL.accepts("ru"));
		assertTrue(HAL.accepts("RU"));
		assertTrue(HAL.accepts("es"));
		assertTrue(HAL.accepts("ES"));
		assertTrue(HAL.accepts("en-gb"));
		assertTrue(HAL.accepts("en-GB"));
		assertTrue(HAL.accepts("EN-GB"));
		assertFalse(HAL.accepts("da"));	//	An item in the list, but with a zero quality.
		assertFalse(HAL.accepts("DA"));	//	An item in the list, but with a zero quality.
		assertFalse(HAL.accepts("de"));	//	An item not in the list at all.
		assertFalse(HAL.accepts("DE"));	//	An item not in the list at all.
	}

	public void testAcceptsAll() {
		final HTTPAcceptLanguageList	HAL  = new HTTPAcceptLanguageList("*");

		assertTrue(HAL.accepts("en"));
		assertTrue(HAL.accepts("EN"));
		assertTrue(HAL.accepts("ru"));
		assertTrue(HAL.accepts("RU"));
		assertTrue(HAL.accepts("es"));
		assertTrue(HAL.accepts("ES"));
		assertTrue(HAL.accepts("en-gb"));
		assertTrue(HAL.accepts("en-GB"));
		assertTrue(HAL.accepts("EN-GB"));
		assertTrue(HAL.accepts("da"));	//	An item in the list, but with a zero quality.
		assertTrue(HAL.accepts("DA"));	//	An item in the list, but with a zero quality.
		assertTrue(HAL.accepts("de"));	//	An item not in the list at all.
		assertTrue(HAL.accepts("DE"));	//	An item not in the list at all.
	}
}
