/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2011 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


/*
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Apr 27, 2011
 * Time: 6:42:56 AM
 * To change this template use File | Settings | File Templates.
 */


class HTTPContentTypeDefaultCharSet extends XMLContentTypeDefaultCharSet {
	
	@Override public String get
	(
		final String			Type,	//	Can be null.
		final String			Subtype	//	Can be null.
	)
	{
	
	//	The default character set specified in RFC 2616 is "ISO-8859-1". See RFC 2616, section 3.7.1, 
	//	<http://www.w3.org/Protocols/rfc2616/rfc2616-sec3.html#sec3.7.1>, "Canonicalization and Text Defaults", 
	//	which states: "When no explicit charset parameter is provided by the sender, media subtypes of the 'text' 
	//	type are defined to have a default charset value of 'ISO-8859-1' when received via HTTP. Data in character 
	//	sets other than 'ISO-8859-1' or its subsets MUST be labeled with an appropriate charset value. See section 
	//	3.4.1 for compatibility problems."
		
	//	todo Note: RFC 3023, <http://www.ietf.org/rfc/rfc3023.txt>, section 3.1, says the HTTP specification got it wrong, and the default character set in this case should be US-ASCII.
	
	//	From RFC 3023, <http://www.ietf.org/rfc/rfc3023.txt>, section 3.1:
	//
	//		"Conformant with [RFC2046], if a text/xml entity is received with
	//		the charset parameter omitted, MIME processors and XML processors
	//		MUST use the default charset value of "us-ascii"[ASCII].  In cases
	//		where the XML MIME entity is transmitted via HTTP, the default
	//		charset value is still "us-ascii".  (Note: There is an
	//		inconsistency between this specification and HTTP/1.1, which uses
	//		ISO-8859-1[ISO8859] as the default for a historical reason.  Since
	//		XML is a new format, a new default should be chosen for better
	//		I18N.  US-ASCII was chosen, since it is the intersection of UTF-8
	//		and ISO-8859-1 and since it is already used by MIME.)"
	//
	//	See also <http://www.imc.org/ietf-xml-mime/mail-archive/msg00854.html> for important commentary from one of
	//	the authors of RFC 3023.
	//
	//	if ("text".equalsIgnoreCase(Type))
	//		return "ISO-8859-1";
		
		String					CharSet = super.get(Type, Subtype);
		
		if (CharSet == null) {
			
			if ("text".equalsIgnoreCase(Type))
				CharSet = "ISO-8859-1";
		}
		
		return CharSet;
	}
}