/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2007-2011 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import junit.framework.TestCase;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Oct 1, 2007
 * Time: 8:25:16 PM
 * To change this template use File | Settings | File Templates.
 */
public class HTTPContentTypeJUnit extends TestCase {

	public void testMIMEOnly() {
		assertTrue("text/html".equals(new HTTPContentType("text/html").getMIMEType()));
	}

	public void testCharacterSetOnly1() {
		assertTrue("UTF-8".equals(new HTTPContentType("charset=UTF-8").getCharacterSet()));
	}

	public void testCharacterSetOnly2() {
		assertTrue("UTF-8".equals(new HTTPContentType("charset='UTF-8'").getCharacterSet()));
	}

	public void testCharacterSetOnly3() {
		assertTrue("UTF-8".equals(new HTTPContentType("charset=\"UTF-8\"").getCharacterSet()));
	}
	
	public void testBoth1() {
		final HTTPContentType		CT = new HTTPContentType("text/html; charset=UTF-8");
		
		assertTrue("text/html".equals(CT.getMIMEType()));
		assertTrue("UTF-8".equals(CT.getCharacterSet()));
	}
	
	public void testBoth2() {
		final HTTPContentType		CT = new HTTPContentType("text/html;charset=UTF-8");
		
		assertTrue("text/html".equals(CT.getMIMEType()));
		assertTrue("UTF-8".equals(CT.getCharacterSet()));
	}
	
	public void testBoth3() {
		final HTTPContentType		CT = new HTTPContentType("text/html; charset='UTF-8'");
		
		assertTrue("text/html".equals(CT.getMIMEType()));
		assertTrue("UTF-8".equals(CT.getCharacterSet()));
	}
	
	public void testBoth4() {
		final HTTPContentType		CT = new HTTPContentType("text/html; charset = UTF-8; attribute=value");
		
		assertTrue("text/html".equals(CT.getMIMEType()));
		assertTrue("UTF-8".equals(CT.getCharacterSet()));
	}
	
	public void testBoth5() {
		final HTTPContentType		CT = new HTTPContentType("text/html;charset=UTF-8;attribute=value");
		
		assertTrue("text/html".equals(CT.getMIMEType()));
		assertTrue("UTF-8".equals(CT.getCharacterSet()));
	}
	
	public void testBoth6() {
		final HTTPContentType		CT = new HTTPContentType("text/html;charset='UTF-8';attribute=value");
		
		assertTrue("text/html".equals(CT.getMIMEType()));
		assertTrue("UTF-8".equals(CT.getCharacterSet()));
	}
	
	public void testBoth7() {
		final HTTPContentType		CT = new HTTPContentType("text/html;charset=\"UTF-8\";attribute=value");
		
		assertTrue("text/html".equals(CT.getMIMEType()));
		assertTrue("UTF-8".equals(CT.getCharacterSet()));
	}
	
	public void testBoth8() {
		final HTTPContentType		CT = new HTTPContentType("text/html ; charset=UTF-8");
		
		assertTrue("text/html".equals(CT.getMIMEType()));
		assertTrue("UTF-8".equals(CT.getCharacterSet()));
	}
	
	public void testBothNull() {
		final HTTPContentType		CT = new HTTPContentType(null);
		
		assertTrue(CT.getMIMEType()     == null);
		assertTrue(CT.getCharacterSet() == null);
	}
}
