/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2007-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import edu.utexas.its.eis.tools.qwicap.util.ArrayIterator;
import edu.utexas.its.eis.tools.qwicap.util.ArrayToString;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Oct 25, 2007
 * Time: 5:50:39 AM
 * To change this template use File | Settings | File Templates.
 */
class HTTPLanguageTags implements Iterable<HTTPLanguageTag> {
	
	protected final HTTPLanguageTag[]	Tags;
	
	
	HTTPLanguageTags
	(
		final HTTPLanguageTag[]		Tags
	)
	{
		this.Tags = Tags;
	}
	
	int size() {
		return Tags.length;	
	}
	
	HTTPLanguageTag[] toArray() {
		return Tags;
	}
	
	@Override public Iterator<HTTPLanguageTag> iterator() {
		return new ArrayIterator<HTTPLanguageTag>(Tags);	
	}
	
	HTTPLanguageTag getSupportedIdentifier
	(
		final String			LangStr
	)
	{
		final String			LangStrNormalized = HTTPLanguageTag.normalize(LangStr);
		
		for (final HTTPLanguageTag LangTag : Tags) {
		
			if (LangTag.toString().equals(LangStrNormalized))
				return LangTag;
		}
		
		return null;
	}

	List<HTTPLanguageTag> getMatchingTags
	(
		final HTTPLanguageRange	LangRange
	)
	{
		final ArrayList<HTTPLanguageTag> List = new ArrayList<HTTPLanguageTag>();
		
		for (final HTTPLanguageTag LangTag : Tags) {
			
			if (LangRange.matches(LangTag))
				List.add(LangTag);
		}
		
		return List;
	}
	
	HTTPLanguageTag compareTagToListGetBest
	(
		final HTTPLanguageTag	LangRange
	)
	{
		int						MaxMatchQuality = 0;
		HTTPLanguageTag			MaxMatch        = null;
		
		for (final HTTPLanguageTag LangTag : Tags) {
			final int				MatchQuality = LangRange.getMatchQuality(LangTag);
			
			if (MaxMatchQuality < MatchQuality) {
				
				MaxMatchQuality = MatchQuality;
				MaxMatch        = LangTag;
			}
		}
		
		return MaxMatch;
	}
	
	HTTPLanguageTag compareListToTagGetBest
	(
		final HTTPLanguageTag	CompareToThisTag
	)
	{
		int						MaxMatchQuality = 0;
		HTTPLanguageTag			MaxMatch        = null;
		
		for (final HTTPLanguageTag LangTag : Tags) {
			final int				MatchQuality = LangTag.getMatchQuality(CompareToThisTag);
			
			if (MaxMatchQuality < MatchQuality) {
				
				MaxMatchQuality = MatchQuality;
				MaxMatch        = LangTag;
			}
		}
		
		return MaxMatch;
	}
	
	@Override public String toString() {
		return new ArrayToString(Tags).alwaysQuote().toString();
	}
}
