/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2008-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import edu.utexas.its.eis.tools.qwicap.template.css.Results;
import edu.utexas.its.eis.tools.qwicap.template.xml.TagException;


/**	Encapsulates all of the information about an arbitrary problem that should be reported to the user. Additional 
 * 	debugging information may be included in a comment block within the page, if--and only if--the web application has 
 * 	Qwicap operating in its low-security mode (<code>ReportExceptionToUser</code> is <code>false</code>).
 * 
 * 	@author Chris W. Johnson
 */
abstract class Incident {

/**	Returns the description of this incident that is meant to be seen in the web interface, by the user.
 * 
 * @return						An object containing the description of this incident that is meant to be shown to the
 * 								user.
 */
	abstract Object getHumanInterfaceMessage();

/**	Returns an object containing debugging information related to the incident, if Qwicap is operating in its optional,
 * 	low-security "debug" mode. Returns <code>null</code>, if Qwicap is operating in its default, "high-security" mode.
 * 
 * @return						An object containing debugging information, or it may be <code>null</code> for two 
 * 								reasons: (1) no debugging information was provided by the application, (2) Qwicap is 
 * 								operating in its default, "high-security" mode.
 */	
	abstract Object getDebugInformation();		//	May return null.

/**	Adds the incident report to an application's web page. Note that this does not apply to Qwicap's built-in "crash"
 * 	and "start" pages, which have more specific requirements.
 * 
 * @param ErrorReportDiv		A <code>Results</code> object containing references to one, or more, elements in a 
 * 								body of mutable markup into which the incident report should be inserted. Typically,
 * 								the element is a "div" tag, but it can be any element that is allowed to have content.
 * @throws TagException			If there was a problem manipulating the markup.
 */
	abstract void addReportToPage
	(
		final Results			ErrorReportDiv
	)
	throws TagException;
	
	@Override public String toString() {
		return getHumanInterfaceMessage().toString();	
	}
}
