/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import java.text.DecimalFormat;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Oct 6, 2006
 * Time: 4:36:10 PM
 * To change this template use File | Settings | File Templates.
 */
final class IntMinMaxCount {
	
	private static final int[]	kEmptyIntArray = {};
	
	private int					UpdateCount;
	private int					Value;
	private int					Min;
	private int					Max;
	
	
	IntMinMaxCount() { 
		reset();
	}

	IntMinMaxCount
	(
		final int				Count
	)
	{
		this.UpdateCount = 1;
		this.Value       = Count;
		this.Min         = Count;
		this.Max         = Count;
	}
	
	void set
	(
		final int				NewCount
	)
	{
		Value = NewCount;
		
		if (Max < NewCount)
			Max = NewCount;
		
		if (Min > NewCount)
			Min = NewCount;
			
		UpdateCount++;
	}
	
	void increment() {
		final int				NewCount = ++Value;
		
		if (Max < NewCount)
			Max = NewCount;
		
		if (Min > NewCount)	//	This is only relevant when UpdateCount == 0.
			Min = NewCount;
		
		UpdateCount++;
	}
	
	void decrement() {
		final int				NewCount = --Value;
		
		if (Max < NewCount)	//	This is only relevant when UpdateCount == 0.
			Max = NewCount;
		
		if (Min > NewCount)
			Min = NewCount;
			
		UpdateCount++;
	}
	
	void reset() {
		
		UpdateCount = 0;
		Value       = 0;
		Min         = Integer.MAX_VALUE;
		Max         = Integer.MIN_VALUE;
	}
	
	int getMinimum() {
		return UpdateCount != 0 ? Min : 0;
	}
	
	int getMaximum() {
		return UpdateCount != 0 ? Max : 0;
	}
	
	int getCount() {
		return Value;
	}
	
	String toRangeString
	(
		final DecimalFormat		ValFmt
	)
	{
		if (UpdateCount == 0)
			return "n/a";
		
		if (Min == Max)
			return ValFmt.format(Min);
		
		return ValFmt.format(Min) + '-' + ValFmt.format(Max);
	}
	
	int[] toRangeArray() {
		return UpdateCount != 0 ? new int[] { Min, Max } : kEmptyIntArray;
	}
}
