/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2007-2010 Chris W. Johnson. All rights reserved.
	
	Chris W. Johnson's copyright notice in this evolving version reflects his original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import java.util.LinkedHashMap;
import java.util.Map;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Oct 23, 2007
 * Time: 4:53:28 PM
 * To change this template use File | Settings | File Templates.
 */
final class LRUCacheMap<KeyType, ValueType> extends LinkedHashMap<KeyType, ValueType> {

	private final int			WorstCaseSize;
	private int					DiscardCount;


	LRUCacheMap
	(
		final int				AnticipatedSize,
		final int				WorstCaseSize
	)
	{
		super((int) (AnticipatedSize * 4.0/3.0 + 1), 0.75f, true);
		this.WorstCaseSize = WorstCaseSize;
	}

	@Override protected boolean removeEldestEntry
	(
		final Map.Entry<KeyType, ValueType>	Eldest
	)
	{
		final boolean			Discard = size() > WorstCaseSize;

		if (Discard)
			DiscardCount++;

		return Discard;
	}

	int getDiscardCount
	(
		final boolean			Reset
	)
	{
		final int				DiscardCount = this.DiscardCount;

		if (Reset)
			this.DiscardCount = 0;

		return DiscardCount;
	}
}
