/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import edu.utexas.its.eis.tools.qwicap.template.xml.Match;


/**	<code>LabelElementInfo</code> associates a <code>Match</code> representing a form element (typically a control or
 * 	label element) with one of the applicable label strings, if any, that were found in the form's markup.
 *	
 *	@author Chris W. Johnson
 */
final class LabelElementInfo {
	
	final Match				Element;
	final String			LabelStr;
	
	
	LabelElementInfo
	(
		final Match				Element,
		final String			LabelStr
	)
	{
		this.Element  = Element;
		this.LabelStr = LabelStr;
	}
	
	String getLabelText
	(
		final boolean			Tidy
	)
	{
		return Tidy ? getLabelTextTidy() : LabelStr;
	}
	
	String getLabelTextTidy() {
		String					LabelText = LabelStr.trim();
		
		if (LabelText.endsWith(":"))
			LabelText = LabelText.substring(0, LabelText.length() - 1).trim();	//	Remove the trailing colon.
		
		return LabelText;
	}
}
