/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import java.text.DecimalFormat;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Oct 6, 2006
 * Time: 7:38:06 PM
 * To change this template use File | Settings | File Templates.
 */
final class LongMinMaxTotal {
	
	private static final double[]	kEmptyArray = {};
	
	private int				Samples;
	private long			Total;
	private long			MinSample = Long.MAX_VALUE;
	private long			MaxSample = Long.MIN_VALUE;
	
	
	void add
	(
		final long				Sample
	)
	{
		Total += Sample;
		
		if (MaxSample < Sample)
			MaxSample = Sample;
		
		if (MinSample > Sample)
			MinSample = Sample;
		
		Samples++;
	}
	
	int getSampleCount() {
		return Samples;
	}
	
	long getTotal() {
		return Total;
	}
	
	long getMinimum() {
		return Samples != 0 ? MinSample : 0;
	}
	
	long getMaximum() {
		return Samples != 0 ? MaxSample : 0;
	}
	
	double getAverage() {
		return (double) Total / Samples;
	}
	
	double[] toMinAvgMaxArray
	(
		final double			Divisor
	)
	{
		if (Samples == 0)
			return kEmptyArray;
		
		return new double[] { MinSample / Divisor, getAverage() / Divisor, MaxSample / Divisor };
	}
	
	String toMinAvgMaxString
	(
		final double			Divisor,
		final DecimalFormat		ValFmt
	)
	{
		if (Samples == 0)
			return "n/a";
		
		final String			Min = ValFmt.format(MinSample / Divisor);
		final String			Max = ValFmt.format(MaxSample / Divisor);
		
		if (Min.equals(Max))
			return Min;
		
		return Min + '/' + ValFmt.format(getAverage() / Divisor) + '/' + Max;
	}
	
}
