/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2008-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import edu.utexas.its.eis.tools.table.CellFormatter;
import edu.utexas.its.eis.tools.table.FormattedValue;
import edu.utexas.its.eis.tools.table.CellDataAlignment;
import edu.utexas.its.eis.tools.table.ValueKind;
import edu.utexas.its.eis.tools.qwicap.util.Null;

import java.text.DecimalFormat;
import java.lang.reflect.Array;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Apr 16, 2008
 * Time: 7:41:54 PM
 * To change this template use File | Settings | File Templates.
 */
class NumericArrayCellFormatter implements CellFormatter {
	
	private final DecimalFormat	Fmt;
	private final String		Sep;
	private final boolean		CollapseIdenticalValues;
	private final String		EmptyArrayStr;
	
	
	NumericArrayCellFormatter
	(
		final String			NumberFmt,
		final String			ItemSep
	)
	{
		this(NumberFmt, ItemSep, false, null);
	}
	
	NumericArrayCellFormatter
	(
		final String			NumberFmt,
		final String			ItemSep,
		final boolean			CollapseValues,
		final String			EmptyArrayStr
	)
	{
		this.Fmt                     = new DecimalFormat(NumberFmt);
		this.Sep                     = ItemSep;
		this.CollapseIdenticalValues = CollapseValues;
		this.EmptyArrayStr           = EmptyArrayStr;
	}
	
	@Override public FormattedValue format
	(
		final Object			ArrayObj
	) 
	throws Exception 
	{
		if (ArrayObj == null)
			return new FormattedValue(null, ValueKind.NUMERIC, CellDataAlignment.RIGHT);
				
		if (ArrayObj.getClass().isArray()) {
			final int				Count   = Array.getLength(ArrayObj);
			
			if (Count == 0 && EmptyArrayStr != null)
				return new FormattedValue(EmptyArrayStr, ValueKind.NUMERIC, CellDataAlignment.RIGHT);
			
			String[]				ValStrs = new String[Count];
			
			for (int Index = 0; Index < Count; Index++) {
				final Object			Elem = Array.get(ArrayObj, Index);
				
				if (Elem == null)
					ValStrs[Index] = null;
				else if (Elem instanceof Number)
					ValStrs[Index] = Fmt.format(Elem);
				else
					ValStrs[Index] = Elem.toString();
			}
			
			if (CollapseIdenticalValues && Count > 1) {
				final String			ValStr   = ValStrs[0];
				boolean					AllEqual = true;
				
				for (int Index = 1; Index < Count; Index++) {
				
					if (Null.safeEquals(ValStr, ValStrs[Index]) == false) {
						
						AllEqual = false;
						break;
					}
				}
				
				if (AllEqual)
					ValStrs = new String[] { ValStr };
			}
			
			final StringBuilder		Buff  = new StringBuilder(32);
			
			for (int Index = 0, StrCount = ValStrs.length; Index < StrCount; Index++) {
				final String			ValStr = ValStrs[Index];
				
				if (Index > 0)
					Buff.append(Sep);
				
				Buff.append(ValStr);	
			}
			
			return new FormattedValue(Buff.toString(), ValueKind.NUMERIC, CellDataAlignment.RIGHT);
		
		} else {
		
		//	ArrayObj is not, in fact, a reference to an array.
			
			if (ArrayObj instanceof Number)
				return new FormattedValue(Fmt.format(ArrayObj), ValueKind.NUMERIC, CellDataAlignment.RIGHT);
			else
				return new FormattedValue(ArrayObj, ValueKind.NON_NUMERIC, CellDataAlignment.LEFT);
		}
	}

	@Override public CellDataAlignment getAlignment() {
		return CellDataAlignment.RIGHT;
	}
}
