/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import edu.utexas.its.eis.tools.qwicap.template.xml.TagException;
import edu.utexas.its.eis.tools.qwicap.template.xml.Markup;
import edu.utexas.its.eis.tools.qwicap.template.xml.immutable.XMLString;
import edu.utexas.its.eis.tools.qwicap.template.xml.immutable.XMLDocument;
import junit.framework.TestCase;

import java.io.File;
import java.util.HashSet;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: May 29, 2007
 * Time: 7:07:04 AM
 * To change this template use File | Settings | File Templates.
 */
public class PageJUnit extends TestCase {

	public void testContentTypeAndCharset0() throws TagException {
		final XMLString						Doc      = new XMLString("<html><head></head><body></body></html>");
		final Page.PageMIMETypeAndCharSet	TypeInfo = new Page.PageMIMETypeAndCharSet(Doc);

		System.out.println(TypeInfo);

		assertTrue("Content type/charset mismatch.", "text/html; charset=UTF-8".equals(TypeInfo.toString()));
	}

	public void testContentTypeAndCharset1() throws TagException {
		final XMLString						Doc      = new XMLString("<html><head><meta http-equiv='Content-Type' content='text/html; charset=iso-8859-1'/></head><body></body></html>");
		final Page.PageMIMETypeAndCharSet	TypeInfo = new Page.PageMIMETypeAndCharSet(Doc);

		System.out.println(TypeInfo);
		
	//	Note that the "charset" name is canonicalized by the Content-Type interpretation code, which changes its case.
		
		assertTrue("Content type/charset mismatch.", "text/html; charset=ISO-8859-1".equals(TypeInfo.toString()));
	}

	public void testContentTypeAndCharset2() throws TagException {
		final XMLString						Doc      = new XMLString("<html><head><meta http-equiv='Content-Type' content='text/html ; charset=iso-8859-1'/></head><body></body></html>");
		final Page.PageMIMETypeAndCharSet	TypeInfo = new Page.PageMIMETypeAndCharSet(Doc);

		System.out.println(TypeInfo);
		
	//	Note that the "charset" name is canonicalized by the Content-Type interpretation code, which changes its case.

		assertTrue("Content type/charset mismatch.", "text/html; charset=ISO-8859-1".equals(TypeInfo.toString()));
	}

	public void testContentTypeAndCharset3() throws TagException {
		final XMLString						Doc      = new XMLString("<html><head><meta http-equiv='Content-Type' content='text/html'/></head><body></body></html>");
		final Page.PageMIMETypeAndCharSet	TypeInfo = new Page.PageMIMETypeAndCharSet(Doc);

		System.out.println(TypeInfo);
		
	//	OK, here we have a problem, because, if the XML is obtained via HTTP *and* its MIME type is "text" *and* no 
	//	"charset" was specified, then the default character set should be "ISO-8859-1", as dictated by section 3.7.1 of
	//	the HTTP 1.1 specification, RFC 2616. However, if the XML is obtained by other means, and none of the various 
	//	character set identification methods apply, then the default character set should be "UTF-8".

		assertTrue("Content type/charset mismatch.", "text/html; charset=UTF-8".equals(TypeInfo.toString()));
	}

	public void testContentTypeAndCharset4() throws TagException {
		final XMLString						Doc      = new XMLString("<?xml version='1.0' encoding='US-ASCII'?><html><head><meta http-equiv='Content-Type' content='text/html; charset=UTF-8'/></head><body></body></html>");
		final Page.PageMIMETypeAndCharSet	TypeInfo = new Page.PageMIMETypeAndCharSet(Doc);

		assertTrue("Content type/charset mismatch.", "text/html; charset=US-ASCII".equals(TypeInfo.toString()));
	}

	public void testContentTypeAndCharset5() throws TagException {
		final XMLString						Doc      = new XMLString("<?xml version='1.0' encoding='US-ASCII'?><html><head><meta http-equiv='Content-Type' content='application/xhtml+xml; charset=UTF-8'/></head><body></body></html>");
		final Page.PageMIMETypeAndCharSet	TypeInfo = new Page.PageMIMETypeAndCharSet(Doc);

		assertTrue("Content type/charset mismatch.", "application/xhtml+xml; charset=US-ASCII".equals(TypeInfo.toString()));
	}

	public void testContentTypeAndCharset6() throws TagException {
		final XMLString						Doc      = new XMLString("<?xml version='1.0' encoding='US-ASCII'?><!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"DTD/xhtml1-strict.dtd\"><html xmlns=\"http://www.w3.org/1999/xhtml\"><head><title>Test</title></head><body></body></html>");
		final Page.PageMIMETypeAndCharSet	TypeInfo = new Page.PageMIMETypeAndCharSet(Doc);

		System.out.println(TypeInfo);

		assertTrue("Content type/charset mismatch.", "application/xhtml+xml; charset=US-ASCII".equals(TypeInfo.toString()));
	}

	public void testContentTypeAndCharset7() throws Exception {
		final Markup						Doc      = new XMLDocument(Qwicap.class.getResource("qwicap-crash.html"));
		final Page.PageMIMETypeAndCharSet	TypeInfo = new Page.PageMIMETypeAndCharSet(Doc);

		System.out.println(TypeInfo);

		assertTrue("Content type/charset mismatch.", "text/html; charset=UTF-8".equals(TypeInfo.toString()));
	}

	public void testIncidentMessageTemplateRetrieval() throws Exception {
		final i18nLocaleList		KnownLocales = new i18nLocaleList();
		final i18nDocumentAccessor	i18nDocs     = new i18nDocumentAccessor(
			new DocumentCache(
				new MIMETypeGuesser(),
				new DocumentCensus(10, 10),
				1
			),
			new i18nLocalizationsList(KnownLocales, new HashSet<String>()),
			"jndi:/localhost/test/",
			new File("src/")
		);

		final i18nManager		i18nMgr = new i18nManager(PageJUnit.class.getClassLoader(), KnownLocales, i18nDocs, "en-us");
		final i18nServices		i18n    = i18nMgr.getServicesForClass(Qwicap.class);
		
		i18n.getXML("PageError-Incident" ).getMutable();
		i18n.getXML("PageError-Exception").getMutable();
	}
}
