/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import edu.utexas.its.eis.tools.qwicap.template.xml.mutable.MutableMarkup;
import edu.utexas.its.eis.tools.qwicap.template.xml.Markup;


/**	Describes a "prompt" operation that will, or has, caused Qwicap to block.
 * 	
 * 	@author Chris W. Johnson
 */
public class PromptDescription extends BlockingDescription {
	
	private final String		MarkupName;
	
	
	PromptDescription
	(
		final Qwicap			Q,
		final MutableMarkup		Page
	)
	{
		super(Q, BlockingOperation.PROMPT);
		MarkupName = Page.getMarkupName();
	}

/**	The original source of the markup being sent to the client by the current <code>prompt</code> invocation. If the
 * 	markup originated in a file, this returns the absolute path of the file. If the markup originated in a URL, this
 * 	returns the URL. (If you do not like the names Qwicap automatically assigns to your markup, you can change them
 * 	using the {@link Markup#setMarkupName Markup.setMarkupName} method.)
 * 
 * @return						The name of the markup being sent to the client by the current <code>prompt</code> 
 * 								invocation.
 */	
	public String getMarkupName() {
		return MarkupName;
	}
	
//	public MutableMarkup getPage() {
//
//	WARNING: This method is a bad idea, because someone will use it for pre- or post-processing of their page markup.
//	         That's the sort of thing that could be harmless, but could just as easily cause apparent malfunctions by 
// 	         undermining basic machinery in Qwicap.
//
//		return Page;
//	}
	
	@Override public String toString() {
		return "prompt(" + MarkupName + ')';
	}
}
