/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2007 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


/**	A <code>QwicapBlockingListener</code> provides a mechanism by which an application can be notified just before and 
 * 	after Qwicap blocks in one of its <code>prompt</code> or <code>redirect</code> methods. Such notifications allow 
 * 	applications, for instance, to automatically release and re-acquire resources like database connections which 
 * 	they might not want to leave tied-up while waiting for the user to supply input, or for an abandoned session to 
 * 	time-out.
 * 
 * 	<p>Invocations of {@link #qwicapBlocked qwicapBlocked} and {@link #qwicapUnblocked qwicapUnblocked} are always 
 * 	paired, and are never nested. If an invocation of <code>qwicapBlocked</code> throws an exception, the corresponding 
 * 	invocation of <code>qwicapUnblocked</code> will not be performed. The exception will not prevent Qwicap from 
 * 	blocking, because Qwicap will catch, log and discard the exception.
 * 	</p>
 * 
 * 	@author Chris W. Johnson
 */
public interface QwicapBlockingListener {

/**	Invoked just before Qwicap blocks to await a user's response to a <code>prompt</code> or <code>redirect</code> 
 * 	invocation. This method may return any object, or <code>null</code>. Whatever it returns is passed to the next 
 * 	invocation of your {@link #qwicapUnblocked qwicapUnblocked} method as the <code>YourObject</code> parameter.
 * 
 * @param BlockInfo					A description of the event that is causing blocking to occur.
 * @return							Any object, or <code>null</code>. This value will be passed to your
 * 									{@link #qwicapUnblocked qwicapUnblocked} method as the <code>YourObject</code> 
 * 									parameter the next time that method is invoked. You may use (or not use) this 
 * 									mechanism in any manner that you wish.
 */
	public Object qwicapBlocked(BlockingDescription BlockInfo);

/**	Invoked just after Qwicap unblocks, but before it returns to your application's code.
 * 
 * @param BlockInfo					A description of the event that caused blocking to occur.
 * @param YourObject				The object, or <code>null</code> that was returned by the previous invocation of 
 * 									your {@link #qwicapBlocked qwicapBlocked} method. You may use (or not use) this 
 * 									mechanism in any manner that you wish.
 */
	public void qwicapUnblocked(BlockingDescription BlockInfo, Object YourObject);
}
