/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


/**	<code>QwicapInputAbandonedException</code> is thrown when an invalid value control value is supplied. The exception 
 *	prevents the client code's parameter processing from continuing after the invalid value has been found, 
 *	enables the prompting for good input, and the restart of input processing from scratch. Logically, it 
 *	could also be named "QwicapInputRejectedException", but the current name suggests its lineage, and that the 
 *	Qwicap "prompt pattern" must be implemented in order for the exception to have its intended effect.
 *	
 *	@author Chris W. Johnson
 */
final class QwicapInputAbandonedException extends QwicapAbandonmentException {
	
	QwicapInputAbandonedException
	(
		final Page				P
	) 
	{
		super(P, true);
	}
	
	QwicapInputAbandonedException
	(
		final Page				P,
		final String			ControlName
	)
	{
		super(P, true, "Rejected the value of control(s): " + ControlName + '.');
	}
	
	@Override public void rethrowIfThisPageWasAbandoned() {
	//	We never need to rethrow.
	}
}
