/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2011 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


/**	<code>QwicapPageAbandonedException</code> is thrown when a person uses their browser's "back" button to return to, 
 *	and interact with, a previously generated web page from the current web application. This exception causes the 
 *	processing of the current page to be terminated, and, as it propagates up the application's call chain, terminates 
 *	processing of any other pages that were still in-play, until it reaches the method responsible for the page to 
 *	which the user returned. At that point, the exception's {@link #rethrowIfThisPageWasAbandoned} method ceases to 
 *	rethrow the exception, and processing of the old page resumes normally.
 *	
 *	@author Chris W. Johnson
 */
final class QwicapPageAbandonedException extends QwicapAbandonmentException {
	
	private final FilterAction	NextAction;
	
	
	QwicapPageAbandonedException
	(
		final Page				DestPage,
		final boolean			EvaluateNewInput
	)
	{
		super(DestPage, EvaluateNewInput);
		this.NextAction = null;
	}
	
	QwicapPageAbandonedException
	(
		final Page				DestPage,
		final boolean			EvaluateNewInput,
		final FilterAction		NextAction
	)
	{
		super(DestPage, EvaluateNewInput);
		this.NextAction = NextAction;
	}
	
	@Override public void rethrowIfThisPageWasAbandoned() throws QwicapAbandonmentException {
		
		super.rethrowIfThisPageWasAbandoned();
		
		if (NextAction != null) {
			
			try {
				NextAction.executeAll();
			} catch (QwicapAbandonmentException e) {
				e.rethrowIfThisPageWasAbandoned();
			}
		}
	}
}
