/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2007 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


//  QwicapSessionDeathException.java
//  qwicap
//
//  Created by Chris W. Johnson on Fri Jun 06 2003.


package edu.utexas.its.eis.tools.qwicap.servlet;


/**	<code>QwicapSessionDeathException</code> is thrown when a Qwicap application must quit. There are two common 
 *	reasons: one of the {@link Qwicap#goodbye Qwicap.goodbye} methods has been invoked, or the session has timed-out. 
 * 	The session timeout is the amount of time a web application is allowed to live without having any interaction with 
 * 	its user. When the timeout is reached, it is assumed that the user has abandoned (ceased to use) the application, 
 * 	and that the application should exit in order to release the resources it holds. Once the timeout has occurred, the 
 * 	current <code>Qwicap.prompt</code> method invocation throws this exception and the application is meant to conduct 
 * 	a graceful exit. At this point, any number of <code>Qwicap</code> methods will also throw the 
 * 	<code>QwicapSessionDeathException</code>, since they no longer have a valid function to perform, and to prevent the 
 * 	application from doing things that would impede it from exiting, like accidentally going back into a page 
 * 	processing loop. The timeout is set in the "web-app/session-config/session-timeout" tag of the web application's 
 * 	"web.xml" file.
 *	
 *	@author Chris W. Johnson
 */
public final class QwicapSessionDeathException extends RuntimeException {
	
	QwicapSessionDeathException(final String Msg) { super(Msg); }
}
