/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import edu.utexas.its.eis.tools.qwicap.Version;

import java.security.Provider;
import java.util.regex.Pattern;
import java.util.regex.Matcher;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Apr 5, 2007
 * Time: 5:07:45 PM
 * To change this template use File | Settings | File Templates.
 */
final class Rule30RandomProvider extends Provider {
	
	private static final Pattern	kVersionPattern = Pattern.compile("^([0-9]+[.][0-9]+).*");
	
	static final String				kProviderName              = "Qwicap";
	static final String				kServiceName               = "SecureRandom";
	static final String				kSecureRandomAlgorithmName = "Rule30";
	
	private final Rule30RandomService	Service;
	
	
	Rule30RandomProvider
	(
		final Rule30Random			Rndm	
	) 
	{
		super(kProviderName, getQwicapVersion(), "Rule 30 Cellular Automaton Pseudo-Random Number Generator. See <http://www.wolframscience.com/nksonline/page-317>, and also <http://www.wolframscience.com/nksonline/page-974b-text>.");
		Service = new Rule30RandomService(this, Rndm);
		putService(Service);
	}
	
	void remove() {
	
	//	This seems to made redundant by the "Security.removeProvider(Rule30RandomProvider.kProviderName)" method 
	//	invocation in the "QwicapGlobalServices.close" method, but I'd rather risk redundancy than memory leaks.
		
		removeService(Service);
	}
	
	private static double getQwicapVersion() {
		final Matcher				Match = kVersionPattern.matcher(Version.getString());
		
		if (Match.matches())
			return Double.parseDouble(Match.group(1));
		
		return 1.0;
	}
}
