/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import java.security.SecureRandomSpi;
import java.util.logging.Logger;
import java.util.logging.Level;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Apr 5, 2007
 * Time: 5:07:45 PM
 * To change this template use File | Settings | File Templates.
 */
final class Rule30RandomSPI extends SecureRandomSpi {
		
	private static final Logger Log = Logger.getLogger(Rule30RandomSPI.class.getName());
	
	private Rule30Random		Rndm;
	
	
	Rule30RandomSPI
	(
		final Rule30Random		Rndm
	)
	{
		this.Rndm = Rndm;
		Log.fine("New Rule30RandomSPI instance created.");
	}
	
	@Override protected void engineSetSeed
	(
		final byte[]			Bytes	//	Each byte has a value between 0x00 and 0xFF, inclusive.
	) 
	{
	//	The contract of this method states:
	//	
	//	    "The given seed supplements, rather than replaces, the existing seed. 
	//	     Thus, repeated calls are guaranteed never to reduce randomness."
	// 
	//	That's a problem, because there is, at present, no method of blending new bits into the Rule 30 state (the 
	//	first seed was used only as the initial state, and maybe only a part of that) in a way that ensures that 
	// 	randomness is never reduced. Furthermore, a simple blending could create new state that would produce non-
	//	random output. So, the only way to satisfy the requirement that this method never reduce randomness is to 
	//	ignore the new seed bytes.
		
		Log.info("Thanks for the new seed bytes. Because they could reduce randomness, they have been ignored.");
	}
	
	@Override protected void engineNextBytes
	(
		final byte[]			Bytes	//	Each byte will be set to a value between 0x00 and 0xFF, inclusive.
	) 
	{
		Rndm.nextBytes(Bytes);
	}
	
	@Override protected byte[] engineGenerateSeed
	(
		final int				ByteCount
	)
	{
		final int				BitCount = ByteCount << 3;
		
		Log.log(Level.FINE, "Rule30RandomSPI.engineGenerateSeed({0}) invoked.", ByteCount);
		
		return Bits.toBytes(Rule30Random.ensurePatternIsNonRepeating(Entropy.getBits(new byte[BitCount])));
	}
}