/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Oct 6, 2008
 * Time: 4:15:06 PM
 * To change this template use File | Settings | File Templates.
 */
final class Rule30WidthArbitrary extends Rule30 {

	private final int				BitCount;
	private final int				BitIndex;
	private int						BitOffset;
	private final int				UnusedFinalStateBits;

	private final long[]			State;
	private final long[]			StateLS1;
	private final long[]			StateLS2;


	Rule30WidthArbitrary
	(
		final int				Width,
		final long[]			Seed
	)
	{
		State                = new long[Seed.length];
		StateLS1             = new long[Seed.length];
		StateLS2             = new long[Seed.length];
		BitCount             = Width;
		BitIndex             = BitCount >>> 1;	//	Use the bit in the middle of the automaton's output.
		UnusedFinalStateBits = 64 - BitCount % 64;
		
		System.arraycopy(Seed, 0, State, 0, State.length);
	}
	
	@Override int getBitCount() {
		return BitCount;
	}

//	See <http://www.wolframscience.com/nksonline/page-869b-text> for a general explanation of this implementation.

	@Override Rule30 runOnce() {
		final long[]			P = State;
		final long[]			Q = rotateLeft(P, UnusedFinalStateBits, StateLS1);
		final long[]			R = rotateLeft(Q, UnusedFinalStateBits, StateLS2);

		for (int Index = 0, Count = P.length; Index < Count; Index++)
			P[Index] ^= Q[Index] | R[Index];

		if (--BitOffset < 0)
			BitOffset = BitCount - 1;

		return this;
	}

	private static long[] rotateLeft
	(
		final long[]			SrcLongs,
		final int				UnusedFinalBits,
		final long[]			DestLongs
	)
	{
		long					CarryBit = SrcLongs[0] >>> 63;
		int						Index;
		long					Long;
		long					NewCarryBit;

		if (UnusedFinalBits != 0) {

			Index       = SrcLongs.length - 1;
			Long        = SrcLongs[Index];
			NewCarryBit = Long >>> 63;

			DestLongs[Index] = (Long << 1) | (CarryBit << UnusedFinalBits);
			CarryBit         = NewCarryBit;

		} else
			Index = SrcLongs.length;

		while (--Index >= 0) {

			Long        = SrcLongs[Index];
			NewCarryBit = Long >>> 63;

			DestLongs[Index] = (Long << 1) | CarryBit;
			CarryBit         = NewCarryBit;
		}

		return DestLongs;
	}
	
	@Override int readMiddleBit() {
		return readBit(BitIndex);
	}
	
	@Override int readBit
	(
		final int				BitToRead
	)
	{
		return readBit(State, (BitOffset + BitToRead) % BitCount);
	}
}
