/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Oct 6, 2008
 * Time: 4:14:56 PM
 * To change this template use File | Settings | File Templates.
 */
final class Rule30WidthMultipleOf64 extends Rule30 {

	private final int				BitCount;
	private final int				MiddleLongIndex;
	private final int				MiddleBitShift;

	private final long[]			StateRS;
	private final long[]			State;
	private final long[]			StateLS;
	
	
	Rule30WidthMultipleOf64
	(
		final long[]			Seed
	)
	{
		State    = new long[Seed.length];
		StateRS  = new long[Seed.length];
		StateLS  = new long[Seed.length];
		BitCount = Seed.length * 64;
		
		System.arraycopy(Seed, 0, State, 0, State.length);
		
		final int				MiddleBitIndex  = BitCount >>> 1;
		
		MiddleLongIndex = MiddleBitIndex / 64;
		MiddleBitShift  = 63 - MiddleBitIndex % 64;
	}
	
	@Override int getBitCount() {
		return BitCount;
	}

//	See <http://www.wolframscience.com/nksonline/page-869b-text> for a general explanation of this implementation.

	@Override Rule30 runOnce() {
		final long[]			Q = State;
		final long[]			P = rotateRight(Q, StateRS);
		final long[]			R = rotateLeft(Q, StateLS);

		for (int Index = 0, Count = P.length; Index < Count; Index++)
			Q[Index] = P[Index] ^ (Q[Index] | R[Index]);

		return this;
	}

	private static long[] rotateLeft
	(
		final long[]			SrcLongs,
		final long[]			DestLongs
	)
	{
		long					CarryBit = SrcLongs[0] >>> 63;
		int						Index    = SrcLongs.length;
		long					Long;
		long					NewCarryBit;
		
		while (--Index >= 0) {

			Long        = SrcLongs[Index];
			NewCarryBit = Long >>> 63;

			DestLongs[Index] = (Long << 1) | CarryBit;
			CarryBit         = NewCarryBit;
		}

		return DestLongs;
	}

	private static long[] rotateRight
	(
		final long[]			SrcLongs,
		final long[]			DestLongs
	)
	{
		final int				Limit    = SrcLongs.length - 1;
		long					CarryBit = SrcLongs[Limit] & 0x1;
		long					Long;
		long					NewCarryBit;
		
		for (int Index = 0; Index <= Limit; Index++) {

			Long        = SrcLongs[Index];
			NewCarryBit = Long & 0x1;

			DestLongs[Index] = (Long >>> 1) | (CarryBit << 63);
			CarryBit         = NewCarryBit;
		}

		return DestLongs;
	}
	
	@Override int readMiddleBit() {
		return (int) (State[MiddleLongIndex] >>> MiddleBitShift) & 0x1;
	}
	
	@Override int readBit
	(
		final int				BitToRead
	)
	{
		return readBit(State, BitToRead % BitCount);
	}
}
