/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2008 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Oct 5, 2006
 * Time: 5:08:54 PM
 * To change this template use File | Settings | File Templates.
 */
final class ServiceData {
	
	private static final Runtime	Runtime = java.lang.Runtime.getRuntime();

	ServiceData						PrevData;
	
	private final long				StartMillis;
	private final long				StopMillis;

	private final IntMinMaxCount	ThreadCount         = new IntMinMaxCount();
	private final IntMinMaxCount	ThreadActiveCount   = new IntMinMaxCount();
	private final IntMinMaxCount	ThreadInactiveCount = new IntMinMaxCount();
	private final IntMinMaxCount	ThreadJammedCount   = new IntMinMaxCount();
	
	private int						ThreadUnavailableCount;
	
	private int						ThreadBirthCount;
	private int						ThreadDeathCount;
	
	private final LongMinMaxTotal	SessionTotalMillis  = new LongMinMaxTotal();
	private int						SessionExpiredCount;
	
	private final LongMinMaxTotal	HitRunMillis        = new LongMinMaxTotal();
	private int						HitBadCount;
	
	private final LongMinMaxTotal	MemoryFree          = new LongMinMaxTotal();
	private final LongMinMaxTotal	MemoryTotal         = new LongMinMaxTotal();
	

	ServiceData
	(
		final long				StartMillis,
		final long				StopMillis,
		final ServiceData		PrevData
	)
	{
		this.PrevData    = PrevData;
		this.StartMillis = StartMillis;
		this.StopMillis  = StopMillis;
	}
	
	void init
	(
		final QwicapThreadPool	QTP	
	)
	{
		if (QTP != null) {
			final int				Active   = QTP.getActiveThreadCount();
			final int				Inactive = QTP.getInactiveThreadCount();
			
			ThreadCount.set(Active + Inactive);
			ThreadActiveCount.set(Active);
			ThreadInactiveCount.set(Inactive);
			ThreadJammedCount.set(QTP.getJammedThreadCount());
			
		} else {
			
			ThreadCount.set(0);
			ThreadActiveCount.set(0);
			ThreadInactiveCount.set(0);
			ThreadJammedCount.set(0);
		}
		
		updateMemoryInfo();
	}
	
	long getDataStartTimeMillis() {
		return StartMillis;
	}
	
	long getDataStopTimeMillis() {
		return StopMillis;
	}
	
	long getDataTotalTimeMillis() {
		return StopMillis - StartMillis;
	}
	
	IntMinMaxCount getThreadCountInfo() {
		return ThreadCount;
	}
	
	IntMinMaxCount getThreadActiveCountInfo() {
		return ThreadActiveCount;
	}
	
	IntMinMaxCount getThreadInactiveCountInfo() {
		return ThreadInactiveCount;
	}
	
	int getThreadBirthCount() {
		return ThreadBirthCount;
	}
	
	int getThreadDeathCount() {
		return ThreadDeathCount;
	}
	
	IntMinMaxCount getThreadJammedCountInfo() {
		return ThreadJammedCount;
	}
	
	int getThreadUnavailableCount() {
		return ThreadUnavailableCount;
	}
	
	LongMinMaxTotal getHitRunMillisInfo() {
		return HitRunMillis;
	}
	
	int getHitCount() {
		return HitRunMillis.getSampleCount();
	}
	
	int getBadHitCount() {
		return HitBadCount;
	}
	
	LongMinMaxTotal getMemoryFreeInfo() {
		return MemoryFree;
	}
	
	LongMinMaxTotal getMemoryTotalInfo() {
		return MemoryTotal;
	}
	
	int getSessionCompletedCount() {
		return SessionTotalMillis.getSampleCount();
	}
	
	int getSessionExpiredCount() {
		return SessionExpiredCount;
	}
	
	LongMinMaxTotal getSessionTotalMillisInfo() {
		return SessionTotalMillis;
	}
	
	void threadUnavailable() {
		
		ThreadUnavailableCount++;
		updateMemoryInfo();
	}

	void threadBorn
	(
		final int				ActiveThreads,
		final int				InactiveThreads
	) 
	{
		ThreadBirthCount++;
		ThreadCount.set(ActiveThreads + InactiveThreads);
		ThreadActiveCount.set(ActiveThreads);
		ThreadInactiveCount.set(InactiveThreads);
		
		updateMemoryInfo();
	}

	void threadDead
	(
		final int				ActiveThreads,
		final int				InactiveThreads
	)
	{
		ThreadDeathCount++;
		ThreadCount.set(ActiveThreads + InactiveThreads);
		ThreadActiveCount.set(ActiveThreads);
		ThreadInactiveCount.set(InactiveThreads);
		
		updateMemoryInfo();
	}

	void threadActive
	(
		final int				ActiveThreads,
		final int				InactiveThreads
	)
	{
		ThreadActiveCount.set(ActiveThreads);
		ThreadInactiveCount.set(InactiveThreads);
		ThreadCount.set(ActiveThreads + InactiveThreads);
		
		updateMemoryInfo();
	}

	void threadInactive
	(
		final int				ActiveThreads,
		final int				InactiveThreads,
		final long				SessionMillis,
		final boolean			Expired
	)
	{
		SessionTotalMillis.add(SessionMillis);
		
		if (Expired)
			SessionExpiredCount++;
		
		ThreadActiveCount.set(ActiveThreads);
		ThreadInactiveCount.set(InactiveThreads);
		
		updateMemoryInfo();
	}
	
	void threadJammed() {
		
		ThreadJammedCount.increment();
		updateMemoryInfo();
	}
	
	void hit
	(
		final long				StartMillis
	)
	{
		updateMemoryInfo();
		HitRunMillis.add(System.currentTimeMillis() - StartMillis);
	}
	
	void badDataSet() {
		
		HitBadCount++;
		updateMemoryInfo();
	}
		
	private void updateMemoryInfo() {
		final java.lang.Runtime	Runtime = ServiceData.Runtime;
		
		MemoryFree.add(Runtime.freeMemory());
		MemoryTotal.add(Runtime.totalMemory());
	}
}
