/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2007-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import edu.utexas.ots.chrisj.util.Hexadecimal;

import java.util.Random;


/**	Base class for all immutable classes representing unique ID numbers.
 * 
 * 	@author Chris W. Johnson
 */
abstract class UniqueID {

	private static final int	kUniqueIDBytes = 8;

	private final byte[]		IDBytes;
	private final String		IDStr;
	private final int			HashCode;


	UniqueID
	(
		final Random			Rndm		
	)
	{
		final byte[]			Bytes = new byte[kUniqueIDBytes];

		Rndm.nextBytes(Bytes);

		IDBytes  = Bytes;
		IDStr    = Hexadecimal.fromBytes(Bytes);
		HashCode = (Bytes[0] & 0xFF) << 24 | (Bytes[1] & 0xFF) << 16 | (Bytes[2] & 0xFF) << 8 | Bytes[3] & 0xFF;
	}

	UniqueID
	(
		final String			IDStr
	)
	{
		final byte[]			Bytes = Hexadecimal.toBytes(IDStr);
		
		this.IDBytes  = Bytes;
		this.IDStr    = IDStr;
		this.HashCode = (Bytes[0] & 0xFF) << 24 | (Bytes[1] & 0xFF) << 16 | (Bytes[2] & 0xFF) << 8 | Bytes[3] & 0xFF;
	}

	@Override public int hashCode() {
		return HashCode;
	}

	@Override public boolean equals
	(
		final Object			Other
	)
	{
		if (Other instanceof UniqueID) {
			final byte[]			ID1 = IDBytes;
			final byte[]			ID2 = ((UniqueID) Other).IDBytes;

			for (int Index = 0, Count = kUniqueIDBytes; Index < Count; Index++) {

				if (ID1[Index] != ID2[Index])
					return false;
			}

			return true;
		}

		return IDStr.equals(Other);
	}

	@Override public String toString() {
		return IDStr;
	}
}
