/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2007 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import edu.utexas.its.eis.tools.table.lltable.MutableLLTable;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.logging.Level;
import java.util.logging.Logger;


/**	A list of all <code>Locale</code> instances available within this JVM, as obtained by invoking 
 * 	<code>Locale.getAvailableLocales</code>. Getting the complete list of locales is time-consuming, so retaining an
 * 	instance of this class can save a lot of time.
 * 
 * 	@author Chris W. Johnson
 */
final class i18nLocaleList {

	private static final Logger				Log = Logger.getLogger(i18nLocaleList.class.getName());

	private final HashMap<String, HTTPLanguageTag> KnownHTTPLangs = new HashMap<String, HTTPLanguageTag>(150);
	private final HashMap<HTTPLanguageTag, Locale> KnownLocales   = new HashMap<HTTPLanguageTag, Locale>(150);


	i18nLocaleList() {
		
		Log.fine("Initializing list of known locales. This may take a while....");
		
		final Locale[]				Locales = Locale.getAvailableLocales();
		
		for (final Locale L : Locales) {
			final HTTPLanguageRange	HTTPLang = new HTTPLanguageRange(L);
			
			KnownHTTPLangs.put(HTTPLang.toString(), HTTPLang);
			KnownLocales.put(HTTPLang, L);
		}
		
		Log.log(Level.INFO, "List of known locales initialized. {0} locales found.", KnownLocales.size());
		
		if (Log.isLoggable(Level.FINEST))
			Log.log(Level.FINEST, "List of known locales:\n{0}", new MutableLLTable(KnownLocales).sortRowsAscending());
	}
	
	int size() {
		return KnownLocales.size();	
	}
	
	HTTPLanguageTag getKnownHTTPLanguageTag
	(
		final String			PotentialLocalizationName
	)
	{
		return KnownHTTPLangs.get(HTTPLanguageTag.normalize(PotentialLocalizationName));
	}
	
	Locale getLocale
	(
		final HTTPLanguageTag	Lang
	)
	{
		return KnownLocales.get(Lang);
	}
	
	List<Locale> getMatchingLocales
	(
		final HTTPLanguageRange	LangRange
	)
	{
		final ArrayList<Locale>	List = new ArrayList<Locale>();
		
		for (final HTTPLanguageTag LocTag : KnownLocales.keySet()) {
			
			if (LangRange.matches(LocTag))
				List.add(KnownLocales.get(LocTag));
		}
		
		return List;
	}
	
	List<HTTPLanguageTag> getMatchingTags
	(
		final HTTPLanguageRange	LangRange
	)
	{
		final ArrayList<HTTPLanguageTag>	List = new ArrayList<HTTPLanguageTag>();
		
		for (final HTTPLanguageTag LocTag : KnownLocales.keySet()) {
			
			if (LangRange.matches(LocTag))
				List.add(LocTag);
		}
		
		return List;
	}
}


/*	Just FYI, the following are all of the known locales in my installation of Java 1.5 SE:

	ar
	ar_AE
	ar_BH
	ar_DZ
	ar_EG
	ar_IQ
	ar_JO
	ar_KW
	ar_LB
	ar_LY
	ar_MA
	ar_OM
	ar_QA
	ar_SA
	ar_SD
	ar_SY
	ar_TN
	ar_YE
	be
	be_BY
	bg
	bg_BG
	ca
	ca_ES
	cs
	cs_CZ
	da
	da_DK
	de
	de_AT
	de_CH
	de_DE
	de_LU
	el
	el_GR
	en
	en_AU
	en_CA
	en_GB
	en_IE
	en_IN
	en_NZ
	en_US
	en_ZA
	es
	es_AR
	es_BO
	es_CL
	es_CO
	es_CR
	es_DO
	es_EC
	es_ES
	es_GT
	es_HN
	es_MX
	es_NI
	es_PA
	es_PE
	es_PR
	es_PY
	es_SV
	es_UY
	es_VE
	et
	et_EE
	fi
	fi_FI
	fr
	fr_BE
	fr_CA
	fr_CH
	fr_FR
	fr_LU
	hi_IN
	hr
	hr_HR
	hu
	hu_HU
	is
	is_IS
	it
	it_CH
	it_IT
	iw
	iw_IL
	ja
	ja_JP
	ko
	ko_KR
	lt
	lt_LT
	lv
	lv_LV
	mk
	mk_MK
	nl
	nl_BE
	nl_NL
	no
	no_NO
	no_NO_NY
	pl
	pl_PL
	pt
	pt_BR
	pt_PT
	ro
	ro_RO
	ru
	ru_RU
	sk
	sk_SK
	sl
	sl_SI
	sq
	sq_AL
	sv
	sv_SE
	th
	th_TH
	th_TH_TH
	tr
	tr_TR
	uk
	uk_UA
	vi
	vi_VN
	zh
	zh_CN
	zh_HK
	zh_TW
*/