/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2007 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import edu.utexas.its.eis.tools.qwicap.util.ArrayToString;
import junit.framework.TestCase;

import java.util.LinkedHashSet;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Oct 24, 2007
 * Time: 5:07:25 PM
 * To change this template use File | Settings | File Templates.
 */
public class i18nLocaleSettingsJUnit extends TestCase {

	private final i18nLocaleList	KnownLocales = new i18nLocaleList();
	
	
	public void test() {
		final LinkedHashSet<String>	TopLevelRsrcs = new LinkedHashSet<String>();
		
		TopLevelRsrcs.add("/de/");
		TopLevelRsrcs.add("/en/");
		TopLevelRsrcs.add("/en-us/");
		TopLevelRsrcs.add("/en-gb/");
		TopLevelRsrcs.add("/es/");
		TopLevelRsrcs.add("/META-INF/");
		TopLevelRsrcs.add("/number-guess.css");
		TopLevelRsrcs.add("/number-guess.html");
		TopLevelRsrcs.add("/WEB-INF/");

		final i18nLocalizationsList	TransList = new i18nLocalizationsList(KnownLocales, TopLevelRsrcs);
		
		performTest(TransList, "es-mx,de-at;q=0.8,en-gb;q=0.7,en-us;q=0.5,en;q=0.3,ar;q=0.2", "es-mx", "es", "es");

		performTest(TransList, "en, en-us;q=0.9, es;q=0.3, ar;q=0.2", "en",    "en",    "en");
		performTest(TransList, "en-us, en;q=0.9, es;q=0.3, ar;q=0.2", "en-us", "en-us", "en-us", "en");
		performTest(TransList, "en-au, es;q=0.3, ar;q=0.2",           "en-au", "en",    "en");
	}

	private void performTest
	(
		final i18nLocalizationsList	Translations,
		final String				AcceptLanguage,
		final String				ExpectedLocale,
		final String				ExpectedTranslation,
		final String...				ExpectedTranslationDirs
	)
	{
		final HTTPAcceptLanguageList	AccLangs = new HTTPAcceptLanguageList(AcceptLanguage);
		final i18nLocaleSettings		Settings = i18nLocaleSettings.create(KnownLocales, Translations, AccLangs);
		
		System.out.println("Accept-Language: " + AccLangs);
		
		System.out.println("         Locale: " + Settings.getLocale());
		assertTrue(KnownLocales.getLocale(new HTTPLanguageTag(ExpectedLocale)).equals(Settings.getLocale()));

		System.out.println("     Locale Tag: " + Settings.getLocaleAsLanguageTag());
		assertTrue(ExpectedLocale.equals(Settings.getLocaleAsLanguageTag().toString()));

		System.out.println("    Translation: " + Settings.getWebAppTranslation());
		assertTrue(ExpectedTranslation.equals(Settings.getWebAppTranslation().toString()));

		final String[]					SearchDirs = Settings.getWebAppTranslationSearchDirs();

		System.out.println("    Search Dirs: " + new ArrayToString(SearchDirs));
		
		if (SearchDirs.length != ExpectedTranslationDirs.length)
			fail();
		
		for (int Index = 0, Length = SearchDirs.length; Index < Length; Index++)
			assertTrue(ExpectedTranslationDirs[Index].equals(SearchDirs[Index]));

		System.out.println();
	}
	
	public void testSearchDirWithDefault() {
		final LinkedHashSet<String>	TopLevelRsrcs = new LinkedHashSet<String>();
		
		TopLevelRsrcs.add("/de/");
		TopLevelRsrcs.add("/en/");
		TopLevelRsrcs.add("/en-us (default)/");
		TopLevelRsrcs.add("/en-gb/");
		TopLevelRsrcs.add("/es/");
		TopLevelRsrcs.add("/es-mx/");
		TopLevelRsrcs.add("/META-INF/");
		TopLevelRsrcs.add("/number-guess.css");
		TopLevelRsrcs.add("/number-guess.html");
		TopLevelRsrcs.add("/WEB-INF/");

		final i18nLocalizationsList		TransList  = new i18nLocalizationsList(KnownLocales, TopLevelRsrcs);
		final HTTPAcceptLanguageList	AccLangs   = new HTTPAcceptLanguageList("es-mx");
		final String[]					SearchDirs = i18nLocaleSettings.getSearchDirs(TransList, AccLangs.getBestLanguageTag(TransList));

		System.out.println(new ArrayToString(SearchDirs).alwaysQuote());
		
		assertTrue(SearchDirs.length == 4);
		assertTrue("es-mx".equals(SearchDirs[0]));
		assertTrue("es".equals(SearchDirs[1]));
		assertTrue("en-us (default)".equals(SearchDirs[2]));
		assertTrue("en".equals(SearchDirs[3]));
	}
	
	public void testSearchDirWithDefault2() {
		final LinkedHashSet<String>	TopLevelRsrcs = new LinkedHashSet<String>();
		
		TopLevelRsrcs.add("/de/");
		TopLevelRsrcs.add("/en/");
		TopLevelRsrcs.add("/en-us (default)/");
		TopLevelRsrcs.add("/en-gb/");
		TopLevelRsrcs.add("/es/");
		TopLevelRsrcs.add("/es-mx/");
		TopLevelRsrcs.add("/META-INF/");
		TopLevelRsrcs.add("/number-guess.css");
		TopLevelRsrcs.add("/number-guess.html");
		TopLevelRsrcs.add("/WEB-INF/");

		final i18nLocalizationsList		TransList  = new i18nLocalizationsList(KnownLocales, TopLevelRsrcs);
		final HTTPAcceptLanguageList	AccLangs   = new HTTPAcceptLanguageList("en-us");
		final String[]					SearchDirs = i18nLocaleSettings.getSearchDirs(TransList, AccLangs.getBestLanguageTag(TransList));

		System.out.println(new ArrayToString(SearchDirs).alwaysQuote());
		
		assertTrue(SearchDirs.length == 2);
		assertTrue("en-us (default)".equals(SearchDirs[0]));
		assertTrue("en".equals(SearchDirs[1]));
	}
	
	public void testSearchDirWithDefault3() {
		final LinkedHashSet<String>	TopLevelRsrcs = new LinkedHashSet<String>();
		
		TopLevelRsrcs.add("/de/");
		TopLevelRsrcs.add("/en/");
		TopLevelRsrcs.add("/en-us (default)/");
		TopLevelRsrcs.add("/en-gb/");
		TopLevelRsrcs.add("/es/");
		TopLevelRsrcs.add("/es-mx/");
		TopLevelRsrcs.add("/META-INF/");
		TopLevelRsrcs.add("/number-guess.css");
		TopLevelRsrcs.add("/number-guess.html");
		TopLevelRsrcs.add("/WEB-INF/");

		final i18nLocalizationsList		TransList  = new i18nLocalizationsList(KnownLocales, TopLevelRsrcs);
		final HTTPAcceptLanguageList	AccLangs   = new HTTPAcceptLanguageList("en");
		final String[]					SearchDirs = i18nLocaleSettings.getSearchDirs(TransList, AccLangs.getBestLanguageTag(TransList));

		System.out.println(new ArrayToString(SearchDirs).alwaysQuote());
		
		assertTrue(SearchDirs.length == 2);
		assertTrue("en".equals(SearchDirs[0]));
		assertTrue("en-us (default)".equals(SearchDirs[1]));
	}

	public void testSearchDirsWithoutDefault() {
		final LinkedHashSet<String>	TopLevelRsrcs = new LinkedHashSet<String>();
		
		TopLevelRsrcs.add("/de/");
		TopLevelRsrcs.add("/en/");
		TopLevelRsrcs.add("/en-us/");
		TopLevelRsrcs.add("/en-gb/");
		TopLevelRsrcs.add("/es/");
		TopLevelRsrcs.add("/es-mx/");
		TopLevelRsrcs.add("/META-INF/");
		TopLevelRsrcs.add("/number-guess.css");
		TopLevelRsrcs.add("/number-guess.html");
		TopLevelRsrcs.add("/WEB-INF/");

		final i18nLocalizationsList		TransList  = new i18nLocalizationsList(KnownLocales, TopLevelRsrcs);
		final HTTPAcceptLanguageList	AccLangs   = new HTTPAcceptLanguageList("es-mx");
		final String[]					SearchDirs = i18nLocaleSettings.getSearchDirs(TransList, AccLangs.getBestLanguageTag(TransList));

		System.out.println(new ArrayToString(SearchDirs).alwaysQuote());
		
		assertTrue(SearchDirs.length == 2);
		assertTrue("es-mx".equals(SearchDirs[0]));
		assertTrue("es".equals(SearchDirs[1]));
	}
	
}
