/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2007-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import edu.utexas.its.eis.tools.qwicap.template.xml.immutable.XMLString;
import edu.utexas.its.eis.tools.qwicap.template.xml.immutable.XMLDocument;
import edu.utexas.its.eis.tools.qwicap.template.xml.TagException;

import java.util.Locale;
import java.util.Date;
import java.util.Calendar;
import java.util.MissingResourceException;
import java.math.BigInteger;
import java.math.BigDecimal;
import java.text.ParseException;
import java.io.IOException;
import java.net.URL;
import java.sql.Time;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Sep 16, 2007
 * Time: 8:16:10 PM
 * To change this template use File | Settings | File Templates.
 */
final class i18nProxy implements i18nServices {
	
	private final i18nManager				Mgr;
	private final Class<?>					C;
	private i18nXMLPropertiesLocaleChain	Props;
	private MissingResourceException		PropsException;
	
	
	i18nProxy
	(
		final i18nManager					Mgr,
		final Class<?>						C,
		final i18nXMLPropertiesLocaleChain	Props,	//	Can be null.
		final MissingResourceException		PropsException
	)	
	{
		this.Mgr            = Mgr;
		this.C              = C;
		this.Props          = Props;
		this.PropsException = PropsException;
	}
	
	Class<?> getProxyClass() {
		return C;	
	}
	
	void setProxyBundle
	(
		final i18nXMLPropertiesLocaleChain	Props
	)
	{
		this.Props = Props;
	}
	
	@Override public Locale getLocale() {
		return Mgr.getLocale();	
	}

	@Override public String getHTTPLanguageString() {
		return Mgr.getHTTPLanguageTag().toString();	
	}
	
	@Override public BigInteger parseInteger
	(
		final String			IntStr
	)
	throws ParseException, ArithmeticException	//	ArithmeticException is thrown when there's a fractional component to this would-be intereger quantity.
	{
		return Mgr.parseInteger(IntStr);
	}
	
	@Override public BigDecimal parseDecimal
	(
		final String			DecStr
	)
	throws ParseException
	{
		return Mgr.parseDecimal(DecStr);
	}
	
	@Override public String formatNumber
	(
		final Number			No
	)
	{
		return Mgr.formatNumber(No);
	}
	
	@Override public String formatInteger
	(
		final Number			No
	)
	{
		return Mgr.formatInteger(No);
	}
	
	@Override public String formatDecimal
	(
		final Number			No
	)
	{
		return Mgr.formatDecimal(No);
	}
	
	@Override public Date parseDateTime
	(
		final String			Text
	)
	throws ParseException
	{
		return Mgr.parseDateTime(Text);
	}
	
	@Override public java.sql.Date parseDate
	(
		final String			Text
	)
	throws ParseException
	{
		return Mgr.parseDate(Text);
	}
	
	@Override public Time parseTime
	(
		final String			Text
	)
	throws ParseException
	{
		return Mgr.parseTime(Text);
	}
	
	@Override public String formatDateTime
	(
		final long				DTMillis
	)
	{
		return Mgr.formatDateTime(DTMillis);
	}
	
	@Override public String formatDateTime
	(
		final Date				DTMillis
	)
	{
		return Mgr.formatDateTime(DTMillis);
	}
	
	@Override public String formatDateTime
	(
		final Calendar DTMillis
	)
	{
		return Mgr.formatDateTime(DTMillis);
	}
	
	@Override public String formatDate
	(
		final long				DTMillis
	)
	{
		return Mgr.formatDate(DTMillis);
	}
	
	@Override public String formatDate
	(
		final Date				DTMillis
	)
	{
		return Mgr.formatDate(DTMillis);
	}
	
	@Override public String formatDate
	(
		final Calendar			DTMillis
	)
	{
		return Mgr.formatDate(DTMillis);
	}
	
	@Override public String formatTime
	(
		final long				DTMillis
	)
	{
		return Mgr.formatTime(DTMillis);
	}
	
	@Override public String formatTime
	(
		final Date				DTMillis
	)
	{
		return Mgr.formatTime(DTMillis);
	}
	
	@Override public String formatTime
	(
		final Calendar			DTMillis
	)
	{
		return Mgr.formatTime(DTMillis);
	}
	
	@Override public XMLDocument getDocument
	(
		final String			FileName
	) 
	throws TagException, IOException
	{
		return Mgr.getXMLDocument(FileName);
	}
	
	@Override public XMLDocument getDocument
	(
		final URL				DocURL
	) 
	throws TagException, IOException
	{
		return Mgr.getXMLDocument(DocURL);
	}
	
	@Override public String getString
	(
		final String			Key
	)
	{
		if (PropsException != null)
			throw PropsException;
		
		return Props.getString(Key);
	}

	@Override public String getMessageString
	(
		final String			Key,
		final Object...			Values
	)
	{
		if (PropsException != null)
			throw PropsException;
		
		return Props.getMessageString(Key, Values);
	}
	
	@Override public XMLString getMessageXML
	(
		final String			Key,
		final Object...			Values
	)
	throws TagException
	{
		if (PropsException != null)
			throw PropsException;
		
		return Props.getMessageXML(Key, Values);
	}
	
	@Override public XMLString getXML
	(
		final String			Key
	)
	throws TagException
	{
		if (PropsException != null)
			throw PropsException;
		
		return Props.getXML(Key);
	}
}
