/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2007 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import edu.utexas.its.eis.tools.qwicap.template.css.Results;
import edu.utexas.its.eis.tools.qwicap.template.xml.Markup;
import edu.utexas.its.eis.tools.qwicap.template.xml.Match;

import java.util.HashMap;
import java.util.logging.Logger;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Sep 17, 2007
 * Time: 1:53:52 AM
 * To change this template use File | Settings | File Templates.
 */
final class i18nXMLProperties {
	
	private static final Logger				Log = Logger.getLogger(i18nXMLProperties.class.getName());
	
	private final String					PropsName;
	private final HashMap<String, String>	Props;
	
	
//	static i18nXMLProperties get
//	(
//		final URL				DocURL
//	)
//	throws TagException, IOException 
//	{
//		if (DocURL == null)
//			return null;
//		
//		final Markup			Doc = XMLCache.getDocument(DocURL);
//		
//		if (Doc == null)
//			return null;
//		
//		return new i18nXMLProperties(Doc);
//	}
	
	i18nXMLProperties
	(
		final Markup			Doc
	)
	{
		this.PropsName = Doc.getMarkupName();
		
	//	Load the properties from the XML file.

		final Results					Entries    = Doc.get("properties entry");
		final int						EntryCount = Entries.size();
		final HashMap<String, String>	Props      = new HashMap<String, String>(EntryCount);
		
		for (final Match Entry : Entries) {
			final String			Key   = Entry.getAttribute("key");

			if (Key != null) {
				final Results			Content = Entry.getContent();
				final Results			CDATAs  = Content.getCDATA();
				final String			Value;
			
				if (CDATAs.notEmpty())
					Value = CDATAs.getContent().toString();
				else
					Value = Content.toString();
			
				Props.put(Key, Value);
			
			} else
				Log.warning("In the XML properties file \"" + PropsName + "\", the \"key\" attribute is not present in the entry: " + Entry);
		}

		this.Props = Props;
	}
	
	String getName() {
		return PropsName;	
	}
	
	String getString
	(
		final String			Key	
	)
	{
		return Props.get(Key);	//	Return null, if the key is not found.
	}
}
