/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2007 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import edu.utexas.its.eis.tools.qwicap.template.xml.TagException;

import java.io.IOException;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Sep 17, 2007
 * Time: 2:51:31 AM
 * To change this template use File | Settings | File Templates.
 */
final class i18nXMLPropertiesClassLoaderChain {
	
	private static final Logger					Log = Logger.getLogger(i18nXMLPropertiesClassLoaderChain.class.getName());
	
	private final i18nXMLPropertiesWithShadow[]	Chain;
	private final String						ResourceName;
	
	
	i18nXMLPropertiesClassLoaderChain
	(
		final DocumentCache		Docs,
		final ClassLoader		FirstClassLoader,
		final String			ResourceName
	)
	{
		final ArrayList<i18nXMLPropertiesWithShadow>	List = new ArrayList<i18nXMLPropertiesWithShadow>();
		
		try {
			ClassLoader				CLdr = FirstClassLoader;
			
			while (CLdr != null) {
				
				try {
					List.add(new i18nXMLPropertiesWithShadow(Docs, CLdr, ResourceName));
				} catch (IOException e) {
				//	Resource does not exist in this class loader.
				} catch (TagException e) {
					Log.log(Level.SEVERE, "Resource \"" + ResourceName + "\" could not be loaded, because there was an error in its XML.", e);
				}
				
				CLdr = CLdr.getParent();
			}
		
		} catch (SecurityException e) {
		//	Discard. In a security-managed environment, we expect this sort of thing when we go exploring.
		}
				
		this.Chain        = List.toArray(new i18nXMLPropertiesWithShadow[List.size()]);
		this.ResourceName = ResourceName;
	}
	
	String getString
	(
		final String			Key
	)
	{
		for (final i18nXMLPropertiesWithShadow Prop : Chain) {
			final String			Value = Prop.getString(Key);
			
			if (Value != null)
				return Value;
		}
		
		return null;
	}
	
	boolean isNotEmpty() {
		return Chain.length != 0;	
	}
	
	String getResourceName() {
		return ResourceName;	
	}
}
