/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2007 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import edu.utexas.its.eis.tools.qwicap.template.xml.TagException;
import edu.utexas.its.eis.tools.qwicap.template.xml.immutable.XMLString;
import edu.utexas.its.eis.tools.qwicap.util.Null;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Locale;
import java.util.MissingResourceException;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Sep 17, 2007
 * Time: 3:05:44 AM
 * To change this template use File | Settings | File Templates.
 */
final class i18nXMLPropertiesLocaleChain {
	
	private final i18nManager							i18n;
	private final i18nXMLPropertiesClassLoaderChain[]	Chain;
	private final String								BaseResourceName;


	static i18nXMLPropertiesLocaleChain get
	(
		final i18nManager		i18n,
		final ClassLoader		FirstClassLoader,
		final Class<?>			BaseClass,
		final Locale			Location
	)
	throws MissingResourceException
	{
		final String						BaseResourceName = BaseClass.getName().replace('.', '/');
		final i18nXMLPropertiesLocaleChain	Chain1 = new i18nXMLPropertiesLocaleChain(i18n, FirstClassLoader, BaseResourceName, Location);
		
		if (Chain1.containsMoreThanDefault())
			return Chain1;
		
		final i18nXMLPropertiesLocaleChain	Chain2 = new i18nXMLPropertiesLocaleChain(i18n, FirstClassLoader, BaseResourceName, Locale.getDefault());
		
		if (Chain2.containsMoreThanDefault())
			return Chain2;

	//	We only look at Chain1 below, because the default bundle for Chain1 and Chain2 is exactly the same bundle, so 
	//	they either both have it, or they both don't.
		
		if (Chain1.empty())
			throw new MissingResourceException("No XML properties file could be found for base name \"" + BaseResourceName + "\", location \"" + Location + "\", and class loader \"" + FirstClassLoader.getClass().getName() + "\".", "", "");
		
		return Chain1;
	}

	private i18nXMLPropertiesLocaleChain
	(
		final i18nManager		i18n,
		final ClassLoader		FirstClassLoader,
		final String			BaseResourceName,	//	Lacks the postfix ".properties.xml"
		final Locale			Location
	)
	{
		final ArrayList<i18nXMLPropertiesClassLoaderChain>	List = new ArrayList<i18nXMLPropertiesClassLoaderChain>();

		final DocumentCache					Docs     = i18n.getDocumentCache();
		final String						Language = Location.getLanguage();
		final String						Country  = Location.getCountry();
		final String						Variant  = Location.getVariant();
		i18nXMLPropertiesClassLoaderChain	Properties;

		if (!Null.orEmpty(Variant)) {

			try {
				Properties = new i18nXMLPropertiesClassLoaderChain(Docs, FirstClassLoader, BaseResourceName + '_' + Language + '_' + Country + '_' + Variant + ".properties.xml");
				if (Properties.isNotEmpty())
					List.add(Properties);
			} catch (Exception e) {
			//	Discard exception.
			}
		}

		if (!Null.orEmpty(Country)) {

			try {
				Properties = new i18nXMLPropertiesClassLoaderChain(Docs, FirstClassLoader, BaseResourceName + '_' + Language + '_' + Country + ".properties.xml");
				if (Properties.isNotEmpty())
					List.add(Properties);
			} catch (Exception e) {
			//	Discard exception.
			}
		}

		if (!Null.orEmpty(Language)) {

			try {
				Properties = new i18nXMLPropertiesClassLoaderChain(Docs, FirstClassLoader, BaseResourceName + '_' + Language + ".properties.xml");
				if (Properties.isNotEmpty())
					List.add(Properties);
			} catch (Exception e) {
			//	Discard exception.
			}
		}

		try {
			Properties = new i18nXMLPropertiesClassLoaderChain(Docs, FirstClassLoader, BaseResourceName + ".properties.xml");
			if (Properties.isNotEmpty())
				List.add(Properties);
		} catch (Exception e) {
		//	Discard exception.
		}

		this.i18n             = i18n;
		this.Chain            = List.toArray(new i18nXMLPropertiesClassLoaderChain[List.size()]);
		this.BaseResourceName = BaseResourceName;
	}

	boolean containsMoreThanDefault() {
		return Chain.length > 1;
	}

	boolean empty() {
		return Chain.length == 0;
	}

	public String getString
	(
		final String			Key
	)
	{
		for (final i18nXMLPropertiesClassLoaderChain Props : Chain) {
			final String			Value = Props.getString(Key);

			if (Value != null)
				return Value;
		}

		return null;
	}

	public String getMessageString
	(
		final String			Key,
		final Object...			Values
	)
	{
		final String			RawStr = getString(Key);

		if (RawStr == null)
			throw new MissingResourceException("The key \"" + Key + "\" was not found in the properties locale chain \"" + BaseResourceName + "\".", BaseResourceName, Key);

		final MessageFormat 	MsgFmt    = new MessageFormat(RawStr, i18n.getLocale());
		final int				RawStrLen = RawStr.length();
		final StringBuffer		Buff      = new StringBuffer(RawStrLen + (RawStrLen >> 1));

		return MsgFmt.format(Values, Buff, null).toString();
	}

	public XMLString getMessageXML
	(
		final String			Key,
		final Object...			Values
	)
	throws TagException {
		return new XMLString(getMessageString(Key, Values));
	}

	public XMLString getXML
	(
		final String			Key
	)
	throws TagException
	{
		return new XMLString(getString(Key));
	}
}
