/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2007 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.servlet;


import edu.utexas.its.eis.tools.qwicap.template.xml.TagException;

import java.net.URL;
import java.io.IOException;
import java.util.logging.Logger;
import java.util.logging.Level;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Sep 17, 2007
 * Time: 2:06:29 AM
 * To change this template use File | Settings | File Templates.
 */
final class i18nXMLPropertiesWithShadow {
	
	private static final Logger		Log = Logger.getLogger(i18nXMLPropertiesWithShadow.class.getName());
	
	private final i18nXMLProperties	Props1;		//	A resource from, potentially, anywhere in the web application.
	private final i18nXMLProperties	Props2;		//	A resource with the same name, from the Qwicap library, or null.
	
	
	i18nXMLPropertiesWithShadow
	(
		final DocumentCache		Docs,
		final ClassLoader		CLdr,
		final String			ResourceName
	)
	throws TagException, IOException 
	{
		final URL				Props1URL = CLdr.getResource(ResourceName);
		
		if (Props1URL == null)
			throw new IOException("Resource \"" + ResourceName + "\" is not known to class loader " + CLdr + '.');
		
		{
			final DocumentSource	DocSrc = Docs.getDocumentSource(Props1URL);
			
			if (DocSrc.exists())
				Props1 = new i18nXMLProperties(DocSrc.getXML());
			else
				Props1 = null;
		}
		
	//	Props1 = i18nXMLProperties.get(Props1URL);
		Props2 = getShadowedResource(Docs, CLdr, ResourceName, Props1URL);
		
		if (Props1 == null && Props2 == null)
			throw new IOException("Resource \"" + ResourceName + "\" is known to class loader " + CLdr + ", but could not be loaded.");
	}
	
	String getString
	(
		final String			Key
	)
	{
		String					Value = Props1.getString(Key);
		
		if (Value == null && Props2 != null)
			Value = Props2.getString(Key);
		
		return Value;
	}
	
	private static i18nXMLProperties getShadowedResource
	(
		final DocumentCache		Docs,
		final ClassLoader		CLdr,
		final String			ResourceName,
		final URL				Rsrc1URL
	)
	throws TagException
	{
		final String			Rsrc1URLStr = Rsrc1URL.toString();
		
	//	If Rsrc1URL references a resource somwhere in a web application's "WEB-INF/classes" subdirectory. there's a 
	//	possibility that it is hiding a resource of the same name in the Qwicap library, which will be located in a  
	//	"JAR" file, somewhere in the "WEB-INF/lib" subdirectory. Here, we check for exactly that case, and return 
	//	a properties list based on that hidden (shadowed) resource, if it exists.
		
		if (Rsrc1URLStr.contains("/WEB-INF/classes/")) {
			final String			QwicapLibURLRoot = getQwicapLibraryURLRoot(CLdr);
			final String			ResourceURLStr   = QwicapLibURLRoot + ResourceName;
				
			try {
				final DocumentSource	DocSrc = Docs.getDocumentSource(new URL(ResourceURLStr));
				
				return DocSrc.exists() ? new i18nXMLProperties(DocSrc.getXML()) : null;
			//	return i18nXMLProperties.get(new URL(ResourceURLStr));
			} catch (IOException e) {
				Log.log(Level.INFO, "Optional, shadowed resource \"" + ResourceURLStr + "\" not found.", e);
			}
		}
		
		return null;
	}
	
	private static String getQwicapLibraryURLRoot
	(
		final ClassLoader		CLdr	
	) 
	{
	//	Example URLs. First, the URL for a resource in the "WEB-INF/classes" directory of a web application. Second,
	//	the URL for a resource in a JAR file in the "WEB-INF/lib" directory of a web application.
	//	
	//	file:/usr/local/tomcat/webapps/guess/WEB-INF/classes/edu/utexas/its/eis/tools/qwicap/servlet/Qwicap.properties.xml
	//	
	//	jar:file:/usr/local/tomcat/webapps/guess/WEB-INF/lib/qwicap-1.4b11.jar!/edu/utexas/its/eis/tools/qwicap/servlet/Qwicap.properties.xml
			
		final URL				QwicapURL = CLdr.getResource(Qwicap.class.getName().replace('.', '/') + ".class");
		
	//	If the Qwicap library is not loaded by this class loader, there's nothing more to do.
		
		if (QwicapURL == null)
			return null;
		
		final String			URLStr      = QwicapURL.toString();
		final int				LastBangOff = URLStr.lastIndexOf("!/");
		
		if (LastBangOff >= 0)
			return URLStr.substring(0, LastBangOff + 2);
		
		return null;
	}
}
