/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2008 The University of Texas at Austin ("U.T. Austin"). All rights reserved.

	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert
	copyright over changes contributed by others.

	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
	any later version.

	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
	more details.

	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


//	qwicap.js - Revision 79


(function() {
	
//	If the window already has an "onload" function, we want to retain a reference to it, so we can call it from
//	the "onload" handler we're about to install.
	
	var OriginalOnloadFunction = (window.onload != undefined) ? window.onload : null;
	
//	If the window already has an "onunload" function, we want to retain a reference to it, so we can call it from
//	the "onunload" handler we're about to install.
	
	var OriginalOnunloadFunction = (window.onunload != undefined) ? window.onunload : null;
	
//	Install our "onload" event-handler function.
	
	window.onload = pageLoaded;
	
//	Install our "onunload" event-handler function.
	
	window.onunload = pageUnloaded;
	
	
	exportSymbol("edu.utexas.its.eis.tools.qwicap.pageLoaded",   pageLoaded);
	exportSymbol("edu.utexas.its.eis.tools.qwicap.pageUnloaded", pageUnloaded);
	exportSymbol("edu.utexas.its.eis.tools.qwicap.deleteByID",   deleteByID);
	
	
	function pageLoaded() {
	
		requestPageState();
	
		if (OriginalOnloadFunction != null)
			OriginalOnloadFunction();	
	}
	
	function pageUnloaded() {
	
	//	This function does nothing but call the window's pre-existing "onunload" function. The important thing
	//	about having an "onunload" function, even one that does nothing, is that it causes browsers NOT to cache
	//	the DOM structure of the page, which means that if the user goes back to the page, the "onload" event
	//	fires again, allowing our "onload" handler to update the page state.
	
		if (OriginalOnunloadFunction != null)
			OriginalOnunloadFunction();
	}
	
	function requestPageState() {
		var http_request = false;
	
		if (window.XMLHttpRequest) {		//	Mozilla, Safari, ...
	
			http_request = new XMLHttpRequest();
	//		if (http_request.overrideMimeType)
	//			http_request.overrideMimeType('text/xml');
	
		} else if (window.ActiveXObject) {	//	IE
	
			try {
				http_request = new ActiveXObject("Msxml2.XMLHTTP");
			} catch (e) {
				try {
					http_request = new ActiveXObject("Microsoft.XMLHTTP");
				} catch (e) { }
			}
		}
	
		if (!http_request)
			return;		//	Give-up quietly.
	
		http_request.onreadystatechange = function() { updatePageState(http_request); };
		http_request.open('GET', QwicapXMLURL + '?qwicap-charset=iso-8859-1&qwicap-cmd=pgstat&pageid=' + QwicapPageID + '&event=onload', true);
		http_request.setRequestHeader('Content-Type',      'application/xml');
		http_request.setRequestHeader('Cache-Control',     'no-cache');
		http_request.setRequestHeader('If-Modified-Since', 'Wed, 1 Jan 1900 00:00:00 GMT');	//	Required to make Safari work. See <http://www.bitterpill.org/bp/2005-06/safari-xmlhttprequest-undefine.html>
		http_request.send('');
	}
	
	function updatePageState(http_request) {
			
		if (http_request.readyState == 4) {
	
			if (http_request.status == 200) {
				var xmldoc    = http_request.responseXML;
				var root_node = getSubElement(xmldoc, 'qwicap');
				var command   = getSubElement(root_node, 'command').textContent;
	
				if ('pgstat' == command) {
					var pgstat = getSubElement(root_node, 'pgstat');
					var status = getSubElement(pgstat, 'page-state').textContent;
	
					if ('unknown' == status) {
						var message = getSubElement(pgstat, 'message').textContent;
						var OK      = getSubElement(pgstat, 'ok-label').textContent;
						
						try {
							var pages    = getSubElements(getSubElement(pgstat, 'pages'), 'page');
							var pageInfo = new Array(pages.length);
							
							for (var Index = 0; Index < pages.length; Index++) {
								var page      = pages[Index];
								var pageID    = getSubElement(page, 'id'   ).textContent;
								var pageTitle = getSubElement(page, 'title').textContent;
								var pageType  = getSubElement(page, 'type' ).textContent;
								
								pageInfo[Index] = { id: pageID, title: pageTitle, type: pageType };
							}
							
							showNavigationError(message, OK, getSubElement(pgstat, 'pages-label').textContent, pageInfo);
						
						} catch (e) {
							
						//	There was no "pages" element in the response, because there are no active pages.
							
							showNavigationError(message, OK, '', []);
						}
					}
	
				} else
					throw('Unrecognized response: ' + command + '\n\n' + http_request.responseText);
			}
		}
	}

	function disablePage() {
		
	//	Create the "barrier" div that grays-out the page contents, and prevents the user from
	//	interacting with them.
		
		var body    = document.getElementsByTagName('body').item(0);
		var barrier = document.createElement('div');
	
		barrier.setAttribute('class', 'qwicap-barrier');
		body.appendChild(barrier);
	}
	
	function showNavigationError(MessageStr, OKLabel, PagesPrompt, Pages) {
		var body    = document.getElementsByTagName('body').item(0);
		var overlay = document.createElement('div');
	
		overlay.setAttribute('class', 'qwicap-dialog-overlay');
		
		var barrier = document.createElement('div');
		
		barrier.setAttribute('class', 'qwicap-barrier');
		
		overlay.appendChild(barrier);
		
		var dialog  = document.createElement('div');
		
		dialog.setAttribute('class',  'qwicap-dialog');
		
		overlay.appendChild(dialog);
		
		var message = document.createElement('p');
		
		message.setAttribute('class', 'qwicap-page-navigation-error-message-text');
		message.appendChild(document.createTextNode(MessageStr))
		
		dialog.appendChild(message);
		
		var buttons = document.createElement('p');
		
		buttons.setAttribute('class', 'qwicap-dialog-buttons');
		
		if (Pages.length > 1) {
			var pageSelect = document.createElement('select');
			
			pageSelect.onchange = function() { window.location=QwicapXMLURL + "?qwicap-page-id=" + pageSelect.value + "&qwicap-charset=iso-8859-1&qwicap-noop"; };
			
			var option = document.createElement('option');
			
			option.setAttribute('disabled', 'disabled');
			option.setAttribute('selected', 'selected');
			option.appendChild(document.createTextNode(PagesPrompt));
			pageSelect.appendChild(option);
			
			var preModal = false;
			
			for (var Index = Pages.length; --Index >= 0; ) {
				var page = Pages[Index];
				
				option = document.createElement('option');
				
			//	Note: The "onclick" event does not work on "option" elements in Safari.
				
				option.setAttribute('value', page.id);
				option.appendChild(document.createTextNode(page.title));
				
				if (preModal)
					option.setAttribute('disabled', 'disabled');
				
				pageSelect.appendChild(option);
				
				if ('modal' == page.type)
					preModal = true;
			}
			
			buttons.appendChild(pageSelect);
			
		} else {
			var button  = document.createElement('input');
			
			button.setAttribute('type', 'button');
			button.setAttribute('value', OKLabel);
			
			if (Pages.length == 1)
				button.setAttribute('onclick', 'window.location=QwicapXMLURL + "?qwicap-page-id=' + Pages[0].id + '&qwicap-charset=iso-8859-1&qwicap-noop";');
			else
				button.setAttribute('onclick', 'window.location=QwicapXMLURL + "?qwicap-page-id=" + QwicapPageID + "&qwicap-charset=iso-8859-1&qwicap-noop"');
			
			buttons.appendChild(button);
		}
		
		dialog.appendChild(buttons);
		
		body.appendChild(overlay);
	}
	
	function getSubElement(parent, subElementName) {
		var children = parent.childNodes;
	
		for (var index = 0; index < children.length; index++) {
			var child = children.item(index);
	
			if (child.nodeType == Node.ELEMENT_NODE && child.nodeName == subElementName)
				return child;
		}
	
		throw 'Element "' + subElementName + '" not found in parent "' + parent.nodeName + '"';
	}
	
	function getSubElements(parent, subElementName) {
		var subElems     = [];
		var subElemCount = 0;
		var children     = parent.childNodes;
	
		for (var index = 0; index < children.length; index++) {
			var child = children.item(index);
	
			if (child.nodeType == Node.ELEMENT_NODE && child.nodeName == subElementName) {
				
				subElems.length = subElemCount + 1;
				subElems[subElemCount++] = child;
			}
		}
		
		return subElems;
	}
	
	function log(Message) {
		var http_request = false;
	
		if (window.XMLHttpRequest) {		//	Mozilla, Safari, ...
	
			http_request = new XMLHttpRequest();
	//		if (http_request.overrideMimeType)
	//			http_request.overrideMimeType('text/xml');
	
		} else if (window.ActiveXObject) {	//	IE
	
			try {
				http_request = new ActiveXObject("Msxml2.XMLHTTP");
			} catch (e) {
				try {
					http_request = new ActiveXObject("Microsoft.XMLHTTP");
				} catch (e) { }
			}
		}
	
		if (!http_request)
			return;		//	Give-up quietly.
	
		http_request.onreadystatechange = function() { updatePageState(http_request); };
		http_request.open('GET', QwicapXMLURL + '?qwicap-charset=iso-8859-1&qwicap-cmd=log&pageid=' + QwicapPageID + '&message=' + encodeURIComponent(Message), true);
		http_request.setRequestHeader('Content-Type',      'application/xml');	//	todo Shouldn't this be "application/x-www-form-urlencoded"?
		http_request.setRequestHeader('Cache-Control',     'no-cache');
		http_request.setRequestHeader('If-Modified-Since', 'Wed, 1 Jan 1900 00:00:00 GMT');	//	Required to make Safari work. See <http://www.bitterpill.org/bp/2005-06/safari-xmlhttprequest-undefine.html>
		http_request.send('');
	}
	
	function deleteByID(IDStr) {
		var elem = document.getElementById(IDStr); 
		if (elem != null)
			elem.parentNode.removeChild(elem);
	}
	
	function exportSymbol(name, symbol) {
		window[name] = symbol;
	}
	
})();