/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


//  Combinator.java
//  qwicap
//
//  Created by Chris W. Johnson on Thu Jun 12 2003.


package edu.utexas.its.eis.tools.qwicap.template.css;


import edu.utexas.its.eis.tools.qwicap.template.xml.DescendantEnumerator;


/**	<code>Combinator</code> is the abstract base-class for all combinator classes.
 *	
 *	@author Chris W. Johnson
 */
abstract class Combinator extends CSSPatternComponent {
	
	
	static Combinator read
	(
		final CharacterIterator	Chars
	)
	{
		boolean					FoundSpace = false;
		char					CombChar = 0;
		
		while (Chars.hasNext()) {
			final char				Char = Chars.next();
			
			if (Char == ' ') {
				
				FoundSpace = true;
				
			} else if (Char == '>' || Char == '+') {
				
				if (CombChar != 0)
					throw new IllegalArgumentException("Consecutive combinators without intervening selectors are not permitted. In this case, '" + CombChar + "' and '" + Char + "' are adjacent. (See " + Chars + ".)");
				
				CombChar = Char;
				
			} else {
				
				Chars.goBack();
				break;
			}
		}
		
		if (CombChar == '>')
			return new CombinatorChild();
		else if (CombChar == '+')
			return new CombinatorAdjacent();
 		else if (FoundSpace)
 			return new CombinatorDescendant();
		
		return null;
	}
	
	abstract void search(DescendantEnumerator Elems, Selector Sel, Results Matches);
	
	@Override public abstract String toString();
}







