/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


//  CombinatorAdjacent.java
//  qwicap
//
//  Created by Chris W. Johnson on Thu Jun 12 2003.


package edu.utexas.its.eis.tools.qwicap.template.css;


import edu.utexas.its.eis.tools.qwicap.template.xml.structure.*;
import edu.utexas.its.eis.tools.qwicap.template.xml.DescendantEnumerator;


/**	<code>CombinatorAdjacent</code> is the combinator responsible for implementing Adjacent Selectors, of the form 
 *	"E + F", which "Matches any F element immediately preceded by an element E."
 *	
 *	@author Chris W. Johnson
 */
final class CombinatorAdjacent extends Combinator {


	@Override void search
	(
		final DescendantEnumerator	MasterElems,	//	This won't be altered, because we use a clone internally.
		final Selector				Sel,			//	Can be null.
		final Results				Matches
	)
	{
		if (Sel == null)
			return;
		
		final Combinator			Comb = (Combinator) Sel.NextPart;
		final DescendantEnumerator	Elems = MasterElems.clone();
		final int					ParentCount = Elems.getParentCount();
		final int					DesiredChildNo = Elems.getChildNo() + 1;
		
		while (Elems.hasMoreElements()) {
			final Range					Elem = Elems.nextRange();
			final int					ElemParentCount = Elems.getParentCount();
			
			if (ElemParentCount > ParentCount)
				continue;
			if (ElemParentCount < ParentCount)
				break;
			
			if (Elem.isTagWithAttributes()) {
				final int					ElemChildNo = Elems.getChildNo();
				
				if (ElemChildNo != DesiredChildNo)
					break;
				
				if (Sel.selects((TagWithAttributes) Elem, ElemChildNo)) {
					
					if (Comb != null)
						Comb.search(Elems, (Selector) Comb.NextPart, Matches);
					else
						Matches.addMatch(Elems.getMatch());
				}
			}
		}
	}
	
	@Override public String toString() {
		return "+";
	}
}





