/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2006 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.template.css;


import junit.framework.TestCase;
import edu.utexas.its.eis.tools.qwicap.template.xml.mutable.MutableMarkup;
import edu.utexas.its.eis.tools.qwicap.template.xml.immutable.XMLString;
import edu.utexas.its.eis.tools.qwicap.template.xml.TagException;
import edu.utexas.its.eis.tools.qwicap.template.xml.Markup;

import java.math.BigDecimal;
import java.math.BigInteger;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Feb 22, 2006
 * Time: 12:39:26 AM
 * To change this template use File | Settings | File Templates.
 */
public class ResultsJUnit extends TestCase {


	public void test_IF_ELSE_ENDIF() throws TagException {
		final XMLString				Master = new XMLString("<thing></thing>");
		MutableMarkup				Doc;


		Doc = Master.getMutable();
		Doc.get("thing").IF(true).addClass("true");

		assertTrue(Doc.toString(), Doc.toString().equals("<thing class='true'></thing>"));



		Doc = Master.getMutable();
		Doc.get("thing").IF(true).addClass("true").ELSE().addClass("false");

		assertTrue(Doc.toString(), Doc.toString().equals("<thing class='true'></thing>"));



		Doc = Master.getMutable();
		Doc.get("thing").IF(false).addClass("true");

		assertTrue(Doc.toString(), Doc.toString().equals("<thing></thing>"));



		Doc = Master.getMutable();
		Doc.get("thing").IF(false).addClass("true").ELSE().addClass("false");

		assertTrue(Doc.toString(), Doc.toString().equals("<thing class='false'></thing>"));



		Doc = Master.getMutable();
		Doc.get("thing").IF(true).addClass("true").ENDIF().setContent("Done.");

		assertTrue(Doc.toString(), Doc.toString().equals("<thing class='true'>Done.</thing>"));



		Doc = Master.getMutable();
		Doc.get("thing").IF(false).addClass("true").ENDIF().setContent("Done.");

		assertTrue(Doc.toString(), Doc.toString().equals("<thing>Done.</thing>"));



		Doc = Master.getMutable();
		Doc.get("thing").IF(true).addClass("true").ELSE().addClass("false").ENDIF().setContent("Done.");

		assertTrue(Doc.toString(), Doc.toString().equals("<thing class='true'>Done.</thing>"));



		Doc = Master.getMutable();
		Doc.get("thing").IF(false).addClass("true").ELSE().addClass("false").ENDIF().setContent("Done.");

		assertTrue(Doc.toString(), Doc.toString().equals("<thing class='false'>Done.</thing>"));
	}

	public void testToString() throws TagException {
		final Markup			Doc = new XMLString("<html><body><span class='test'>Hello World.</span></body></html>");

	//	If we've screwed-up Results.toString, this should catch the mistake.

		assertTrue("<span class='test'>Hello World.</span>".equals(Doc.get("span.test").toString()));
	}

	public void testNumberRetrieval() throws Exception {
		final Markup			Doc = new XMLString("<html><integer>90</integer><decimal>90.5</decimal></html>");
	
		tryIntegerRetrieval(Doc.get("integer"), new BigInteger("90"), new BigDecimal("90"), "90");
		tryDecimalRetrieval(Doc.get("decimal"), new BigDecimal("90.5"), "90.5");
	}
	
	private static void tryIntegerRetrieval
	(
		final Results			NumberElement,
		final BigInteger		IntegerValue,
		final BigDecimal		DecimalValue,
		final String			StringValue
	)
	{
		assertTrue(DecimalValue.compareTo(NumberElement.getBigDecimal()) == 0);
		assertTrue(IntegerValue.compareTo(NumberElement.getBigInteger()) == 0);
		assertTrue((byte) IntegerValue.intValue()  == NumberElement.getByte());
		assertTrue(DecimalValue.doubleValue()      == NumberElement.getDouble());
		assertTrue(NumberElement.getFloat()        == NumberElement.getFloat());
		assertTrue(IntegerValue.intValue()         == NumberElement.getInt());
		assertTrue(IntegerValue.longValue()        == NumberElement.getLong());
		assertTrue((short) IntegerValue.intValue() == NumberElement.getShort());
		assertTrue(StringValue.trim().equals(NumberElement.getText().trim()));
	}
	
	private static void tryDecimalRetrieval
	(
		final Results			NumberElement,
		final BigDecimal		DecimalValue,
		final String			StringValue
	)
	{
		assertTrue(DecimalValue.compareTo(NumberElement.getBigDecimal()) == 0);
		assertTrue(DecimalValue.doubleValue() == NumberElement.getDouble());
		assertTrue(NumberElement.getFloat()   == NumberElement.getFloat());
		assertTrue(StringValue.trim().equals(NumberElement.getText().trim()));
	}
	
	public void testSelectionOfAttributeWithAColonInItsName() throws Exception {
		final Markup			Doc = new XMLString("<thing xmlns:wombat='http://www.wombat.edu/'></thing>");
		
		assertTrue("Selection of an attribute with a colon (':') in its name failed.", Doc.get("[xmlns\\:wombat]").notEmpty());
	}
	
	public void testSelectionOfAttributeWithAColonInItsNameWhenNamespaceAware() throws Exception {
		final Markup			Doc = new XMLString("<thing xmlns:wombat='http://www.wombat.edu/'></thing>", true);
		
		assertTrue(
			"Selection of an attribute with a colon (':') in its name failed when operating on markup that is " +
			"treated as namespace-aware. This is correct, but special-casing attribute names containing colons " +
			"might be a good idea for the future, since it would provide consistent behavior between namespace-aware " +
			"and non-namespace-aware markup. This test is merely a reminder of the issue.", 
			Doc.get("[xmlns\\:wombat]").notEmpty()
		);
		
	//	todo We should also consider modifying the Results and Match "getAttribute", "setAttribute", etc. methods to 
	//	     accept a colon-delimited namespace prefix in their "attribute name" parameters. 
	}
	
	public void testSelectionOfAttributeInTheXMLNSNamespaceWhenNamespaceAware() throws Exception {
		final Markup			Doc = new XMLString("<thing xmlns:wombat='http://www.wombat.edu/'></thing>", true);
		
		assertTrue("Selection of an attribute in the XMLNS namespace failed.", Doc.get("[xmlns|wombat]").notEmpty());
	}
	
	public void testRetrievalOfAttributeWithAColonInItsName() throws Exception {
		final Markup			Doc = new XMLString("<thing xmlns:wombat='http://www.wombat.edu/'></thing>");
		
		assertTrue("Selection of an attribute with a colon (':') in its name failed.", Doc.get("[xmlns\\:wombat]").notEmpty());
		
		final Results			Attrs = Doc.get("[xmlns\\:wombat]");
		
		assertTrue("Could not use Results.getAttribute to get an attribute whose name includes a colon (':').", Attrs.getAttribute("xmlns:wombat") != null);
	}
	
	public void testRetrievalOfAttributeInTheXMLNSNamespaceWhenNamespaceAware() throws Exception {
		final Markup			Doc = new XMLString("<thing xmlns:wombat='http://www.wombat.edu/'></thing>", true);
		
		assertTrue("Selection of an attribute in the XMLNS namespace failed.", Doc.get("[xmlns|wombat]").notEmpty());
		
		final Results			Attrs = Doc.get("[xmlns|wombat]");
		
		assertTrue(
			"Could not use Results.getAttribute to get an attribute explicitly identified by the \"xmlns:\" prefix " +
			"as being in the XMLNS namespace. This is correct, but special-casing namespace prefixes in attribute " +
			"names passed to \"getAttribute\", \"setAttribute\", etc. should be considered. This test is merely a " +
			"reminder of the issue.", 
			Attrs.getAttribute("xmlns:wombat") != null
		);
	}
}
