/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


//  SelectorAttributeExists.java
//  qwicap
//
//  Created by Chris W. Johnson on Wed Jun 11 2003.


package edu.utexas.its.eis.tools.qwicap.template.css;


import edu.utexas.its.eis.tools.qwicap.template.xml.structure.*;
import edu.utexas.its.eis.tools.qwicap.util.Characters;


/**	<code>SelectorAttributeExists</code> is the attribute selector responsible for supporting CSS selectors of the form 
 *	"[foo]", which tests for the presence of a tag attribute named "foo".
 *	
 *	@author Chris W. Johnson
 */
final class SelectorAttributeExists extends SelectorAttribute {
	
	private final Namespace		AttrNs;
	private final Characters	AttrName;
	private String				AttrNameStr;	//	Cached results of toString method.
	
	
	SelectorAttributeExists
	(
		final Namespace			AttributeNs,	//	Can be null.
		final String			AttributeName
	)
	{
		AttrNs   = AttributeNs;
		AttrName = new Characters(AttributeName);
	}
	
	@Override boolean selects
	(
		final TagWithAttributes	TWA,
		final int				ChildNo
	)
	{
		final Attribute			Attr = TWA.getAttribute(AttrName);
		
		if (Attr == null)
			return false;
		
		if (AttrNs == null)
			return true;	//	We already know that the attribute exists, and we don't have to bother about namespaces, so we're done.
		
		return AttrNs.equals(Attr.getNamespace());
	}
	
	@Override public String toString() {
		
		if (AttrNameStr == null) {
			final StringBuilder		Buff = new StringBuilder(AttrName.length() + 2);
			
			Buff.append('[');
			
			if (AttrNs != null)
				Buff.append(AttrNs.getName()).append('|');
			
			AttrName.append(Buff);
			Buff.append(']');
			
			AttrNameStr = Buff.toString();
		}
		
		return AttrNameStr;
	}
}







