/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


//  SelectorClass.java
//  qwicap
//
//  Created by Chris W. Johnson on Thu Jun 12 2003.


package edu.utexas.its.eis.tools.qwicap.template.css;


import edu.utexas.its.eis.tools.qwicap.template.xml.structure.TagWithAttributes;
import edu.utexas.its.eis.tools.qwicap.util.Characters;


/**	<code>SelectorClass</code> is the selector responsible for supporting CSS selectors of the form "E.thingy", which 
 *	"Matches any E element whose 'class' attribute value is a list of space-separated values, one of which is exactly 
 *	equal to 'thingy'."
 *	
 *	@author Chris W. Johnson
 */
final class SelectorClass extends Selector {
	
	private final Characters	ClassName;
	
	
	SelectorClass
	(
		final String			ClassNameStr	//	Should not include the preceding period (.)
	) 
	{
		this.ClassName = new Characters(ClassNameStr);
	}
	
	@Override boolean selects
	(
		final TagWithAttributes	Other,
		final int				ChildNo
	)
	{
		return Other.isOfClass(ClassName);
	}
	
	@Override public String toString() {		
		return '.' + ClassName.toString();
	}
}

