/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


//  SelectorCompound.java
//  qwicap
//
//  Created by Chris W. Johnson on Thu Jun 12 2003.


package edu.utexas.its.eis.tools.qwicap.template.css;


import edu.utexas.its.eis.tools.qwicap.template.xml.structure.TagWithAttributes;


/**	<code>SelectorCompound</code> is the selector responsible for combining the function of any number of other 
 *	<code>Selectors</code> to allow more specific selections than any single <code>Selector</code> provides.
 *	
 *	@author Chris W. Johnson
 */
final class SelectorCompound extends Selector {
	
	private Selector			List;
	private Selector			ListEnd;
	
	
	void add
	(
		final Selector			S
	)
	{
		if (List == null)
			List = S;
		else
			ListEnd.NextSel = S;
		
		ListEnd = S;
	}
	
	Selector simplify() {
		
		if (List != null) {
			
			if (List != ListEnd) {	//	If there's more than one selector in this list...
				
				if (List instanceof SelectorUniversal)
					List = List.NextSel;	//	We don't need a leading SelectorUniversal, so remove it from the list.
			}
			
			if (List == ListEnd)	//	If there's only one selector in this list...
				return List;
		}
		
		return this;
	}

	@Override boolean selects
	(
		final TagWithAttributes	OtherTag,
		final int				OtherTagChildNo
	)
	{
		boolean					Selected = true;
		Selector				CurSel   = List;
		
		while (CurSel != null && Selected) {
			
			Selected = CurSel.selects(OtherTag, OtherTagChildNo);
			CurSel   = CurSel.NextSel;
		}
		
		return Selected;
	}

	@Override public String toString() {
		final StringBuilder		Buff   = new StringBuilder(64);
		Selector				CurSel = List;
		
		while (CurSel != null) {
			
			Buff.append(CurSel.toString());
			CurSel = CurSel.NextSel;
		}
		
		return Buff.toString();
	}
}




