/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2008 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


//  CloneableEnumeration.java
//  qwicap
//
//  Created by Chris W. Johnson on Tue Jun 17 2003.


package edu.utexas.its.eis.tools.qwicap.template.xml;


import edu.utexas.its.eis.tools.qwicap.template.xml.structure.Range;

import java.util.Enumeration;
import java.util.NoSuchElementException;
import java.util.Iterator;


/**	The <code>MarkupEnumeration</code> interface defines the basic functionality required from all of the various 
 *	enumerators that provide access to the elements of <code>Markup</code> objects. Internally, the most significant 
 *	quality of this interface is that it mandates support for cloning, which allows an enumeration-in-progress to be 
 *	cloned so that it can be passed to something that does specialized processing on some subset of the original
 *	enumeration, and when that's complete, the original enumeration can resume where it left-off.
 *	
 *	@author Chris W. Johnson
 */
public interface MarkupEnumeration extends Enumeration<Range>, Iterator<Range>, Cloneable {
	
/**	Returns the next <code>Range</code> (the parent interface of all markup elements) in the <code>Markup</code> that is
 *	being enumerated. This is equivalent to the <code>nextElement</code> method, except for its return type.
 * 
 * @return								The next <code>Range</code> (the parent class of all markup elements) in the 
 * 										<code>Markup</code> that is being enumerated.
 * @throws NoSuchElementException		If this enumeration has been asked to return a <code>Range</code> after the 
 * 										final <code>Range</code>.
 */
	public Range				nextRange() throws NoSuchElementException;
	
/**	Returns the <code>Markup</code> object that is being enumerated.
 *
 * @return								The <code>Markup</code> object that is being enumerated.
 */
	public Markup				getMarkup();

/**	Returns the index of the most recently returned markup element within the <code>Markup</code> object's internal 
 *	list of elements.
 *	
 * @return								The <code>Markup</code> object's internal index number for the element most 
 * 										recently returned by <code>nextElement</code> or <code>nextRange</code>.
 */
	public int					getIndex();

/**	Returns a <code>Match</code> object uniquely identifying the occurrence of the most recently returned markup 
 *	element within the the <code>Markup</code> object that is being enumerated.
 *
 * @return								A <code>Match</code> object identifying the element most recently returned by 
 * 										<code>nextElement</code> or <code>nextRange</code>.
 */
	public Match				getMatch();

/**	Clones this enumeration. Typically this is done so that other code can begin processing at this point in the
 * 	<code>Markup</code>, without interfering with the ability of the code that created the enumeration to continue its 
 *	processing of the <code>Markup</code>.
 *
 * @return								A clone of this enumeration.
 */
	public MarkupEnumeration	clone();
}







