/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


//  OneItemEnumerator.java
//  qwicap
//
//  Created by Chris W. Johnson on Mon Jun 23 2003.


package edu.utexas.its.eis.tools.qwicap.template.xml;


import edu.utexas.its.eis.tools.qwicap.template.xml.structure.Range;

import java.util.*;


/**	Used by the {@link Match#enumerate} method to enumerate any match that does not begin with a <code>StartTag</code>. 
 *	In other words, for any match that correspond to a single element, rather than a potential hierarchy of elements.
 *	<code>MatchEnumerator</code> is used for the hiearchies.
 *	
 *	@author Chris W. Johnson
 */
final class OneItemEnumerator implements MarkupEnumeration {
	
	private Match				Curr;
	private Match				Next;
	
	
	OneItemEnumerator
	(
		final Match				Match
	)
	{
		this.Next = Match;
	}
	
	private OneItemEnumerator
	(
		final OneItemEnumerator	Master
	)
	{
		Curr = Master.Curr;
		Next = Master.Next;
	}
	
	@Override public Markup getMarkup() {
		return Curr.getMarkup();
	}
	
	@Override public int getIndex() {
		return Curr.getIndex();
	}
	
	@Override public Match getMatch() {
		return Curr;
	}
	
	@Override public boolean hasNext() {
		return hasMoreElements();
	}
	
	@Override public Range next() {
		return nextRange();
	}
	
	@Override public void remove() {
		throw new UnsupportedOperationException();
	}
	
	@Override public boolean hasMoreElements() {
		return Next != null;
	}
	
	@Override public Range nextRange() {
		
		if (Next == null)
			throw new NoSuchElementException();
		
		Curr = Next;
		Next = null;
		
		return Curr.get();
	}
	
	@Override public Range nextElement() {
		return nextRange();
	}
	
	@Override public OneItemEnumerator clone() {
		return new OneItemEnumerator(this);
	}
}




