/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2011 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


//  RangeListEnumerator.java
//  qwicap
//
//  Created by Chris W. Johnson on Fri Jun 13 2003.


package edu.utexas.its.eis.tools.qwicap.template.xml;


import edu.utexas.its.eis.tools.qwicap.template.xml.structure.Range;

import java.util.NoSuchElementException;


/**	Enumerates every component of a body of markup in order.
 *	
 *	@author Chris W. Johnson
 */
final class RangeListEnumerator implements MarkupEnumeration {
	
	private final Markup		Markup;
	private final RangeList		List;
	private int					Index;
	private Range				CurrElement;
	private int					CurrIndex;
	private Range				NextElement;
	
	
	private RangeListEnumerator
	(
		final RangeListEnumerator	Master
	)
	{
		Markup      = Master.Markup;
		List        = Master.List;
		Index       = Master.Index;
		CurrElement = Master.CurrElement;
		CurrIndex   = Master.CurrIndex;
		NextElement = Master.NextElement;
	}
	
	RangeListEnumerator
	(
		final Markup			Markup
	)
	{
		this.Markup      = Markup;
		this.List        = Markup.getList();
		this.Index       = -1;
		this.NextElement = getNext();
	}
	
	private Range getNext() {
		Range					Next = null;
		
		Index++;
		
		if (Index < List.size())
			Next = List.Array[Index];
		
		return Next;
	}
	
	@Override public boolean hasNext() {
		return hasMoreElements();
	}
	
	@Override public Range next() throws NoSuchElementException {
		return nextRange();
	}
	
	@Override public void remove() {
		throw new UnsupportedOperationException();
	}
	
	@Override public boolean hasMoreElements() {
		return NextElement != null;	
	}
	
	@Override public Range nextRange() throws NoSuchElementException {
		
		CurrElement = NextElement;
		CurrIndex   = Index;
		
		NextElement = getNext();
		
		if (CurrElement == null)
			throw new NoSuchElementException("Iteration of markup \"" + Markup.getMarkupName() + "\" is already finished.");
		
		return CurrElement;
	}
	
	@Override public Range nextElement() throws NoSuchElementException {
		return nextRange();
	}
	
	@Override public int getIndex() {
		return CurrIndex;
	}
	
	@Override public Match getMatch() {
		return Match.create(Markup, CurrElement, CurrIndex);
	}
	
	@Override public Markup getMarkup() {
		return Markup;
	}

	@Override public RangeListEnumerator clone() {
		return new RangeListEnumerator(this);
	}
}

















