/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


//  EndTag.java
//  qwicap
//
//  Created by Chris W. Johnson on Sat Jun 07 2003.


package edu.utexas.its.eis.tools.qwicap.template.xml.immutable;


import edu.utexas.its.eis.tools.qwicap.template.xml.structure.EndTag;


/**	Represents immutable "end" tags in XML documents.
 * 
 *	@author Chris W. Johnson
 */
final class ImEndTag extends ImTag implements EndTag {
	
	private static final char[]	kBeginSequence	= "</".toCharArray();
	static final char[]			kEndSequence	=  ">".toCharArray();
	
	
//	Starts with "</", ends with ">". Name, but no attributes.
	
	ImEndTag
	(
		final char[]			Chars,
		final int				PyStart,
		final int				PyEnd
	)
	{
		super(Chars, PyStart, PyEnd);
		
		ContentStart = PyStart + kBeginSequence.length;
		ContentEnd   = PyEnd   - kEndSequence.length;
		
		eatTrailingWhitespace();	//	I believe leading whitespace is illegal, so trailing is all we have to watch for.
		
		NameStart = ContentStart;
		NameEnd   = ContentEnd;
	}
	
	@Override public boolean isEndTag() { return true; }
}





