/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


//  Item.java
//  qwicap
//
//  Created by Chris W. Johnson on Sat Jun 07 2003.


package edu.utexas.its.eis.tools.qwicap.template.xml.immutable;


import edu.utexas.its.eis.tools.qwicap.template.xml.structure.Item;
import edu.utexas.its.eis.tools.qwicap.util.Characters;


/**	Common ancestor of all classes representing immutable sections of XML documents that contain material other than
 *	whitespace.
 * 
 *	@author Chris W. Johnson
 */
abstract class ImItem extends ImRange implements Item {
	
	int						ContentStart;
	int						ContentEnd;
	
	public boolean			HadLeadingWhitespace;
	public boolean			HadTrailingWhitespace;
	
	
	ImItem
	(
		final char[]			Chars,
		final int				PyStart,
		final int				PyEnd
	)
	{
		super(Chars, PyStart, PyEnd);
	}
	
	final void trimContent() {
		eatLeadingWhitespace();
		eatTrailingWhitespace();
	}
	
	private void eatLeadingWhitespace() {
		boolean					HadLeadingWhitespace = false;
		int						Index = ContentStart;
		final int				Limit = ContentEnd;
		final char[]			Chars = this.Chars;
		
		while (Index < Limit && isWhitespace(Chars[Index])) {
			
			HadLeadingWhitespace = true;
			Index++;
		}
		
		ContentStart = Index;
		this.HadLeadingWhitespace = HadLeadingWhitespace;
	}
	
	final void eatTrailingWhitespace() {
		final boolean			HadTrailingWhitespace;
		int						Index = ContentEnd - 1;
		final int				Limit = ContentStart;
		final char[]			Chars = this.Chars;
		
		if (Index >= Limit && isWhitespace(Chars[Index])) {
			
			HadTrailingWhitespace = true;
			
			do {
				Index--;
			} while (Index >= Limit && isWhitespace(Chars[Index]));
			
			ContentEnd = Index + 1;
			this.HadTrailingWhitespace = HadTrailingWhitespace;
		}
	}
	
	@Override public final boolean contentEquals
	(
		final Characters		Other
	)
	{
		return Other.equals(Chars, ContentStart, ContentEnd - ContentStart);
	}
	
	@Override public final Characters getContent() {
		return new Characters(Chars, ContentStart, ContentEnd - ContentStart);
	}
}





