/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.template.xml.immutable;


import edu.utexas.its.eis.tools.qwicap.util.Characters;
import edu.utexas.its.eis.tools.qwicap.template.xml.structure.Namespace;


/**	Represents a namespace in an XML document. This is not part of Qwicap's public API, but it
 *	could not be hidden because uses of it span package boundaries.
 *
 *	@author Chris W. Johnson
 */
public final class ImNamespace implements Namespace {
	
//	From "Namespace in XML", <http://www.w3.org/TR/REC-xml-names/#ns-using>:
//	"The prefix xml is by definition bound to the namespace name http://www.w3.org/XML/1998/namespace."


	private final boolean		IsDefault;
	private final Characters	Name;
	private final Characters	URI;
	
	
	public ImNamespace
	(
		final Characters		Name,
		final Characters		URI
	)
	{
		this.IsDefault = Name == null;
		this.Name      = Name;
		this.URI       = URI;
	}
	
	public ImNamespace
	(
		final String			Name,	//	Can be null
		final String			URI		//	Can be null. If the URI was empty (zero length) please pass null.
	)
	{
		this.IsDefault = Name == null;
		this.Name      = Name == null ? null : new Characters(Name);
		this.URI       = URI  == null ? null : new Characters(URI );
	}
	
	@Override public boolean isDefault() {
		return IsDefault;
	}
	
	@Override public final Characters getName() {
		return Name;	//	Can be null
	}
	
	@Override public final Characters getURI() {
		return URI;		//	Can be null
	}
	
	@Override public boolean equals
	(
		final Object			OtherObj
	)
	{
		if (this == OtherObj)
			return true;
		
		if (OtherObj instanceof Namespace) {
			final Characters		OtherURI = ((Namespace) OtherObj).getURI();
			
			if (URI == null)
				return OtherURI == null;
			else
				return URI.equals(OtherURI);
			
		} else if (OtherObj instanceof Characters || OtherObj instanceof String) {
			
			if (URI != null)
				return URI.equals(OtherObj);
		}
		
		return false;
	}
	
	@Override public int hashCode() {
		return URI != null ? URI.hashCode() : 0;	
	}
	
	@Override public String toString() {
		return URI == null ? "" : URI.toString();
	}
}
