/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


//  ImXMLDeclaration.java
//  qwicap
//
//  Created by Chris W. Johnson on Sat Jun 07 2003.


package edu.utexas.its.eis.tools.qwicap.template.xml.immutable;


import edu.utexas.its.eis.tools.qwicap.util.Characters;
import edu.utexas.its.eis.tools.qwicap.template.xml.structure.XMLDeclaration;
import edu.utexas.its.eis.tools.qwicap.template.xml.structure.Attribute;
import edu.utexas.its.eis.tools.qwicap.template.xml.AttributeEnumeration;

import java.util.Iterator;


/** Represents the immutable XML declaration tag ("<samp>&lt;?xml version="1.0" encoding="UTF-8"?&gt;</samp>") at the 
 * 	start of any XML document.
 * 
 *	@author Chris W. Johnson
 */
final class ImXMLDeclaration extends ImItem implements XMLDeclaration {

	private static final char[]		kBeginSequence	= "<?".toCharArray();
	static final char[]				kEndSequence	= "?>".toCharArray();

	private final ImAttribute[]		Attributes;


//	Starts with "<?", ends with "?>". No name.

	ImXMLDeclaration
	(
		final char[]			Chars,
		final int				PyStart,
		final int				PyEnd
	)
	{
		super(Chars, PyStart, PyEnd);
		
		ContentEnd   = PyEnd - kEndSequence.length;
		ContentStart = readUntilWhitespace(Chars, PyStart + kBeginSequence.length, ContentEnd);
		
		Attributes   = ImAttribute.readAll(Chars, ContentStart, ContentEnd);
	}

	@Override public String getVersion() {
		return getAttribute("version");
	}

	@Override public String getEncoding() {
		return getAttribute("encoding");
	}

	@Override public final String getAttribute
	(
		final String			AttrName
	)
	{
		String					Value = null;
		final Attribute			Attr  = getAttribute(new Characters(AttrName));

		if (Attr != null)
			Value = Attr.getValue().toString();

		return Value;
	}

	@Override public final Attribute getAttribute
	(
		final Characters		AttrName
	)
	{
		for (final ImAttribute Attr : Attributes) {

			if (Attr.nameEquals(AttrName))
				return Attr;
		}

		return null;
	}

	@Override public final AttributeEnumeration enumerateAttributes() {

		if (Attributes.length == 0)
			return NoAttributeEnumerator.kTheOnlyInstanceYouWillEverNeed;
		else
			return new AttributeEnumeratorArray(Attributes);
	}
	
	@Override public Iterator<Attribute> iterator() {
		return enumerateAttributes();
	}
}




