/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2008-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.template.xml.immutable;


import edu.utexas.its.eis.tools.qwicap.template.xml.IntraElementContentMatch;


/**	Represents the content inside of an immutable CDATA or comment element, as though the content were a child of the 
 * 	CDATA or comment element. (Internally, the content is <i>not</i> separate from the CDATA or comment element, so 
 * 	treating the content as a child is merely a convenient fiction intended to keep the markup manipulation API simple 
 * 	and relatively consistent.)
 * 
 * 	@author Chris W. Johnson
 */
final class ImmutableIntraElementContentMatch extends IntraElementContentMatch {

	ImmutableIntraElementContentMatch
	(
		final ImmutableMatch	ElementMatch
	)
	{
		super(ElementMatch);
	}
	
	@Override public boolean equals
	(
		final Object			Other
	)
	{
		if (this == Other)
			return true;
		
		if (Other instanceof ImmutableIntraElementContentMatch)
			return ElementMatch.equals(((ImmutableIntraElementContentMatch) Other).ElementMatch);
		
		return false;
	}
}
