/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


//  NoAttributeEnumerator.java
//  qwicap
//
//  Created by Chris W. Johnson on Fri Jul 18 2003.


package edu.utexas.its.eis.tools.qwicap.template.xml.immutable;


import edu.utexas.its.eis.tools.qwicap.template.xml.structure.Attribute;
import edu.utexas.its.eis.tools.qwicap.template.xml.AttributeEnumeration;

import java.util.*;


/**	<code>NoAttributeEnumerator</code> provides an extremely economical enumerator for the attributes of markup 
 *	elements that have no attributes. In other words, it does nothing but inform its callers that there are no 
 *	attributes to enumerate.
 *	
 *	@author Chris W. Johnson
 */
final class NoAttributeEnumerator implements AttributeEnumeration {
	
	static final NoAttributeEnumerator	kTheOnlyInstanceYouWillEverNeed = new NoAttributeEnumerator();
	
	
	private NoAttributeEnumerator() { }
	
	@Override public int size() {
		return 0;
	}
	
	@Override public boolean hasNext() {
		return hasMoreElements();
	}
	
	@Override public Attribute next() {
		return nextAttribute();
	}
	
	@Override public void remove() {
		throw new UnsupportedOperationException();
	}
	
/**	Returns false in order to inform the caller that there are no attributes to be enumerated.
 *
 * @return	false, because there are no attributes.
 */
	@Override public boolean hasMoreAttributes() {
		return false;
	}
	
	@Override public boolean hasMoreElements() {
		return false;
	}

/**	Throws a NoSuchElementException to inform the caller that there are no attributes to be enumerated.
 *
 * @return							Never returns anything; always throws a NoSuchElementException.
 * @throws NoSuchElementException	Informs the caller that there are no attributes to be enumerated.
 */
	@Override public Attribute nextAttribute() throws NoSuchElementException {
		throw new NoSuchElementException();
	}
	
	@Override public Attribute nextElement() throws NoSuchElementException {
		throw new NoSuchElementException();
	}
}
