/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.template.xml.immutable;


import edu.utexas.its.eis.tools.qwicap.util.ArrayToString;

import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: May 25, 2007
 * Time: 6:40:59 PM
 * To change this template use File | Settings | File Templates.
 */
final class UnsupportedEncodingForXHTMLDocumentException extends UnsupportedEncodingException {

	private final String		MarkupName;
	private final String		DefaultCharset;
	private final String		SpecifiedCharset;
	private final String		SpecifierTag;


	UnsupportedEncodingForXHTMLDocumentException
	(
		final String						MarkupName,
		final String						DefaultCharset,
		final String						SpecifiedCharset,
		final String						SpecifierTag,
		final UnsupportedEncodingException	Cause
	)
	{
		super(SpecifiedCharset);
		initCause(Cause);

		this.MarkupName       = MarkupName;
		this.DefaultCharset   = DefaultCharset;
		this.SpecifiedCharset = SpecifiedCharset;
		this.SpecifierTag     = SpecifierTag;
	}

	@Override public String getMessage() {
		
		if (MarkupName != null) {
			
			return "The attempt to switch from interpreting the markup \"" + MarkupName + "\" as if it consisted of " +
				"characters in the default character set, \"" + DefaultCharset + "\", to interpreting it as characters " +
				"in the set \"" + SpecifiedCharset + "\", as specified by the tag \"" + SpecifierTag + "\", failed " +
				"because that character set is not supported by Java. (Java supports the following character sets: " +
				new ArrayToString(Charset.availableCharsets().keySet()) + ".)";
			
		} else {
			
			return "The attempt to switch from interpreting the markup (not named) as if it consisted of characters " +
				"in the default character set, \"" + DefaultCharset + "\", to interpreting it as characters in the set \"" +
				SpecifiedCharset + "\", as specified by the tag \"" + SpecifierTag + "\", failed because that character " +
				"set is not supported by Java. (Java supports the following character sets: " +
				new ArrayToString(Charset.availableCharsets().keySet()) + ".)";			
		}
	}
}
