/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2009 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.template.xml.immutable;


import edu.utexas.its.eis.tools.qwicap.template.xml.TagException;
import junit.framework.TestCase;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: May 25, 2007
 * Time: 3:56:01 PM
 * To change this template use File | Settings | File Templates.
 */
public class XHTMLReaderJUnit extends TestCase {

	public void testReaderForDocumentLongerThanParseBuffer() throws Exception {

	//	The significant point about this test is that it requires the XHTMLReader to deal with a document whose
	//	length exceeds the amount of the document that XHTMLReader will parse before giving up on its search for
	//	character set information.

		final int				DesiredLength = XHTMLReader.kParseBufferSize + 1024;
		final StringBuilder		SetupBuff     = new StringBuilder(DesiredLength);

		SetupBuff.append("<?xml version=\"1.0\"?><html><head><title>Test</title></head><body><h1>Hello World</h1></body></html>");

		for (int Count = DesiredLength - SetupBuff.length(); --Count >= 0;)
			SetupBuff.append(' ');

		final String			CharSet = "UTF-8";
		final String			TestDoc = SetupBuff.toString();
		final XHTMLReader		Rdr     = new XHTMLReader("Test Markup", new ByteArrayInputStream(TestDoc.getBytes(CharSet)));
		final StringBuilder		Buff    = new StringBuilder(TestDoc.length());
		int						Char    = Rdr.read();

		while (Char >= 0) {

			Buff.append((char) Char);

			Char = Rdr.read();
		}

		if (TestDoc.equals(Buff.toString()) == false)
			fail("The document as read by the XHTMLReader does not match the original document. The original looks " +
				 "like this:\n\n" + TestDoc + "\n\nThe document as read looks like this:\n\n" + Buff);

	//	The following line will throw an exception if there's any problem parsing the document.

		new XMLDocument(new ByteArrayInputStream(TestDoc.getBytes(CharSet)), 0, "Test Markup");
	}

	public void testReader1() throws IOException, TagException {
		final String			CharSet = "UTF-8";
		final String			TestDoc = "<?xml version=\"1.0\" encoding=\"" + CharSet + "\"?><html><head><title>Test</title></head><body><h1>\u2022 Hello World \u2022</h1></body></html>";

		final XHTMLReader		Rdr     = new XHTMLReader("Test Markup", new ByteArrayInputStream(TestDoc.getBytes(CharSet)));
		final StringBuilder		Buff    = new StringBuilder(TestDoc.length());
		int						Char    = Rdr.read();

		while (Char >= 0) {

			Buff.append((char) Char);

			Char = Rdr.read();
		}

		if (TestDoc.equals(Buff.toString()) == false)
			fail("The document as read by the XHTMLReader does not match the original document. The original looks " +
				 "like this:\n\n" + TestDoc + "\n\nThe document as read looks like this:\n\n" + Buff);

	//	The following line will throw an exception if there's any problem parsing the document.

		new XMLDocument(new ByteArrayInputStream(TestDoc.getBytes(CharSet)), 0, "Test Markup");
	}

	public void testReader2() throws IOException, TagException {
		final String			CharSet = "UTF-8";
		final String			TestDoc = "<?xml version=\"1.0\"?><html><head><title>Test</title></head><body><h1>Hello World</h1></body></html>";

		System.out.println("You should see a warning logged next....");

		final XHTMLReader		Rdr     = new XHTMLReader("Test Markup", new ByteArrayInputStream(TestDoc.getBytes(CharSet)));
		final StringBuilder		Buff    = new StringBuilder(TestDoc.length());
		int						Char    = Rdr.read();

		while (Char >= 0) {

			Buff.append((char) Char);

			Char = Rdr.read();
		}

		if (TestDoc.equals(Buff.toString()) == false)
			fail("The document as read by the XHTMLReader does not match the original document. The original looks " +
				 "like this:\n\n" + TestDoc + "\n\nThe document as read looks like this:\n\n" + Buff);

		System.out.println("You should see a warning logged next....");

	//	The following line will throw an exception if there's any problem parsing the document.

		new XMLDocument(new ByteArrayInputStream(TestDoc.getBytes(CharSet)), 0, "Test Markup");
	}

	public void testReader3() throws IOException, TagException {
		final String			CharSet   = "UTF-8";
		final String			TestDoc   = "<?xml version=\"1.0\"?><html><head><title>Test</title></head><body><h1>Hello World</h1></body></html>";

		System.out.println("You should see a warning logged next....");

		final XHTMLReader		Rdr       = new XHTMLReader("Test Markup", new ByteArrayInputStream(TestDoc.getBytes(CharSet)));
		final StringBuilder		StrBuff   = new StringBuilder(TestDoc.length());
		final char[]			Buff      = new char[TestDoc.length()];
		int						CharsRead = Rdr.read(Buff);

		while (CharsRead >= 0) {

			StrBuff.append(Buff, 0, CharsRead);

			CharsRead = Rdr.read(Buff);
		}

		if (TestDoc.equals(StrBuff.toString()) == false)
			fail("The document as read by the XHTMLReader does not match the original document. The original looks " +
				 "like this:\n\n" + TestDoc + "\n\nThe document as read looks like this:\n\n" + StrBuff);

		System.out.println("You should see a warning logged next....");

	//	The following line will throw an exception if there's any problem parsing the document.

		new XMLDocument(new ByteArrayInputStream(TestDoc.getBytes(CharSet)), 0, "Test Markup");
	}

	public void testReader4() throws IOException, TagException {
		final String			CharSet   = "UTF-8";
		final String			TestDoc   = "<html><head><meta http-equiv='Content-Type' content='text/html; charset=" + CharSet + "'/></head><body><h1>\u2022 Hello World \u2022</h1></body></html>";

		final XHTMLReader		Rdr       = new XHTMLReader("Test Markup", new ByteArrayInputStream(TestDoc.getBytes(CharSet)));
		final StringBuilder		StrBuff   = new StringBuilder(TestDoc.length());
		final char[]			Buff      = new char[TestDoc.length()];
		int						CharsRead = Rdr.read(Buff);

		while (CharsRead >= 0) {

			StrBuff.append(Buff, 0, CharsRead);

			CharsRead = Rdr.read(Buff);
		}

		if (TestDoc.equals(StrBuff.toString()) == false)
			fail("The document as read by the XHTMLReader does not match the original document. The original looks " +
				 "like this:\n\n" + TestDoc + "\n\nThe document as read looks like this:\n\n" + StrBuff);

	//	The following line will throw an exception if there's any problem parsing the document.

		new XMLDocument(new ByteArrayInputStream(TestDoc.getBytes(CharSet)), 0, "Test Markup");
	}

	public void testXMLCharsetAutoDetect() throws IOException {
		final String			CharSet = "UTF-8";
		final String			TestDoc = "<?xml version=\"1.0\" encoding=\"" + CharSet + "\"?><html><head><title>Test</title></head><body><h1>\u2022 Hello World \u2022</h1></body></html>";

		final String			CharSetInUse = testCharsetAutoDetect(CharSet, TestDoc, true);

		assertTrue("The character set should have automatically switched to \"" + CharSet + "\", but is actually \"" + CharSetInUse + "\".", CharSet.equalsIgnoreCase(CharSetInUse));
	}

	public void testXMLCharsetAutoDetectWithIncorrectCharsetSpecification() throws IOException {
		final String			CharSet = "US-ASCII";
		final String			TestDoc = "<?xml version=\"1.0\" encoding=\"" + CharSet + "\"?><html><head><title>Test</title></head><body><h1>\u2022 Hello World \u2022</h1></body></html>";

		final String			CharSetInUse = testCharsetAutoDetect(CharSet, TestDoc, false);

		assertTrue("The character set should have automatically switched to \"" + CharSet + "\", but is actually \"" + CharSetInUse + "\".", CharSet.equalsIgnoreCase(CharSetInUse));
	}

	public void testXMLCharsetAutoDetectWithoutCharsetSpecification() throws IOException, TagException {
		final String			CharSet = "US-ASCII";
		final String			TestDoc = "<?xml version='1.0'?><html><head><title>Test</title></head><body><h1>\u2022 Hello World \u2022</h1></body></html>";

		System.out.println("You should see a warning logged next....");

		final String			CharSetInUse = testCharsetAutoDetect(CharSet, TestDoc, false);

		new XMLString(TestDoc);

	//	A warning will be logged to tell use that no character set specification was found. 
	//	We currently have no automatic means of testing this.
	}

	public void testXHTMLCharsetAutoDetect() throws IOException {
		final String			CharSet = "UTF-8";
		final String			TestDoc = "<html><head><meta http-equiv='Content-Type' content='text/html; charset=" + CharSet + "'/></head><body><h1>\u2022 Hello World \u2022</h1></body></html>";

		final String			CharSetInUse = testCharsetAutoDetect(CharSet, TestDoc, true);

		assertTrue("The character set should have automatically switched to \"" + CharSet + "\", but is actually \"" + CharSetInUse + "\".", CharSet.equalsIgnoreCase(CharSetInUse));
	}

	public void testXHTMLCharsetAutoDetectWithUTF8BOM() throws IOException {
		final String				CharSet = "UTF-8";
		final ByteArrayOutputStream	BOut    = new ByteArrayOutputStream();

		BOut.write(0xEF);
		BOut.write(0xBB);
		BOut.write(0xBF);
		BOut.write("Hello World.".getBytes(CharSet));

		final byte[]			Bytes = BOut.toByteArray();

//		System.out.println(Hexadecimal.andASCIIFromBytes(Bytes));

		final String			CharSetInUse = testCharsetAutoDetect(new ByteArrayInputStream(Bytes));

		assertTrue("The character set should have automatically switched to \"" + CharSet + "\", but is actually \"" + CharSetInUse + "\".", CharSet.equalsIgnoreCase(CharSetInUse));
	}

	public void testXHTMLCharsetAutoDetectWithUTF16BEBOM() throws IOException {
		final String				CharSet = "UTF-16BE";
		final ByteArrayOutputStream	BOut    = new ByteArrayOutputStream();

		BOut.write(0xFE);
		BOut.write(0xFF);
		BOut.write("Hello World.".getBytes(CharSet));

		final byte[]			Bytes = BOut.toByteArray();

//		System.out.println(Hexadecimal.andASCIIFromBytes(Bytes));

		final String			CharSetInUse = testCharsetAutoDetect(new ByteArrayInputStream(Bytes));

		assertTrue("The character set should have automatically switched to \"" + CharSet + "\", but is actually \"" + CharSetInUse + "\".", CharSet.equalsIgnoreCase(CharSetInUse));
	}

	public void testXHTMLCharsetAutoDetectWithUTF16LEBOM() throws IOException {
		final String				CharSet = "UTF-16LE";
		final ByteArrayOutputStream	BOut    = new ByteArrayOutputStream();

		BOut.write(0xFF);
		BOut.write(0xFE);
		BOut.write("Hello World.".getBytes(CharSet));

		final byte[]			Bytes = BOut.toByteArray();

//		System.out.println(Hexadecimal.andASCIIFromBytes(Bytes));

		final String			CharSetInUse = testCharsetAutoDetect(new ByteArrayInputStream(Bytes));

		assertTrue("The character set should have automatically switched to \"" + CharSet + "\", but is actually \"" + CharSetInUse + "\".", CharSet.equalsIgnoreCase(CharSetInUse));
	}

	public void testXHTMLCharsetAutoDetectWithIncorrectCharsetSpecification() throws IOException {
		final String			CharSet = "US-ASCII";
		final String			TestDoc = "<html><head><meta http-equiv='Content-Type' content='text/html; charset=" + CharSet + "'/></head><body><h1>\u2022 Hello World \u2022</h1></body></html>";

		final String			CharSetInUse = testCharsetAutoDetect(CharSet, TestDoc, false);

		assertTrue("The character set should have automatically switched to \"" + CharSet + "\", but is actually \"" + CharSetInUse + "\".", CharSet.equalsIgnoreCase(CharSetInUse));
	}

	public void testXHTMLCharsetAutoDetectWithoutCharsetSpecification() throws IOException {
		final String			CharSet = "US-ASCII";
		final String			TestDoc = "<html><head></head><body><h1>\u2022 Hello World \u2022</h1></body></html>";

		System.out.println("You should see a warning logged next....");

		final String			CharSetInUse = testCharsetAutoDetect(CharSet, TestDoc, false);

	//	A warning will be logged to tell use that no character set specification was found. 
	//	We currently have no automatic means of testing this.
	}

	private static String testCharsetAutoDetect
	(
		final String			TestDocCharset,
		final String			TestDoc,
		final boolean			CorrectEqualityTestOutcome
	)
	throws IOException
	{
		final XHTMLReader		In   = new XHTMLReader("TestDoc", new ByteArrayInputStream(TestDoc.getBytes(TestDocCharset)));
		final StringBuilder		Buff = new StringBuilder(TestDoc.length());
		int						Char = In.read();

		while (Char >= 0) {

			Buff.append((char) Char);

			Char = In.read();
		}

		In.close();

		final String			TestDocRead = Buff.toString();

		if (CorrectEqualityTestOutcome != TestDocRead.equals(TestDoc)) {

			if (CorrectEqualityTestOutcome)
				fail("The document that was read from the HTMLReader differs from the one that supplied to the reader.");
			else
				fail("The document that was read from the HTMLReader is identical to the one that was supplied to the reader, but should NOT be.");
		}

		return In.getCharacterSetName();
	}
	
	private static String testCharsetAutoDetect
	(
		final InputStream		TestDoc
	)
	throws IOException
	{
		final XHTMLReader		In   = new XHTMLReader("TestDoc", TestDoc);
		final StringBuilder		Buff = new StringBuilder(256);
		int						Char = In.read();

		while (Char >= 0) {

			Buff.append((char) Char);

			Char = In.read();
		}

		In.close();

		return In.getCharacterSetName();
	}

/*	public void testGetCharsetFromContentType1() {
		final String			Charset = XHTMLReader.getCharSetFromContentType("text/html; charset=utf-8");

		assertTrue("The character set should be \"utf-8\". Instead it has been parsed as \"" + Charset + "\".", "utf-8".equals(Charset));
	}

	public void testGetCharsetFromContentType2() {
		final String			Charset = XHTMLReader.getCharSetFromContentType("charset=utf-8");

		assertTrue("The character set should be \"utf-8\". Instead it has been parsed as \"" + Charset + "\".", "utf-8".equals(Charset));
	}

	public void testGetCharsetFromContentType3() {
		final String			Charset = XHTMLReader.getCharSetFromContentType("charset = utf-8");

		assertTrue("The character set should be \"utf-8\". Instead it has been parsed as \"" + Charset + "\".", "utf-8".equals(Charset));
	}

	public void testGetCharsetFromContentType4() {
		final String			Charset = XHTMLReader.getCharSetFromContentType("text/html; charset = utf-8; thing");

		assertTrue("The character set should be \"utf-8\". Instead it has been parsed as \"" + Charset + "\".", "utf-8".equals(Charset));
	}

	public void testGetCharsetFromContentType5() {
		final String			Charset = XHTMLReader.getCharSetFromContentType("text/html;charset=utf-8;thing");

		assertTrue("The character set should be \"utf-8\". Instead it has been parsed as \"" + Charset + "\".", "utf-8".equals(Charset));
	}

	public void testGetCharsetFromContentType6() {
		final String			Charset = XHTMLReader.getCharSetFromContentType("text/html;charset='utf-8';thing");

		assertTrue("The character set should be \"utf-8\". Instead it has been parsed as \"" + Charset + "\".", "utf-8".equals(Charset));
	}

	public void testGetCharsetFromContentType7() {
		final String			Charset = XHTMLReader.getCharSetFromContentType("text/html;charset=\"utf-8\";thing");

		assertTrue("The character set should be \"utf-8\". Instead it has been parsed as \"" + Charset + "\".", "utf-8".equals(Charset));
	}

	public void testGetCharsetFromContentType8() {
		final String			Charset = XHTMLReader.getCharSetFromContentType("text/html; charset=\"utf-8\"");

		assertTrue("The character set should be \"utf-8\". Instead it has been parsed as \"" + Charset + "\".", "utf-8".equals(Charset));
	}
*/
}
