/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


//  MuAttribute.java
//  qwicap
//
//  Created by Chris W. Johnson on Tue Jul 15 2003.


package edu.utexas.its.eis.tools.qwicap.template.xml.mutable;


import edu.utexas.its.eis.tools.qwicap.template.xml.structure.*;
import edu.utexas.its.eis.tools.qwicap.util.Characters;
import edu.utexas.its.eis.tools.qwicap.util.HTMLEntityCodec;

import java.util.*;


/**	Represents a mutable attribute in a mutable XML document.
 * 
 *	@author Chris W. Johnson
 */
class MuAttribute extends MuNamedItem implements Attribute {
	
/*	private static final HTMLEntityCodec	DoubleQuoteNonEncoder = new HTMLEntityCodec(HTMLEntityCodec.kUnprintableOmit, HTMLEntityCodec.kLineSepUnaltered, false);
	private static final HTMLEntityCodec	DoubleQuoteEncoder    = MutableMarkup.HTML;
	
	static {
	
	//	Instruct the "normal" encoder not to encode double quote characters.
	
		DoubleQuoteNonEncoder.setEncoding('"', null);
	}
*/	
	private Characters			Value;			//	In Unicode, i.e. it has been through the entity decoder.
	private boolean				AnalyzeQuotes = true;
	private boolean				HasSingleQuote;
	private boolean				HasDoubleQuote;
	
	
	MuAttribute
	(
		final Characters		Name,
		final Characters		Value,
		final Namespace			Ns
	)
	{
	//	if (Name.equals("xmlns"))
	//		;	//	@todo Do something relevant to the fact that a default namespace is being defined. This is 
				//	challenging given our cache-friendly code and data structures which assert that tags and attributes 
				//	do not, and should not, be aware of their surroundings. 
		
	//	@todo How do we deal with namespace definitions of the form "xmlns:http='...'", given that the "xmlns:" prefix 
	//	will have already been stripped-off? Yes, the Ns parameter will correctly reflect the namespace to which this 
	//	attribute belongs, but it doesn't seem that attributes belong to the namespaces they define. Even if that's 
	//	not a problem, we still can't know directly from these parameters that this attribute is responsible for 
	//	defining a namespace.
		
		setName(Name);
		setValue(Value);
		setNamespace(Ns);
	}
	
	@Override public final boolean valueEquals
	(
		final Characters		Other
	)
	{
		return getValue().equals(Other);
	}
	
	@Override public final boolean valueIncludes
	(
		final Characters		Other
	)
	{
		final StringTokenizer	ST = new StringTokenizer(getValue().toString());
		
		while (ST.hasMoreTokens()) {
			
			if (ST.nextToken().equals(Other))
				return true;
		}
		
		return false;
	}
	
/**	This method performs the type of test specified by the Cascading Style Sheets (CSS)
	"|=" attribute selector. That selector matches the start of a hyphen-sepearated value
	list. For the match to be successful, the search string must either match everything
	from the start of the value list up to a hyphen character, or must match the entire
	value list. The search string MUST NOT include a trailing hyphen.
	
	An example from the CSS2 specification, section 5.8.1:
	
	The following rule will match for values of the "lang" attribute that begin with "en", 
	including "en", "en-US", and "en-cockney":

		*[LANG|="en"] { color : red }
*/	
	@Override public final boolean valueStartsWith
	(
		final Characters		Other
	)
	{
		if (valueEquals(Other))
			return true;
		
	//	Do a "startsWith" test here which insists that the character following the last 
	//	matching character be a hyphen.
		
		if (getValue().startsWith(Other)) {
			
			if (getValue().charAt(Other.length()) == '-')
				return true;
		}
		
		return false;
	}
	
	@Override public Characters getValue() {
		return Value;
	}
	
	public void setValue
	(
		final Characters		NewValue
	)
	{
		Value = NewValue;
		
//		if (NewValue != null)
//			Value = DoubleQuoteEncoder.decode(NewValue);
//		else
//			Value = null;
			
		AnalyzeQuotes = true;
	}
	
	final void addValue
	(
		final Characters		ValueToAppend
	)
	{
		if (Value != null)
			Value = Value.append(ValueToAppend);
		else
			Value = ValueToAppend;
		
		AnalyzeQuotes = true;
	}
	
	@Override public final Characters getContent() {
		return getValue();
	}
	
	@Override public final void setContent
	(
		final Characters		NewContent
	)
	{
		setValue(NewContent);
	}
	
	@Override public String toString() {
		return append(new StringBuilder(64)).toString();
	}
	
	@Override public final StringBuffer append
	(
		final StringBuffer		Buff
	)
	{
		
	//	NOTE: It is vital that we retrieve our value using the "getValue" method rather than using our instance
	//	variable "value", because MuClassAttribute overrides this class, and depends on this method (and most others
	//	in this class) calling getValue.
		
		final Characters		Value = getValue();
		
		if (AnalyzeQuotes) {
			
			if (Value != null) {
				
				HasSingleQuote = Value.indexOf('\'') >= 0;
				HasDoubleQuote = Value.indexOf('"' ) >= 0;
			
			} else {
				
				HasSingleQuote = false;
				HasDoubleQuote = false;
			}
			
			AnalyzeQuotes = false;
		}
		
		char					QuoteChar = '\'';
		HTMLEntityCodec			Encoder   = null;
		
		if (HasSingleQuote && HasDoubleQuote) {
		
		//	Cause the double quote characters in the value to be encoded, so that we can use double quotes to bound the value.
		
			Encoder   = AttributeUtilities.DoubleQuoteEncoder;
			QuoteChar = '"';
			
		} else if (HasSingleQuote)
			QuoteChar = '"';
		
		getName().append(Buff);
		Buff.append('=').append(QuoteChar);
		
		if (Value != null) {
			
			if (Encoder == null)
				Value.append(Buff);
			else {
				
			//	Note that we should not end-up here, if attribute names and (in this case) values are being encoded
			//	on the way in, and decoded on the way out, by the higher-level attribute-oriented methods in the Markup 
			//	classes.
				
			//	@todo Once we're more confident that all of those aformentioned bases have been covered, this should be eliminated.
				
				Encoder.encode(Value, Buff);
			}
		}
		
		Buff.append(QuoteChar);
		
		return Buff;
	}
	
	@Override public final StringBuilder append
	(
		final StringBuilder		Buff
	)
	{
		
	//	NOTE: It is vital that we retrieve our value using the "getValue" method rather than using our instance
	//	variable "value", because MuClassAttribute overrides this class, and depends on this method (and most others
	//	in this class) calling getValue.
		
		final Characters		Value = getValue();
		
		if (AnalyzeQuotes) {
			
			if (Value != null) {
				
				HasSingleQuote = Value.indexOf('\'') >= 0;
				HasDoubleQuote = Value.indexOf('"' ) >= 0;
			
			} else {
				
				HasSingleQuote = false;
				HasDoubleQuote = false;
			}
			
			AnalyzeQuotes = false;
		}
		
		char					QuoteChar = '\'';
		HTMLEntityCodec			Encoder   = null;
		
		if (HasSingleQuote && HasDoubleQuote) {
		
		//	Cause the double quote characters in the value to be encoded, so that we can use double quotes to bound the value.
		
			Encoder   = AttributeUtilities.DoubleQuoteEncoder;
			QuoteChar = '"';
			
		} else if (HasSingleQuote)
			QuoteChar ='"';
		
		getName().append(Buff);
		Buff.append('=').append(QuoteChar);
		
		if (Value != null) {
			
			if (Encoder == null)
				Value.append(Buff);
			else {
				
			//	Note that we should not end-up here, if attribute names and (in this case) values are being encoded
			//	on the way in, and decoded on the way out, by the higher-level attribute-oriented methods in the Markup 
			//	classes.
				
			//	@todo Once we're more confident that all of those aformentioned bases have been covered, this should be eliminated.
				
				Encoder.encode(Value, Buff);
			}
		}
		
		Buff.append(QuoteChar);
		
		return Buff;
	}
	
/*	public static String encodeAttributeName
	(
		final String			UnName
	)
	{
		return UnName;
	
	//	There's nothing wrong with the following line, but I'm concerned that its cumulative cost in execution 
	//	speed and memory usage is not proportional to its actual value, given that, as best I can tell, people 
	//	basically never use attribute names that require encoding/decoding.	
	//
	//	return MutableMarkup.HTML.encode(UnName);
	}
	
	public static Characters encodeAttributeName
	(
		final Characters		UnName
	)
	{
		return UnName;	
	
	//	There's nothing wrong with the following line, but I'm concerned that its cumulative cost in execution 
	//	speed and memory usage is not proportional to its actual value, given that, as best I can tell, people 
	//	basically never use attribute names that require encoding/decoding.	
	//
	//	return MutableMarkup.HTML.encode(UnName);
	}
	
	public static String decodeAttributeName
	(
		final String			EncName
	)
	{	
		return EncName;
	
	//	There's nothing wrong with the following line, but I'm concerned that its cumulative cost in execution 
	//	speed and memory usage is not proportional to its actual value, given that, as best I can tell, people 
	//	basically never use attribute names that require encoding/decoding.	
	//
	//	return MutableMarkup.HTML.decode(EncName);
	}
	
	public static Characters decodeAttributeName
	(
		final Characters		EncName
	)
	{
		return EncName;
	
	//	There's nothing wrong with the following line, but I'm concerned that its cumulative cost in execution 
	//	speed and memory usage is not proportional to its actual value, given that, as best I can tell, people 
	//	basically never use attribute names that require encoding/decoding.	
	//
	//	return MutableMarkup.HTML.decode(EncName);
	}
	
	public static String encodeAttributeValue
	(
		final String			UnValue
	)
	{
		if (UnValue == null)
			return null;
		
		final boolean			HasSingleQuote = UnValue.indexOf('\'') >= 0;
		final boolean			HasDoubleQuote = UnValue.indexOf('"') >= 0;
		HTMLEntityCodec			Encoder = DoubleQuoteNonEncoder;
		
		if (HasSingleQuote && HasDoubleQuote) {
			
		//	Cause the double quote characters in the value to encoded, so we can use double quotes to bound the value.
			
			Encoder = DoubleQuoteEncoder;
		}
		
		return Encoder.encode(UnValue);
	}
	
	public static Characters encodeAttributeValue
	(
		final Characters		UnValue	//	Unicode without any HTML entity encoding
	)
	{
		if (UnValue == null)
			return null;
		
		final boolean			HasSingleQuote = UnValue.indexOf('\'') >= 0;
		final boolean			HasDoubleQuote = UnValue.indexOf('"') >= 0;
		HTMLEntityCodec			Encoder = DoubleQuoteNonEncoder;
		
		if (HasSingleQuote && HasDoubleQuote) {
			
		//	Cause the double quote characters in the value to encoded, so we can use double quotes to bound the value.
			
			Encoder = DoubleQuoteEncoder;
		}
		
		return Encoder.encode(UnValue);
	}
	
	public static String decodeAttributeValue
	(
		final String			EncValue
	)
	{
		return MutableMarkup.HTML.decode(EncValue);
	}
	
	public static Characters decodeAttributeValue
	(
		final Characters		EncValue
	)
	{
		return MutableMarkup.HTML.decode(EncValue);
	}
*/
	
//	public boolean definesNamespace() {
//		
//	}
//	
//	public boolean beNamespaceAware() {
//		final boolean			HasNamespaceInfo = super.beNamespaceAware();	
//		
//		if (HasNamespaceInfo) {
//			
//			
//			
//		}
//		
//	}
	
}


/*
	public boolean beNamespaceAware() {
		final boolean			HasNamespaceInfo = super.beNamespaceAware();
		
		if (HasNamespaceInfo) {
		
		//	This attribute's name was found to contain a namespace prefix, e.g. "prefix:name".
			
			final Characters		Prefix = getNamespacePrefix();
			
			if (Namespace.kNamespaceAttributeName.equals(Prefix)) {
				
			//	The attribute's prefix is "xmlns", which means that it defines a namespace. Amongst this tag's children,
			//	this namespace will be specified by including this attribute's name as the prefix of tag/attribute names.
				
				setNamespace(new ImNamespace(getName(), getValue()));
				DefinesNamespace = true;
				
			} // else The attribute's namespace prefix references a namespace defined previously in the document. (Or 
			//	the namespace could be defined outside the document, if the document was intended for use with a 
			//	validating XML parser, in which case the document shouldn't be used with Qwicap, because Qwicap's 
			//	parser is non-validating.)
			
		} else if (nameEquals(Namespace.kNamespaceAttributeName)) {
			
		//	This attribute is named "xmlns" and has no prefix, so it defines the default namespace for this tag's
		//	children.
		//
		//	Note from "Namespaces in XML", <http://www.w3.org/TR/REC-xml-names/#defaulting>:
		//	"If the URI reference in a default namespace declaration is empty, then unprefixed elements in the scope 
		//	of the declaration are not considered to be in any namespace."
		//	
		//	We set the namespace of elements that are not in any namespace to the static constant Namespace.kNoNamespace.
			
			final Characters		URI = getValue();
			
			if (URI.length() == 0)
				setNamespace(Namespace.kNoNamespace);
			else
				setNamespace(new ImNamespace(null, getValue()));
			
			DefinesNamespace = true;
			
		} else {
			
		//	From "Namspaces in XML", <http://www.w3.org/TR/REC-xml-names/#defaulting>:
		//	"Note that default namespaces do not apply directly to attributes."
		//	
		//	This attribute has no namespace prefix, and its name is other than "xmlns", therefore it has no namespace.
			
			setNamespace(Namespace.kNoNamespace);
		}
		
		return HasNamespaceInfo;
	}
*/






