/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2008-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.template.xml.mutable;


import edu.utexas.its.eis.tools.qwicap.template.xml.structure.Declaration;
import edu.utexas.its.eis.tools.qwicap.util.Characters;

import java.io.IOException;
import java.io.PrintWriter;


/**	Represents mutable declaration elements (<samp>&lt;! ... &gt;</samp>) in XML documents.
 * 
 *	@author Chris W. Johnson
 */
class MuDeclaration extends MuNamedItem implements Declaration {

	private Characters		Content;


	MuDeclaration
	(
		final Declaration		Master
	)
	{
		Content = Master.getContent();
		setName(Master.getName());
	}

	@Override public void setContent
	(
		final Characters		NewContent
	)
	{
		Content = NewContent;
	}

	@Override public Characters getContent() {
		return Content;
	}

	@Override public String toString() {
		final Characters		Name = getName();
		final StringBuilder		Buff = new StringBuilder(2 + Name.length() + 1 + Content.length() + 1);
		
		Buff.append("<!");
		Name.append(Buff);
		
		if (Content.length() != 0) {
			
			Buff.append(' ');
			Content.append(Buff);
		}
		
		Buff.append('>');
		
		return Buff.toString();
	}

	@Override public void write
	(
		final PrintWriter Out
	)
	throws IOException
	{
		Out.write("<!");
		getName().write(Out);
		
		if (Content.length() != 0) {
			
			Out.write(' ');
			Content.write(Out);
		}
		
		Out.write('>');
	}
}
