/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


//  MuNamedItem.java
//  qwicap
//
//  Created by Chris W. Johnson on Tue Jul 15 2003.


package edu.utexas.its.eis.tools.qwicap.template.xml.mutable;


import edu.utexas.its.eis.tools.qwicap.template.xml.structure.*;
import edu.utexas.its.eis.tools.qwicap.util.Characters;


/**	Common ancestor of all mutable named items in mutable XML documents.
 * 
 *	@author Chris W. Johnson
 */
abstract class MuNamedItem extends MuItem implements NamedItem {
	
	private Characters			Name;
	private Namespace			Ns;
	private boolean				NsResolved;
	
	
	@Override public final boolean nameEquals
	(
		final NamedItem			NI
	)
	{
		return nameEquals(NI.getName());
	}
	
	@Override public final boolean nameEquals
	(
		final Characters		OtherName
	)
	{
		return Name.equals(OtherName);
	}
	
	@Override public final Characters getName() {
		return Name;
	}
	
	@Override public final String getNameString() {
		return Name.toString();
	}
	
	final void setName
	(
		final String			NewName
	)
	{
		setName(new Characters(NewName));
	}
	
	final void setName
	(
		final Characters		NewName
	)
	{
		Name = NewName;
	}
	
	@Override public final Namespace getNamespace() {
		return Ns;
	}
	
	public void setNamespace
	(
		final Namespace			NewNs
	) 
	{
		NsResolved = true;
		Ns         = NewNs;
	}
	
	boolean beNamespaceAware() {
		return Ns != null;
	}
	
	final Characters getNamespacePrefix() {
		
		if (NsResolved && Ns != null)
			return Ns.getName();	
		
		return null;
	}
	
	boolean namespaceResolved() {
		return NsResolved;
	}
}






