/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.template.xml.mutable;


import edu.utexas.its.eis.tools.qwicap.template.xml.structure.Namespace;
import edu.utexas.its.eis.tools.qwicap.util.Characters;


/**	Represents a mutable namespace in a mutable XML document.
 * 
 *	@author Chris W. Johnson
 */
final class MuNamespace implements Namespace {
	
	private Characters			Name;	//	Can be null
	private Characters			URI;	//	Can be null
	
	
	MuNamespace
	(
		final Namespace			Master
	)
	{
		this(Master.getName(), Master.getURI());
	}
	
	MuNamespace
	(
		final Characters		Name,
		final Characters		URI
	)
	{
		setName(Name);
		setURI(URI);
	}
	
	@Override public boolean isDefault()
	{
		return Name == null;
	}
	
	@Override public Characters getName()	//	Can return null.
	{
		return Name;
	}
	
	void setName
	(
		final String			Name
	)
	{
		setName(new Characters(Name));
	}
	
	void setName
	(
		final Characters		Name
	) 
	{
		this.Name = Name;
	}
	
	@Override public Characters getURI()
	{
		return URI;
	}

	void setURI
	(
		final Characters		URI
	)
	{
		this.URI = URI;
	}
	
	void setURI
	(
		final String			URI
	)
	{
		setURI(new Characters(URI));
	}
	
	@Override public boolean equals
	(
		final Object			OtherObj
	)
	{
		if (this == OtherObj)
			return true;
		
		if (OtherObj instanceof Namespace) {
			final Characters		OtherURI = ((Namespace) OtherObj).getURI();
			
			if (URI == null)
				return OtherURI == null;
			else
				return URI.equals(OtherURI);
			
		} else if (OtherObj instanceof Characters || OtherObj instanceof String) {
			
			if (URI != null)
				return URI.equals(OtherObj);
		}
		
		return false;
	}
	
	@Override public int hashCode() {
		return URI != null ? URI.hashCode() : 0;	
	}
	
	@Override public String toString() {
		return URI == null ? "" : URI.toString();
	}
}
