/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


//  Range.java
//  qwicap
//
//  Created by Chris W. Johnson on Mon Jul 14 2003.


package edu.utexas.its.eis.tools.qwicap.template.xml.mutable;


import edu.utexas.its.eis.tools.qwicap.template.xml.structure.Range;
import edu.utexas.its.eis.tools.qwicap.util.Characters;

import java.io.PrintWriter;
import java.io.IOException;


/**	Common ancestor of all sections of any type of mutable material in mutable XML documents.
 * 
 *	@author Chris W. Johnson
 */
abstract class MuRange implements Range, Mutable {
	
	@Override public final boolean rangeEquals
	(
		final Characters		Other
	)
	{
		return Other.equals(toString());
	}
	
	@Override public Object clone() {
		try {
			return super.clone();
		} catch (CloneNotSupportedException e) {
			throw new RuntimeException(e);
		}
	}
	
	@Override public void write
	(
		final PrintWriter		Out
	)
	throws IOException
	{
	//	Inefficient default implementation. Commonly used subclasses should override this.
	
		Out.write(toString());
	}
	
	@Override public boolean isStartTag()          { return false; }
	@Override public boolean isEndTag()            { return false; }
	@Override public boolean isEmptyTag()          { return false; }
	@Override public boolean isTagWithAttributes() { return false; }
}





