/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2008-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.template.xml.mutable;


import edu.utexas.its.eis.tools.qwicap.template.xml.IntraElementContentMatch;
import edu.utexas.its.eis.tools.qwicap.template.xml.Match;
import edu.utexas.its.eis.tools.qwicap.template.xml.TagException;
import edu.utexas.its.eis.tools.qwicap.template.xml.structure.Item;
import edu.utexas.its.eis.tools.qwicap.template.css.Results;
import edu.utexas.its.eis.tools.qwicap.util.Characters;


/**	Represents the content inside of a mutable CDATA or comment element, as though the content were a child of the 
 * 	CDATA or comment element. (Internally, the content is <i>not</i> separate from the CDATA or comment element, so 
 * 	treating the content as a child is merely a convenient fiction intended to keep the markup manipulation API simple 
 * 	and relatively consistent.)
 * 
 * 	@author Chris W. Johnson
 */
final class MutableIntraElementContentMatch extends IntraElementContentMatch {

	MutableIntraElementContentMatch
	(
		final MutableMatch		ElementMatch
	)
	{
		super(ElementMatch);
	}

	@Override public boolean equals
	(
		final Object			Other
	)
	{
		if (this == Other)
			return true;

		if (Other instanceof MutableIntraElementContentMatch)
			return ElementMatch.equals(((MutableIntraElementContentMatch) Other).ElementMatch);

		return false;
	}

	@Override public Match add
	(
		final boolean			AddBefore,
		final Object			NewStuff
	)
	throws TagException
	{
		((MutableMarkup) getMarkup()).addContent(ElementMatch, AddBefore, NewStuff);
		return this;
	}

	@Override public Match addContent
	(
		final boolean			InsertBeforeExistingContent,
		final Object			Content
	)
	throws TagException
	{
	//	todo Consider: should we just treat this like an invocation of add(boolean, Object)?

		throw new TagException("The addContent method cannot be applied to this Match instance, because this Match " +
				"represents the content of a CDATA or comment element, and content IS content; it does not possess " +
				"content. To add to this content, use the \"add\" method.");
	}

	@Override public void delete() throws TagException {
		((MutableMarkup) getMarkup()).deleteContent(ElementMatch);
	}

	@Override public Match deleteContent() throws TagException {

	//	todo Consider: should we just treat this like an invocation of delete()?

		throw new TagException("The deleteContent method cannot be applied to this Match instance, because this Match " +
				"represents the content of a CDATA or comment element, and content IS content; it does not possess " +
				"content. To delete this content, use the \"delete\" method.");
	}

	@Override public Match duplicate
	(
		final boolean			InsertDuplicateBefore
	)
	throws TagException
	{
		final Characters		Content = ((Item) get()).getContent();
		final StringBuilder		Buff    = new StringBuilder(Content.length() * 2);
		
		Content.append(Buff);
		Content.append(Buff);

		((MutableMarkup) getMarkup()).setContent(ElementMatch, Buff.toString());
		
		return this;
	}
	
	@Override public Results getContent
	(
		final Results			PrevResults
	)
	{
		return new Results(PrevResults);
	}
	
	@Override public Match replace
	(
		final Object			Replacement
	)
	throws TagException
	{
		((MutableMarkup) getMarkup()).setContent(ElementMatch, Replacement);
		return this;
	}
	
	@Override public Match setContent
	(
		final Object			Content
	) 
	throws TagException 
	{

	//	todo Consider: should we just treat this like an invocation of replace()?

		throw new TagException("The setContent method cannot be applied to this Match instance, because this Match " +
				"represents the content of a CDATA or comment element, and content IS content; it does not possess " +
				"content. To repalce this content with new content, use the \"replace\" method.");
	}
}
