package edu.utexas.its.eis.tools.qwicap.template.xml.mutable;


import junit.framework.TestCase;
import edu.utexas.its.eis.tools.qwicap.template.xml.immutable.XMLString;
import edu.utexas.its.eis.tools.qwicap.template.xml.immutable.XMLDocument;
import edu.utexas.its.eis.tools.qwicap.template.xml.immutable.ImmutableMarkup;
import edu.utexas.its.eis.tools.qwicap.template.xml.TagException;
import edu.utexas.its.eis.tools.qwicap.template.css.Results;

import java.io.ByteArrayInputStream;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Aug 4, 2006
 * Time: 1:49:06 PM
 * To change this template use File | Settings | File Templates.
 */
public class MutableMarkupJUnit extends TestCase {
	
	public void testInsertBefore() throws Exception {
		final XMLString			Markup1 = new XMLString("<?xml version='1.0' encoding='UTF-16'?>\n<html>\n<body>\n<p>Hello World.</p>\n</body>\n</html>");
		final MutableMarkup		Doc     = Markup1.getMutable();
		
		Doc.get("p").addBefore(new XMLString("<p>The quick, brown fox jumped over the lazy dog.</p>"));

		System.out.println(Doc);
		
		final String			DesiredOutput;
		
		DesiredOutput = "<?xml version='1.0' encoding='UTF-16'?>\n" +
			"<html>\n" +
			"<body>\n" +
			"<p>The quick, brown fox jumped over the lazy dog.</p><p>Hello World.</p>\n" +
			"</body>\n" +
			"</html>";
		
		assertTrue(Doc.toString().equals(DesiredOutput));
	}
	
	public void testInsertXMLDeclarationRemoval1() throws Exception {
		final XMLString			Markup1 = new XMLString("<?xml version='1.0' encoding='UTF-16'?>\n<html>\n<body>\n<p>Hello World.</p>\n</body>\n</html>");
		final XMLString			Markup2 = new XMLString("<?xml version='1.0' encoding='UTF-8'?><p>The quick, brown fox jumped over the lazy dog.</p>");
		
		final MutableMarkup		Doc = Markup1.getMutable();
		
		Doc.insert(Doc.first().get(0), true, Markup2);

		final String			DesiredOutput;
		
		DesiredOutput = "<?xml version='1.0' encoding='UTF-16'?>\n" +
			"<p>The quick, brown fox jumped over the lazy dog.</p><html>\n" +
			"<body>\n" +
			"<p>Hello World.</p>\n" +
			"</body>\n" +
			"</html>";
		
		assertTrue(Doc.toString().equals(DesiredOutput));
	}
	
	public void testInsertXMLDeclarationRemoval2() throws Exception {
		final XMLString			Markup1 = new XMLString("<?xml version='1.0' encoding='UTF-16'?>\n<html>\n<body>\n<p>Hello World.</p>\n</body>\n</html>");
		final XMLString			Markup2 = new XMLString("<?xml version='1.0' encoding='UTF-8'?><p>The quick, brown fox jumped over the lazy dog.</p>");
		
		final MutableMarkup		Doc = Markup1.getMutable();
		
		Doc.insert(Doc.get("p").get(0), true, Markup2);
		
		final String			DesiredOutput;
		
		DesiredOutput = "<?xml version='1.0' encoding='UTF-16'?>\n" +
			"<html>\n" + 
			"<body>\n" +
			"<p>The quick, brown fox jumped over the lazy dog.</p><p>Hello World.</p>\n" +
			"</body>\n" +
			"</html>";
		
		assertTrue(Doc.toString().equals(DesiredOutput));
	}
	
	public void testInsertXMLDeclarationRetention1() throws Exception {
		final XMLString			Markup1 = new XMLString("<html>\n<body>\n<p>Hello World.</p>\n</body>\n</html>");
		final XMLString			Markup2 = new XMLString("<?xml version='1.0' encoding='UTF-8'?>\n<p>The quick, brown fox jumped over the lazy dog.</p>");
		
		final MutableMarkup		Doc = Markup1.getMutable();
		
		Doc.insert(Doc.first().get(0), true, Markup2);

		final String			DesiredOutput;
		
		DesiredOutput = "<?xml version='1.0' encoding='UTF-8'?>\n" +
			"<p>The quick, brown fox jumped over the lazy dog.</p><html>\n" +
			"<body>\n" +
			"<p>Hello World.</p>\n" +
			"</body>\n" +
			"</html>";
		
		assertTrue(Doc.toString().equals(DesiredOutput));
	}
	
	public void testSetCharSetInXMLDeclaration() throws Exception {
		final XMLString			Markup = new XMLString("<?xml version='1.0' encoding='UTF-16'?>\n<html>\n<body>\n<div id='A'></div>\n<div id='B'></div>\n</body>\n</html>\n");
		
		if ("UTF-16".equals(Markup.getCharacterSet()) == false)
			fail("getCharacterSet() should have returned \"UTF-16\", but actually returned \"" + Markup.getCharacterSet() + "\".");
		
		final MutableMarkup		Doc = Markup.getMutable();
		
		if ("UTF-16".equals(Doc.getCharacterSet()) == false)
			fail("getCharacterSet() should have returned \"UTF-16\", but actually returned \"" + Doc.getCharacterSet() + "\".");
			
		Doc.setCharacterSet("UTF-8");
		
		if ("UTF-8".equals(Doc.getCharacterSet()) == false)
			fail("getCharacterSet() should have returned \"UTF-8\", but actually returned \"" + Doc.getCharacterSet() + "\".");
		
		final String			DocStr = Doc.toString();
		final String			CorrectStr = "<?xml version='1.0' encoding='UTF-8'?>\n<html>\n<body>\n<div id='A'></div>\n<div id='B'></div>\n</body>\n</html>\n";
		
		if (DocStr.equals(CorrectStr) == false)
			fail("The markup should read as follows:\n\n" + CorrectStr + "\n\nHowever, it actually reads:\n\n" + DocStr);
	}
	
	public void testSetCharSetInXHTMLHead() throws Exception {
		final XMLString			Markup = new XMLString("<html>\n<head>\n<meta http-equiv='Content-Type' content='text/html; charset=UTF-16'/>\n</head>\n<body>\n<div id='A'></div>\n<div id='B'></div>\n</body>\n</html>\n");
		
		if ("UTF-16".equals(Markup.getCharacterSet()) == false)
			fail("getCharacterSet() should have returned \"UTF-16\", but actually returned \"" + Markup.getCharacterSet() + "\".");
		
		final MutableMarkup		Doc = Markup.getMutable();
		
		if ("UTF-16".equals(Doc.getCharacterSet()) == false)
			fail("getCharacterSet() should have returned \"UTF-16\", but actually returned \"" + Doc.getCharacterSet() + "\".");
			
		Doc.setCharacterSet("UTF-8");
		
		if ("UTF-8".equals(Doc.getCharacterSet()) == false)
			fail("getCharacterSet() should have returned \"UTF-8\", but actually returned \"" + Doc.getCharacterSet() + "\".");
		
		final String			DocStr = Doc.toString();
		final String			CorrectStr = "<html>\n<head>\n<meta http-equiv='Content-Type' content='text/html; charset=UTF-8' />\n</head>\n<body>\n<div id='A'></div>\n<div id='B'></div>\n</body>\n</html>\n";
		
		if (DocStr.equals(CorrectStr) == false)
			fail("The markup should read as follows:\n\n" + CorrectStr + "\n\nHowever, it actually reads:\n\n" + DocStr);
	}
	
	public void testSetCharSetInAll() throws Exception {
		final XMLString			Markup = new XMLString("<?xml version='1.0' encoding='UTF-16'?>\n<html>\n<head>\n<meta http-equiv='Content-Type' content='text/html; charset=UTF-16'/>\n</head>\n<body>\n<div id='A'></div>\n<div id='B'></div>\n</body>\n</html>\n");
		
		if ("UTF-16".equals(Markup.getCharacterSet()) == false)
			fail("getCharacterSet() should have returned \"UTF-16\", but actually returned \"" + Markup.getCharacterSet() + "\".");
		
		final MutableMarkup		Doc = Markup.getMutable();
		
		if ("UTF-16".equals(Doc.getCharacterSet()) == false)
			fail("getCharacterSet() should have returned \"UTF-16\", but actually returned \"" + Doc.getCharacterSet() + "\".");
			
		Doc.setCharacterSet("UTF-8");
		
		if ("UTF-8".equals(Doc.getCharacterSet()) == false)
			fail("getCharacterSet() should have returned \"UTF-8\", but actually returned \"" + Doc.getCharacterSet() + "\".");
		
		final String			DocStr = Doc.toString();
		final String			CorrectStr = "<?xml version='1.0' encoding='UTF-8'?>\n<html>\n<head>\n<meta http-equiv='Content-Type' content='text/html; charset=UTF-8' />\n</head>\n<body>\n<div id='A'></div>\n<div id='B'></div>\n</body>\n</html>\n";
		
		if (DocStr.equals(CorrectStr) == false)
			fail("The markup should read as follows:\n\n" + CorrectStr + "\n\nHowever, it actually reads:\n\n" + DocStr);
	}
	
	public void testDuplicate() throws Exception {
		final XMLString			Markup = new XMLString("<html>\n<body>\n<div id='A'></div>\n<div id='B'></div>\n</body>\n</html>\n");
		final MutableMarkup		Doc = Markup.getMutable();
		
//		Markup.print();
//		System.out.println();
		
		Doc.get("html body div#A").get(0).duplicate(true).setContent("NEW");

//		Doc.print();
//		System.out.println();
		
		assertTrue(
			"The Match.duplicate(true) method is not functioning correctly.",
			"<html>\n<body>\n<div id='A'>NEW</div><div id='A'></div>\n<div id='B'></div>\n</body>\n</html>\n".equals(Doc.toString())
		);
	}
	
	public void testTextAreaTagSetContent() throws Exception {
		final XMLString			Markup = new XMLString("<html>\n<body>\n<textarea></textarea>\n</body>\n</html>\n");
		final MutableMarkup		Doc = Markup.getMutable();
		
//		Markup.print();
//		System.out.println();
		
	//	The newlines ('\n') should NOT be encoded as "<br/>" in this case.
		
		Doc.get("textarea").setContent("Line 1\nLine 2\nLine 3");
		
//		Doc.print();
//		System.out.println();
		
		assertTrue(
			"Newline characters are being encoded within \"textarea\" elements. This should not happen.",
			Doc.toString().equals("<html>\n<body>\n<textarea>Line 1\nLine 2\nLine 3</textarea>\n</body>\n</html>\n")
		);
	}
	
	public void testPreTagSetContent() throws Exception {
		final XMLString			Markup = new XMLString("<html>\n<body>\n<pre></pre>\n</body>\n</html>\n");
		final MutableMarkup		Doc = Markup.getMutable();
		
//		Markup.print();
//		System.out.println();
		
	//	The newlines ('\n') should NOT be encoded as "<br/>" in this case.
		
		Doc.get("pre").setContent("Line 1\nLine 2\nLine 3");
		
//		Doc.print();
//		System.out.println();
		
		assertTrue(
			"Newline characters are being encoded within \"pre\" elements. This should not happen.",
			Doc.toString().equals("<html>\n<body>\n<pre>Line 1\nLine 2\nLine 3</pre>\n</body>\n</html>\n")
		);
	}
	
	
/*	This test applies equally to all HTML tags, except for "pre" and "textarea", not just "p" tags. However, 
	testing with the "p" tag is sufficient.
*/
	public void testPTagSetContent() throws Exception {
		final XMLString			Markup = new XMLString("<html>\n<body>\n<p></p>\n</body>\n</html>\n");
		final MutableMarkup		Doc    = Markup.getMutable();
		
//		Markup.print();
//		System.out.println();
		
	//	The newlines ('\n') should NOT be encoded as "<br/>" in this case.
		
		Doc.get("p").setContent("Line 1\nLine 2\nLine 3");
		
//		Doc.print();
//		System.out.println();
		
		assertTrue(
			"Newline characters are NOT being encoded within \"p\" elements. This failure to encode newlines is an error.",
			Doc.toString().equals("<html>\n<body>\n<p>Line 1<br/>\nLine 2<br/>\nLine 3</p>\n</body>\n</html>\n")
		);
	}
	
	public void testCommentSetContent() throws Exception {
		final XMLString			Markup = new XMLString("<html><body><p>Hello World.</p><!-- The owls are not what they seem. --></body></html>");
		final MutableMarkup		Doc    = Markup.getMutable(); 
		
		Doc.getComments().setContent("42");

		System.out.println(Doc);
		
		assertTrue(
			"<html><body><p>Hello World.</p><!-- 42 --></body></html>".equals(Doc.toString())
		);
		
		Doc.getComments().setContent("--");

		System.out.println(Doc);
		
		assertTrue(
			"<html><body><p>Hello World.</p><!-- __ --></body></html>".equals(Doc.toString())
		);
	}
	
	public void testCommentDeleteContent() throws Exception {
		final XMLString			Markup = new XMLString("<html><body><p>Hello World.</p><!-- The owls are not what they seem. --></body></html>");
		MutableMarkup			Doc    = Markup.getMutable(); 
		
		Doc.getComments().getContent().delete();
		
		assertTrue(
			"<html><body><p>Hello World.</p><!--  --></body></html>".equals(Doc.toString())
		);
		
		Doc = Markup.getMutable();
		Doc.getComments().deleteContent();
		
		assertTrue(
			"<html><body><p>Hello World.</p><!--  --></body></html>".equals(Doc.toString())
		);
	}
	
	public void testCommentContentRetrieval() throws Exception {
		final XMLString			Markup = new XMLString("<html><body><p>Hello World.</p><![CDATA[ The owls are not what they seem. ]]></body></html>");
		final MutableMarkup		Doc    = Markup.getMutable();

		System.out.println(Doc.getCDATA().getContent().toString());
		
		assertTrue(
			"The owls are not what they seem.".equals(Doc.getCDATA().getContent().toString())
		);
	}
	
	public void testCDATASetContent() throws Exception {
		final XMLString			Markup = new XMLString("<html><body><p>Hello World.</p><![CDATA[ The owls are not what they seem. ]]></body></html>");
		final MutableMarkup		Doc    = Markup.getMutable(); 
		
		Doc.getCDATA().setContent("42");

		System.out.println(Doc);
		
		assertTrue(
			"<html><body><p>Hello World.</p><![CDATA[ 42 ]]></body></html>".equals(Doc.toString())
		);
		
		Doc.getCDATA().setContent("]]>");

		System.out.println(Doc);
		
		assertTrue(
			"<html><body><p>Hello World.</p><![CDATA[ ]]&gt; ]]></body></html>".equals(Doc.toString())
		);
	}
	
	public void testCDATADeleteContent() throws Exception {
		final XMLString			Markup = new XMLString("<html><body><p>Hello World.</p><![CDATA[ The owls are not what they seem. ]]></body></html>");
		MutableMarkup			Doc    = Markup.getMutable(); 
		
		Doc.getCDATA().getContent().delete();
		
		assertTrue(
			"<html><body><p>Hello World.</p><![CDATA[  ]]></body></html>".equals(Doc.toString())
		);
		
		Doc = Markup.getMutable();
		Doc.getCDATA().deleteContent();
		
		assertTrue(
			"<html><body><p>Hello World.</p><![CDATA[  ]]></body></html>".equals(Doc.toString())
		);
	}
	
	public void testCDATAContentRetrieval() throws Exception {
		final XMLString			Markup = new XMLString("<html><body><p>Hello World.</p><![CDATA[ The owls are not what they seem. ]]></body></html>");
		final MutableMarkup		Doc    = Markup.getMutable();

		System.out.println(Doc.getCDATA().getContent().toString());
		
		assertTrue(
			"The owls are not what they seem.".equals(Doc.getCDATA().getContent().toString())
		);
	}
	
	public void testDeclarationSetContent() throws Exception {
		final XMLString			Markup = new XMLString("<html><body><p>Hello World.</p><!DOCTYPE The owls are not what they seem.></body></html>");
		final MutableMarkup		Doc    = Markup.getMutable(); 
		
		Doc.getDeclarations().setContent("42");

		System.out.println(Doc);
		
		assertTrue(
			"<html><body><p>Hello World.</p><!DOCTYPE 42></body></html>".equals(Doc.toString())
		);
	}
	
	public void testDeclarationDeleteContent() throws Exception {
		final XMLString			Markup = new XMLString("<html><body><p>Hello World.</p><!DOCTYPE The owls are not what they seem.></body></html>");
		MutableMarkup			Doc    = Markup.getMutable(); 
		
		Doc.getDeclarations().getContent().delete();
		
		assertTrue(
			"<html><body><p>Hello World.</p><!DOCTYPE></body></html>".equals(Doc.toString())
		);
		
		Doc = Markup.getMutable();
		Doc.getDeclarations().deleteContent();
		
		assertTrue(
			"<html><body><p>Hello World.</p><!DOCTYPE></body></html>".equals(Doc.toString())
		);
	}
	
	public void testDeclarationContentRetrieval() throws Exception {
		final XMLString			Markup = new XMLString("<html><body><p>Hello World.</p><!DOCTYPE The owls are not what they seem.></body></html>");
		final MutableMarkup		Doc    = Markup.getMutable();

		System.out.println(Doc.getDeclarations().getContent().toString());
		
		assertTrue(
			"The owls are not what they seem.".equals(Doc.getDeclarations().getContent().toString())
		);
	}
	
	public void testNamespaceRetentionDuringMutableConversionWhenNamespaceIsDefined() throws Exception {
		final String			DocStr;
		
		DocStr = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
			"<wsdl:definitions targetNamespace='https://xmlgateway.its.utexas.edu:18443/xg/soap/gatewayA' xmlns:wsdl='http://schemas.xmlsoap.org/wsdl/'>\n" +
			"</wsdl:definitions>";
		
	//	Note that we do NOT use XMLString, because it is not namespace-aware by default, and therefore won't use the
	//	code that we're trying to test. (The XMLDocument constructor *is* namespace-aware by default.)
		
		final ImmutableMarkup	Markup = new XMLDocument(new ByteArrayInputStream(DocStr.getBytes("UTF-8")), 0);
		final MutableMarkup		Doc    = Markup.getMutable();

		System.out.println();
		System.out.println(Doc);
		System.out.println();	
		System.out.println();
		System.out.println();	
		
		Doc.get("[targetNamespace]").setAttribute("targetNamespace", "FRED");
		
		System.out.println(Doc);
		
		final String			RevisedDocStr;
		
		RevisedDocStr = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
			"<wsdl:definitions targetNamespace='FRED' xmlns:wsdl='http://schemas.xmlsoap.org/wsdl/'>\n" +
			"</wsdl:definitions>";
			
		assertTrue(RevisedDocStr.equals(Doc.toString()));
	}

	public void testNamespacePrefixRetentionDuringMutableConversionWhenNamespaceIsNotDefined() throws Exception {
		final String			DocStr;
		
		DocStr = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
			"<wsdl:definitions targetNamespace='https://xmlgateway.its.utexas.edu:18443/xg/soap/gatewayA'>\n" +
			"</wsdl:definitions>";
		
	//	Note that we do NOT use XMLString, because it is not namespace-aware by default, and therefore won't use the
	//	code that we're trying to test. (The XMLDocument constructor *is* namespace-aware by default.)
		
		final ImmutableMarkup	Markup = new XMLDocument(new ByteArrayInputStream(DocStr.getBytes("UTF-8")), 0);
		final MutableMarkup		Doc    = Markup.getMutable();

		System.out.println();
		System.out.println(Doc);
		System.out.println();
		System.out.println();
		System.out.println();	
		
		Doc.get("[targetNamespace]").setAttribute("targetNamespace", "FRED");
		
		System.out.println(Doc);
		
		final String			RevisedDocStr;
		
		RevisedDocStr = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
			"<wsdl:definitions targetNamespace='FRED'>\n" +
			"</wsdl:definitions>";
			
		assertTrue(RevisedDocStr.equals(Doc.toString()));
	}
	
	public void testGetsOnNamespaceAwareMarkup() throws TagException {
		final String			DocStr;
		
		DocStr = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
				"<!DOCTYPE html \n" +
				"\tPUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\"\n" +
				"\t\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n" +
				"<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\"> \n" +
				"<head>\n" +
				"\t<meta http-equiv=\"Content-Type\" content='text/html; charset=UTF-8'/>\n" +
				"\t<meta http-equiv=\"Content-Style-Type\" content=\"text/css\"/>\n" +
				"\t<link rel=\"stylesheet\" media=\"screen\" type=\"text/css\" title=\"Preferred\" href=\"namespaces.css\"/>\n" +
				"\t<title>UT Austin XML Gateway</title>\n" +
				"</head>\n" +
				"<body>\n" +
				"\t<div class='system-header'>\n" +
				"\t\t<form method='post' action=''>\n" +
				"\t\t<h1>\n" +
				"\t\t\t<span class='logo-letters'><span class='uta-logo-letters'>UTA</span><span class='xg-logo-letters'>XG</span></span>\n" +
				"\t\t\t<span class='controller'>for <span class='controller-name'>Your Name Here</span></span>\n" +
				"\t\t\t<input type=\"image\" name=\"n.special\" src=\"hidden.gif\" alt=\"Hidden, special options\" />\n" +
				"\t\t</h1>\n" +
				"\t\t</form>\n" +
				"\t</div>\n" +
				"\t\n" +
				"\t<div class='content'>\n" +
				"\t\t\n" +
				"\t\t<div class='legal-warning'>\n" +
				"\t\t<p class='text-align-center'><b>Unauthorized Access Prohibited</b></p>\n" +
				"\t\t<p>Use of University of Texas computer and networking facilities requires prior authorization. Unauthorized \n" +
				"\t\taccess is prohibited. Usage may be subject to security testing and monitoring. Abuse is subject to criminal \n" +
				"\t\tprosecution.\n" +
				"\t\t</p>\n" +
				"\t\t</div>\n" +
				"\t\t\n" +
				"\t\t<p class='welcome'>Welcome to The University of Texas at Austin XML Gateway. Not sure how to make use of \n" +
				"\t\tthe Gateway? See the <a href='docs/'>documentation</a>.\n" +
				"\t\t</p>\n" +
				"\t\t\n" +
				"\t\t<form method='post' action=''>\n" +
				"\t\t\t<table>\n" +
				"\t\t\t\t<tr class='namespace'>\n" +
				"\t\t\t\t\t<td>\n" +
				"\t\t\t\t\t\t<button name='namespace' value='namespace' type='submit'>Namespace Name</button>\n" +
				"\t\t\t\t\t</td>\n" +
				"\t\t\t\t\t<td class='description'>\n" +
				"\t\t\t\t\t\tA Namespace description file will be inserted here.\n" +
				"\t\t\t\t\t</td>\n" +
				"\t\t\t\t</tr>\n" +
				"\t\t\t</table>\n" +
				"\t\t</form>\n" +
				"\t</div>\n" +
				"\t\n" +
				"\t<div class='system-footer'>\n" +
				"\t\t<form method='post' action=''>\n" +
				"\t\t\t<div>\n" +
				"\t\t\t\t<button name='logout' value='logout' type='submit'>Logout</button>\n" +
				"\t\t\t</div>\n" +
				"\t\t</form>\n" +
				"\t</div>\n" +
				"\t\n" +
				"</body>\n" +
				"</html>";
		
		final XMLString			Doc = new XMLString(DocStr, true);
		
		assertTrue("Fully-qualified CSS selector is failing to select items in namespace-aware markup.", Doc.get("@namespace xhtml url(http://www.w3.org/1999/xhtml); xhtml|tr.namespace").notEmpty());
		
		assertTrue("Unqualified CSS selector is failing to select items in namespace-aware markup.", Doc.get("tr.namespace").notEmpty());
	}
	
	public void testReplacingContentWithItself() throws TagException {
		final String			DocStr;
		
	//	This test is expected to fail, since the material being inserted is the current content of the element whose 
	//	content is being set. As the first step in setting the content of an item, its existing contents are deleted. 
	//	In this case, that delete operation destroys the material that was to used as the element's new content.
	//
	//	All of that is normal, which is why this test is expected to fail. However, keeping this test around reminds us 
	//	of a potential improvement to the usability of the XML engine - making it smart enough that this test does not 
	//	fail.
		
		DocStr = "<table><thead><th class='title'>Fred</th></thead></table>";
		
		final MutableMarkup		XML = new XMLString(DocStr).getMutable();
		final Results			TH  = XML.get("*.title");
		
		TH.setContent(TH.getContent());
		
		assertTrue(XML.toString().equals(DocStr));
	}
}
