/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2008 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


//  Attribute.java
//  qwicap
//
//  Created by Chris W. Johnson on Mon Jul 14 2003.


package edu.utexas.its.eis.tools.qwicap.template.xml.structure;


import edu.utexas.its.eis.tools.qwicap.util.Characters;


/**	Common interface to all classes representing the attributes of tags and XML declarations (for example: 
 * 	"<samp>id='thing'</samp>") in XML documents.
 * 
 *	@author Chris W. Johnson
 */
public interface Attribute extends NamedItem {

/**	Tests the value of this attribute for equality with the specified characters. The comparison is sensitive to case.
 * 
 * @param Other				The set of characters to be tested for equality with the value of this attribute.
 * @return					<code>true</code> if the value of this attribute and the specified set of characters are
 * 							equal, <code>false</code> otherwise.
 */
	public boolean			valueEquals(Characters Other);
	
/**	Assumes that the value of this attribute is a whitespace-separated list of values, and determines whether the 
 * 	specified value is present in that list of values. The comparison is sensitive to case.
 * 
 * @param Other				The value to be searched for among the whitespace-separated list of values that make-up the
 * 							value of this attribute.
 * @return					<code>true</code> if the specified value is present in the list of values that make-up the
 * 							value of this attribute, <code>false</code> otherwise.
 */
	public boolean			valueIncludes(Characters Other);

/**	Assumes that the value of this attribute, and the supplied set of characters, are both hyphen ('-') separated lists 
 * 	of values, and determines whether the specified set of values matches the values at the start of this attribute's 
 * 	list. A value in the list is considered to be matched only if it is <i>completely</i> matched by a corresponding 
 * 	value in the supplied set of characters. The supplied set of characters must not end with a hyphen. The comparison
 * 	is sensitive to case.
 * 
 * @param Other				The set of hyphen-separated values to be tested for equality with the initial 
 * 							hyphen-separated values of this attribute. The characters of this parameter must not end 
 * 							with a hyphen.
 * @return					<code>true</code> if the hyphen-separated values at the start of this attribute's value 
 * 							match the specified set hyphen-separated values, <code>false</code> otherwise.
 */
	public boolean			valueStartsWith(Characters Other);

/**	Returns the value of this attribute.
 * 
 * @return					The value of this attribute.
 */
	public Characters		getValue();

/**	Appends this attribute to the specified <code>StringBuffer</code>.
 * 
 * @param Buff				The buffer to which this attribute should be appended.
 * @return					The buffer specified by the <code>Buff</code> parameter.
 */
	public StringBuffer		append(StringBuffer Buff);

/**	Appends this attribute to the specified <code>StringBuilder</code>.
 * 
 * @param Buff				The buffer to which this attribute should be appended.
 * @return					The buffer specified by the <code>Buff</code> parameter.
 */
	public StringBuilder	append(StringBuilder Buff);
}


