/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2008 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


//  Item.java
//  qwicap
//
//  Created by Chris W. Johnson on Mon Jul 14 2003.


package edu.utexas.its.eis.tools.qwicap.template.xml.structure;


import edu.utexas.its.eis.tools.qwicap.util.Characters;


/**	Common ancestor of all classes representing sections of XML documents that contain material other than
 *	whitespace.
 * 
 *	@author Chris W. Johnson
 */
public interface Item extends Range {

/**	Tests the content of this item for equality with a set of characters. (See {@link #getContent} for an explanation 
 * 	of what constitutes "content" in this context.) The test is sensitive to case.
 * 
 * @param Contents		The set of characters to be tested for equality with this item's content.
 * @return				<code>true</code> if the content of this item matches the specified characters, 
 * 						<code>false</code> otherwise.
 */
	public boolean		contentEquals(Characters Contents);

/**	Returns a {@link Characters} object containing the content of this item. For an XML element like 
 * 	"<samp>&lt;a href='qwicap.html'></samp>", the content is "<samp>a href='qwicap.html'</samp>". For the text 
 * 	between XML elements, the content is the text without any leading or trailing whitespace. (If the text is 
 * 	entirely whitespace, the content is considered to consist of zero characters.) For an XML comment like
 * 	"<samp>&lt-- Hello, world. --></samp>", the content is "<samp>Hello, world.</samp>".
 * 
 * @return				The content of this item.
 */	
	public Characters	getContent();
}
