/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2008 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


//  NamedItem.java
//  qwicap
//
//  Created by Chris W. Johnson on Mon Jul 14 2003.


package edu.utexas.its.eis.tools.qwicap.template.xml.structure;


import edu.utexas.its.eis.tools.qwicap.util.Characters;


/**	Common interface of all named items in XML documents.
 * 
 *	@author Chris W. Johnson
 */
public interface NamedItem extends Item {
	
/**	Determines whether or not this item's name is the same as another item's name.	
 * 
 * @param NI			The item whose name should be compared with this item's name.
 * @return				<code>true</code> if the item names match, <code>false</code> otherwise.
 */
	public boolean		nameEquals(NamedItem NI);
	
/**	Determines whether or not this item's name is the same as the sequence of characters represented by a 
 *	{@link Characters} object.	
 * 
 * @param OtherName		A sequence of characters with which this item's name will be compared.
 * @return				<code>true</code> if the item name matches the supplied character sequence, <code>false</code> 
 *						otherwise.
 */	
	public boolean		nameEquals(Characters OtherName);
	
/**	Returns the name of this item. If <code>beNamespaceAware</code> has not been invoked, the name may include a 
 *	namespace prefix.
 * 
 * @return				The name of this item.
 */	
	public Characters	getName();
	
/**	Returns the name of this item as a <code>String</code>. If <code>beNamespaceAware</code> has not been invoked, the 
 *	name may include a namespace prefix.
 * 
 * @return				The name of this item.
 */
	public String		getNameString();
	
/**	Returns a <code>Namespace</code> object describing the XML namespace to which this item belongs.	
 * 
 * @return				The <code>Namespace</code> object describing the namespace to which this item belongs.
 */
	public Namespace	getNamespace();
}
