/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.template.xml.structure;


import edu.utexas.its.eis.tools.qwicap.util.Characters;
import edu.utexas.its.eis.tools.qwicap.template.xml.immutable.ImNamespace;


/** Common interface of all namespace designations in XML documents.
 *	
 *	@author Chris W. Johnson
 */
public interface Namespace {
	
	public static final ImNamespace	kXMLNamespace   = new ImNamespace("xml", "http://www.w3.org/XML/1998/namespace");
	public static final ImNamespace kXMLNSNamespace = new ImNamespace("xmlns", "http://www.w3.org/2000/xmlns/");	//	See <http://www.w3.org/TR/REC-xml-names/#ns-decl>.
	public static final ImNamespace	kNoNamespace    = new ImNamespace((String) null, (String) null);				//	For things not in any namespace.
	public static final ImNamespace	kXHTMLNamespace = new ImNamespace("xhtml", "http://www.w3.org/1999/xhtml");

//	From "Namespace in XML", <http://www.w3.org/TR/REC-xml-names/#ns-using>:
//	"The prefix xml is by definition bound to the namespace name http://www.w3.org/XML/1998/namespace."

	public static final Characters	kXMLNamespacePrefix = new Characters("xml");
	
//	This constant, "xmlns", can be either the name, or the prefix, of an attribute. Either way, the value of the
//	attribute specifies the URI of a namespace. When used as a prefix, the attribute name can be used by 
//	subsequent elements and attributes as a prefix to specify their association with the namespace defined by the
//	original attribute.
	
	public static final Characters	kNamespaceAttributeName = new Characters("xmlns");

	@Override public int		hashCode();
	@Override public boolean	equals(Object Other);	//	Can be another Namespace, Character or String object. In all cases, equivalent URIs cause true to be returned.
	
	public boolean		isDefault();
	public Characters	getName();	//	Can return null.
	public Characters	getURI();
}
