/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2009 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


//  TagWithAttributes.java
//  qwicap
//
//  Created by Chris W. Johnson on Mon Jul 14 2003.


package edu.utexas.its.eis.tools.qwicap.template.xml.structure;


import edu.utexas.its.eis.tools.qwicap.util.Characters;
import edu.utexas.its.eis.tools.qwicap.template.xml.AttributeEnumeration;


/**	Common interface of all tags that can possess attributes (for example: "<samp>&lt;p class='thing'&gt;</samp>") 
 * 	in XML documents.
 * 
 *	@author Chris W. Johnson
 */
public interface TagWithAttributes extends Tag, Iterable<Attribute> {

	public static final Characters	kClassAttributeName = new Characters("class");

	
/**	Returns <code>true</code> if this tag has a "class" attribute and the value of that attribute includes the 
 *	specified class name.
 * 
 * @param ClassName				The name of the class for which to test.
 * @return						<code>true</code> if this tag has a "class" attribute whose value includes the named
 *								class, otherwise <code>false</code>.
 */
	public boolean				isOfClass(Characters ClassName);

/**	Returns a reference to the specified attribute of this tag, or <code>null</code> if this tag has no such attribute.
 * 
 * @param AttrName				The name of the attribute to retrieve from this tag.
 * @return						A reference to the requested attribute of this tag, or <code>null</code> if this tag 
 * 								has no such attribute.
 */
	public Attribute			getAttribute(Characters AttrName);

/**	Returns a <code>String</code> representing the value of the requested attribute of this tag, or <code>null</code> 
 * 	if this tag has no such attribute.
 * 
 * @param AttrName				The name of the attribute whose value should be retrieved.
 * @return						A <code>String</code> representing the value of the requested attribute of this tag, 
 * 								or <code>null</code> if this tag has no such attribute.
 */
	public String				getAttribute(String AttrName);

/**	Returns an enumeration of the attributes of this tag. Note that the <code>{@link Iterable}&gt;Attribute&lt;</code> 
 *	interface, which makes possible the use of Java 1.5's "for each" loop, is also supported by this interface.
 * 
 * @return						An enumeration of the attributes of this tag.
 * @see #iterator
 */
	public AttributeEnumeration	enumerateAttributes();
}



