/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.template.xml.structure;


import edu.utexas.its.eis.tools.qwicap.util.Characters;
import edu.utexas.its.eis.tools.qwicap.template.xml.AttributeEnumeration;

import java.util.Iterator;


/** Common interface of all classes representing XML declarations ("<samp>&lt;?xml version="1.0" encoding="UTF-8"?&gt;</samp>")
 * 	in XML documents.
 * 
 *	@author Chris W. Johnson
 */
public interface XMLDeclaration extends Item, Iterable<Attribute> {

/**	Convenience method for getting the value of the "version" attribute, if any. For example: "1.0".
 * 
 * @return						The value of the "version" attribute, or <code>null</code> if there is no such 
 * 								attribute.
 */
	public String getVersion();

/**	Convenience method for getting the value of the "encoding" attribute, if any. For example: "UTF-8".
 * 
 * @return						The value of the "encoding" attribute, or <code>null</code> if there is no such 
 * 								attribute.
 */
	public String getEncoding();
	
/**	Returns a reference to the specified attribute of this tag, or <code>null</code> if this tag has no such attribute.
 * 
 * @param AttrName				The name of the attribute to retrieve from this tag.
 * @return						A reference to the requested attribute of this tag, or <code>null</code> if this tag 
 * 								has no such attribute.
 */
	public Attribute			getAttribute(Characters AttrName);

/**	Returns a <code>String</code> representing the value of the requested attribute of this tag, or <code>null</code> 
 * 	if this tag has no such attribute.
 * 
 * @param AttrName				The name of the attribute whose value should be retrieved.
 * @return						A <code>String</code> representing the value of the requested attribute of this tag, 
 * 								or <code>null</code> if this tag has no such attribute.
 */
	public String				getAttribute(String AttrName);

/**	Returns an enumeration of the attributes of this tag. Note that the <code>{@link Iterable}&gt;Attribute&lt;</code> 
 * 	interface, which makes possible the use of Java 1.5's "for each" loop, is also supported by this interface.
 * 
 * @return						An enumeration of the attributes of this tag.
 * @see #iterator
 */
	public AttributeEnumeration	enumerateAttributes();

/**	Returns an iterator for the attributes of this tag.
 * 
 * @return						An iterator for the attributes of this tag.
 */
	@Override public Iterator<Attribute>	iterator();
}
