/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2009 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.template.xml.util.xcsd;


import java.io.IOException;
import java.io.InputStream;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Jun 7, 2007
 * Time: 10:44:36 PM
 * To change this template use File | Settings | File Templates.
 */
final class MultiByteCharAccumulator {
	
	private final InputStream	In;
	private final int[]			Bytes;
	private final int			ByteCount;
	int							ByteIndex;

	MultiByteCharAccumulator
	(
		final InputStream		In,
		final int				ByteCount
	)
	{
		this.In        = In;
		this.ByteCount = ByteCount;
		this.Bytes     = new int[ByteCount];
	}

	boolean bytesAreReady() {
		return ByteIndex == Bytes.length;
	}

	int[] getBytes() {
		return Bytes;
	}

	boolean read
	(
		final boolean			CanBlock
	)
	throws IOException
	{
		int						Available      = In.available();
		final int				MinBytesNeeded = ByteCount - ByteIndex;
		
		if (CanBlock && Available < MinBytesNeeded)
			Available = MinBytesNeeded;
		
		if (ByteIndex == ByteCount)
			ByteIndex = 0;
		
		for (final int Count = ByteCount; ByteIndex < Count && Available > 0; ByteIndex++, Available--)
			Bytes[ByteIndex] = In.read();
		
		return bytesAreReady();
	}
}
