/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.template.xml.util.xcsd;


import java.io.InputStream;
import java.io.Reader;
import java.io.IOException;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Jun 7, 2007
 * Time: 10:24:01 PM
 * To change this template use File | Settings | File Templates.
 */
abstract class ReaderOfMultiByteCharacters extends Reader {
	
	private final InputStream				In;
	private final MultiByteCharAccumulator	CharBytes;
	
	
	ReaderOfMultiByteCharacters
	(
		final InputStream		In,
		final int				BytesPerChar
	)
	{
		this.In        = In;
		this.CharBytes = new MultiByteCharAccumulator(In, BytesPerChar);
	}

	@Override public void close() throws IOException {
		In.close();
	}
	
	@Override public int read() throws IOException {
		
	//	Strictly speaking, this implementation is inconsistent with read(char[], int, int), because it does NOT
	//	truncate the character codes it reads to two-byte 'char' values. However, readers based on this class are only
	//	meant to be able to read enough markup to find the character-set identifier in the markup, and all of the
	//	markup up to and including that identifier should specify only character codes in the ASCII range, so this
	//	difference in behaviors shouldn't make any difference, and does enable more complete testing, and slightly
	//	more efficient single-character reads.
		
		CharBytes.read(true);
		if (CharBytes.bytesAreReady())
			return convertBytesToChar(CharBytes);	
	
	//	todo Ideally, we'd be smart enough to encode >2 byte characters in whatever scheme Java uses in character arrays and Strings, but Qwicap's XML parser doesn't support that, anyway.
		
		return -1;
	}
	
	@Override public int read
	(
		final char[]			Chars, 
		final int				Offset, 
		final int				CharCount
	) 
	throws IOException 
	{
		int						CharsRead  = 0;
		int						CharsIndex = Offset;
		boolean					CanBlock   = true;
		
		while (CharsRead < CharCount) {
			
			CharBytes.read(CanBlock);
			if (CharBytes.bytesAreReady()) {
				
				CharsRead++;
				Chars[CharsIndex++] = (char) convertBytesToChar(CharBytes);
				
			//	todo Ideally, we'd be smart enough to encode >2 byte characters in whatever scheme Java uses in character arrays and Strings, but Qwicap's XML parser doesn't support that, anyway.
				
			} else
				break;
			
			CanBlock = false;
		}
		
		return CharsRead;
	}
	
	abstract int convertBytesToChar(MultiByteCharAccumulator MBC);
}
