/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.template.xml.util.xcsd;


import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.util.Set;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Jun 8, 2007
 * Time: 2:57:47 AM
 * To change this template use File | Settings | File Templates.
 */
final class UnsupportedEncodingForXMLDocumentException extends UnsupportedEncodingException {

	private final XMLCharacterSetDetector	CSD;


	UnsupportedEncodingForXMLDocumentException
	(
		final XMLCharacterSetDetector		CSD,
		final UnsupportedEncodingException	Cause
	)
	{
		super(CSD.getCharSetName());
		initCause(Cause);
		
		this.CSD = CSD;
	}
	
	String getCharacterSetName() {
		return CSD.getCharSetName();
	}

	@Override public String getMessage() {
		String					MarkupName = CSD.getMarkupName();
		
		if (MarkupName == null)
			MarkupName = "(no name specified)";
		else
			MarkupName = '"' + MarkupName + '"';
		
		if (CSD.getCharSetDetectionIsDefinitive()) {
		
		//	Note: We're replicating the functionality of ArrayToString so that this "xcsd" package can be built as a
		//	      stand-alone library for use elsewhere.
		
			final Set<String>		CharSets = Charset.availableCharsets().keySet();
			final StringBuilder		Buff     = new StringBuilder(CharSets.size() * 16);
			int						Index    = 0;
			
			for (final String CharSet : CharSets) {
			
				if (Index++ != 0)
					Buff.append(", ");
				
				Buff.append(CharSet);
			}
			
			return "The character set of the markup " + MarkupName + " was definitively specified by a byte-order-mark " +
				"(BOM) as \"" + CSD.getCharSetName() + "\". Unfortunately, Java does not recognize that character set. " +
				"(The character sets recognized by this Java installation are: " + 
				Buff + ".)";
			
		} else {
			
			return "The character set of the markup " + MarkupName + " could not be definitively determined. The " +
				"nearest this code could come to an identification was \"" + CSD.getCharSetName() + "\". Please add " +
				"a byte-order-mark (BOM) at the beginning of the markup in order to make the character set " +
				"identification unambiguous.";
		}
	}
}