/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2009 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.template.xml.util.xcsd;


import junit.framework.TestCase;

import java.io.*;
import java.util.regex.Matcher;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Jun 8, 2007
 * Time: 12:16:14 AM
 * To change this template use File | Settings | File Templates.
 */
public class XMLCharacterSetDetectorJUnit extends TestCase {

	
	public void testRegexCompleteDeclarationWithStandaloneAndContent() {
		final String			TestStr = "<?xml version=\"1.0\" encoding='UTF-16' standalone='no'?>\n<extra>The hounds of the night bayed to lonesome moon.</extra>";
		final Matcher			Match   = XMLCharacterSetDetector.XMLDeclarationPattern.matcher(TestStr);
		
		if (Match.matches() == false)
			fail();

		System.out.println("XML Declaration: " + Match.group(1));
		
		if ("1.0".equals(Match.group(3)) == false)
			fail();
			
		if ("UTF-16".equals(Match.group(5)) == false)
			fail();
	}
	
	public void testRegexCompleteDeclarationWithoutStandalone() {
		final String			TestStr = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
		final Matcher			Match   = XMLCharacterSetDetector.XMLDeclarationPattern.matcher(TestStr);
		
		if (Match.matches() == false)
			fail();

		System.out.println("XML Declaration: " + Match.group(1));
		
		if ("1.0".equals(Match.group(3)) == false)
			fail();
			
		if ("UTF-8".equals(Match.group(5)) == false)
			fail();
	}
	
	public void testRegexIncompleteDeclarationWithoutStandalone() {
		final String			TestStr = "<?xml version=\"1.0\" encoding=\"UTF-8\"";
		final Matcher			Match   = XMLCharacterSetDetector.XMLDeclarationPattern.matcher(TestStr);
		
		if (Match.matches() == false)
			fail();

		System.out.println("XML Declaration: " + Match.group(1));
		
		if ("1.0".equals(Match.group(3)) == false)
			fail();
			
		if ("UTF-8".equals(Match.group(5)) == false)
			fail();
	}
	
	public void testRegexCompleteDeclarationWithoutStandaloneWithContent() {
		final String			TestStr = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<!DOCTYPE properties SYSTEM \"http://java.";
		final Matcher			Match   = XMLCharacterSetDetector.XMLDeclarationPattern.matcher(TestStr);
		
		if (Match.matches() == false)
			fail();

		System.out.println("XML Declaration: " + Match.group(1));
		
		if ("1.0".equals(Match.group(3)) == false)
			fail();
			
		if ("UTF-8".equals(Match.group(5)) == false)
			fail();
	}
	
	public void testUTF8BOM() throws IOException {
		final byte[]					Bytes = { (byte) 0xEF, (byte) 0xBB, (byte) 0xBF, (byte) 'H', (byte) 'e', (byte) 'l', (byte) 'l', (byte) 'o' };
		final XMLCharacterSetDetector	CSD   = new XMLCharacterSetDetector("Test", new ByteArrayInputStream(Bytes));

		if (XMLCharacterSetDetector.UTF_8.equals(CSD.getCharSetName()) == false)
			fail("Character set should have been detected as 'UTF-8', but was actually detected as: " + CSD);

		final String				HelloStr = readString(CSD.getReader());

		if ("Hello".equals(HelloStr) == false)
			fail("Additional characters spelling the word 'Hello' should have been found, but what was actually found was: " + HelloStr);
	}

	public void testUTF8NoBOM() throws IOException {
		final byte[]					Bytes = { (byte) 'H', (byte) 'e', (byte) 'l', (byte) 'l', (byte) 'o' };
		final XMLCharacterSetDetector	CSD   = new XMLCharacterSetDetector("Test", new ByteArrayInputStream(Bytes));

		if (XMLCharacterSetDetector.UTF_8.equals(CSD.getCharSetName()) == false)
			fail("Character set should have defaulted to 'UTF-8', but was actually detected as: " + CSD);

		final String				HelloStr = readString(CSD.getReader());

		if ("Hello".equals(HelloStr) == false)
			fail("Characters spelling the word 'Hello' should have been found, but what was actually found was: " + HelloStr);
	}

	public void testUTF16BEBOM() throws IOException {
		final String					TestStr = "<?xml version='1.0' standalone='no'?><text>Blah blah blah.</text>";
		final XMLCharacterSetDetector	CSD     = new XMLCharacterSetDetector("Test", getStream("UTF-16BE", '\uFEFF' + TestStr));

		if (XMLCharacterSetDetector.UTF_16BE.equals(CSD.getCharSetName()) == false)
			fail("Character set should have been detected as 'UTF-16BE', but was actually detected as: " + CSD);

		final String				XMLStr = readString(CSD.getReader());

		if (TestStr.equals(XMLStr) == false)
			fail("Characters spelling \"" + TestStr + "\" should have been found, but what was actually found was: " + XMLStr);
	}

	public void testUTF16BEBOMWithBadDeclaration() throws IOException {

		System.out.println("The disagreement between the UTF-16BE byte-order-mark and the declared \"US-ASCII\" encoding should be logged immediately below. If not, something is wrong.");
		
		final String					TestStr = "<?xml version='1.0' encoding='US-ASCII' standalone='no'?><text>Blah blah blah.</text>";
		final XMLCharacterSetDetector	CSD     = new XMLCharacterSetDetector("Test", getStream("UTF-16BE", '\uFEFF' + TestStr));

		if (XMLCharacterSetDetector.UTF_16BE.equals(CSD.getCharSetName()) == false)
			fail("Character set should have been detected as 'UTF-16BE', but was actually detected as: " + CSD);

		final String				XMLStr = readString(CSD.getReader());

		if (TestStr.equals(XMLStr) == false)
			fail("Characters spelling \"" + TestStr + "\" should have been found, but what was actually found was: " + XMLStr);
	
	//	NOTE: The goal of this test is to see if the disagreement between the BOM and the declared encoding is logged.
	//	      We can't test for this directly (no easy way to intercept logged material), so the outcome of this test
	//	      can only properly be checked by a person.
	}

	public void testUTF16BENoBOM() throws IOException {
		final String					TestStr = "<?xml version='1.0' standalone='no'?><text>Blah blah blah.</text>";
		final XMLCharacterSetDetector	CSD     = new XMLCharacterSetDetector("Test", getStream("UTF-16BE", TestStr));

		if (XMLCharacterSetDetector.Two_Byte_Big_Endian.equals(CSD.getCharSetName()) == false)
			fail("Character set should have been provisionally detected as some sort of 16-bit, big-endian encoding, but, instead, was definitely detected as: " + CSD);

		final String				XMLStr = readString(CSD.getReader());

		if (TestStr.equals(XMLStr) == false)
			fail("Characters spelling \"" + TestStr + "\" should have been found, but what was actually found was: " + XMLStr);
	}

	public void testUTF16LEBOM() throws IOException {
		final String					TestStr = "<?xml version='1.0' standalone='no'?><text>Blah blah blah.</text>";
		final XMLCharacterSetDetector	CSD     = new XMLCharacterSetDetector("Test", getStream("UTF-16LE", '\uFEFF' + TestStr));

		if (XMLCharacterSetDetector.UTF_16LE.equals(CSD.getCharSetName()) == false)
			fail("Character set should have been detected as 'UTF-16LE', but was actually detected as: " + CSD);

		final String				XMLStr = readString(CSD.getReader());

		if (TestStr.equals(XMLStr) == false)
			fail("Characters spelling \"" + TestStr + "\" should have been found, but what was actually found was: " + XMLStr);
	}

	public void testUTF16LENoBOM() throws IOException {
		final String					TestStr = "<?xml version='1.0' standalone='no'?><text>Blah blah blah.</text>";
		final XMLCharacterSetDetector	CSD     = new XMLCharacterSetDetector("Test", getStream("UTF-16LE", TestStr));

		if (XMLCharacterSetDetector.Two_Byte_Little_Endian.equals(CSD.getCharSetName()) == false)
			fail("Character set should have been provisionally detected as some sort of 16-bit, little-endian encoding, but, instead, was definitely detected as: " + CSD);

		final String				XMLStr = readString(CSD.getReader());

		if (TestStr.equals(XMLStr) == false)
			fail("Characters spelling \"" + TestStr + "\" should have been found, but what was actually found was: " + XMLStr);
	}

	public void testUTF32BEBOM() throws IOException {
		final String					TestStr = "<?xml version='1.0' standalone='no'?><text>Blah blah blah.</text>";
		final XMLCharacterSetDetector	CSD     = new XMLCharacterSetDetector("Test", getStream("UTF-32BE", '\uFEFF' + TestStr));
		
		if (XMLCharacterSetDetector.UTF_32BE.equals(CSD.getCharSetName()) == false)
			fail("Character set should have been detected as 'UTF-32BE', but was actually detected as: " + CSD);
		
		final String				XMLStr = readString(CSD.getReader());
		
		if (TestStr.equals(XMLStr) == false)
			fail("Characters spelling \"" + TestStr + "\" should have been found, but what was actually found was: " + XMLStr);
	}
	
	public void testUTF32BENoBOM() throws IOException {
		final String					TestStr = "<?xml version='1.0' standalone='no'?><text>Blah blah blah.</text>";
		final XMLCharacterSetDetector	CSD     = new XMLCharacterSetDetector("Test", getStream("UTF-32BE", TestStr));

		if (XMLCharacterSetDetector.Four_Byte_Big_Endian.equals(CSD.getCharSetName()) == false)
			fail("Character set should have been provisionally detected as some sort of 32-bit, big-endian encoding, but, instead, was definitely detected as: " + CSD);

		final String				XMLStr = readString(CSD.getReader());

		if (TestStr.equals(XMLStr) == false)
			fail("Characters spelling \"" + TestStr + "\" should have been found, but what was actually found was: " + XMLStr);
	}
	
	public void testEBCDICNoBOM() throws IOException {
		final String					TestStr = "<?xml version='1.0' standalone='no'?><text>Blah blah blah.</text>";
		final XMLCharacterSetDetector	CSD   = new XMLCharacterSetDetector("Test", getStream("IBM1047", TestStr));

		if (XMLCharacterSetDetector.EBCDIC.equals(CSD.getCharSetName()) == false)
			fail("Character set should have been provisionally detected as some sort EBCDIC encoding, but, instead, was definitely detected as: " + CSD);

		final String				XMLStr = readString(CSD.getReader());

		if (TestStr.equals(XMLStr) == false)
			fail("Characters spelling \"" + TestStr + "\" should have been found, but what was actually found was: " + XMLStr);
	}
	
	public void testEBCDICNoBOMWithEncodingAttr() throws IOException {
		final String					TestStr = "<?xml version='1.0' encoding='IBM918' standalone='no'?><text>Blah blah blah.</text>";
		final XMLCharacterSetDetector	CSD   = new XMLCharacterSetDetector("Test", getStream("IBM1047", TestStr));

		if ("IBM918".equals(CSD.getCharSetName()) == false)
			fail("Character set should have been detected as the \"IBM918\" EBCDIC code page, but, instead, was definitely detected as: " + CSD);

		final String				XMLStr = readString(CSD.getReader());

		if (TestStr.equals(XMLStr) == false)
			fail("Characters spelling \"" + TestStr + "\" should have been found, but what was actually found was: " + XMLStr);
	}
	
	public void testASCIINoBOM() throws IOException {
		final String					TestStr = "<?xml version='1.0' encoding='US-ASCII' standalone='no'?><text>Blah blah blah.</text>";
		final XMLCharacterSetDetector	CSD     = new XMLCharacterSetDetector("Test", getStream("US-ASCII", TestStr));

		if ("US-ASCII".equals(CSD.getCharSetName()) == false)
			fail("Character set should have been identified as US-ASCII, but, instead, was identified as: " + CSD);
		
		final String				XMLStr = readString(CSD.getReader());

		if (TestStr.equals(XMLStr) == false)
			fail("The test string:\n\n" + TestStr + "\n\n...should have been found, but what was actually found was:\n\n" + XMLStr);
	}
	
	public void testBadDeclarationNoBOM() throws IOException {
		final String					TestStr = "<?xml version='1.0' encoding='IBM1047' standalone='no'?><text>Blah blah blah.</text>";
		final XMLCharacterSetDetector	CSD     = new XMLCharacterSetDetector("Test", getStream("US-ASCII", TestStr));

		if ("UTF-8".equals(CSD.getCharSetName()) == false)
			fail("Character set should have been identified as US-ASCII, but, instead, was identified as: " + CSD);
		
		final String				XMLStr = readString(CSD.getReader());

		if (TestStr.equals(XMLStr) == false)
			fail("The test string:\n\n" + TestStr + "\n\n...should have been found, but what was actually found was:\n\n" + XMLStr);
	}

	private static String readString
	(
		final Reader			RIn
	)
	throws IOException
	{
		final StringBuilder		Buff = new StringBuilder(64);
		int						Char = RIn.read();
		
		while (Char >= 0) {

			Buff.append((char) Char);

			Char = RIn.read();
		}

		RIn.close();

		return Buff.toString();
	}
	
	private static byte[] getWrittenBytes
	(
		final String			CharSet,
		final String			XMLStr
	) 
	throws IOException 
	{
		final ByteArrayOutputStream	BOut = new ByteArrayOutputStream(1024);
		final OutputStreamWriter	Out  = new OutputStreamWriter(BOut, CharSet);
		
		Out.write(XMLStr);
		Out.close();
		
		return BOut.toByteArray();
	}
	
	private static ByteArrayInputStream getStream
	(
		final String			CharSet,
		final String			XMLStr
	)
	throws IOException
	{
		return new ByteArrayInputStream(getWrittenBytes(CharSet, XMLStr));
	}
}
