/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2007 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.util;


import java.util.ArrayList;
import java.util.EmptyStackException;
import java.util.Stack;
import java.util.Vector;


/**	Implements a stack as a subclass of {@link ArrayList} (a non-synchronized class), unlike 
 *	{@link Stack} which subclasses the {@link Vector} class (which is synchronized). So, where
 *	a stack doesn't need to be safely accessed by concurrent threads, this class should be faster than 
 *	{@link Stack}.
 * 
 * 	@author Chris W. Johnson
 */
public final class ALStack<T> extends ArrayList<T> {
	
	public ALStack() { }
	
	public ALStack
	(
		final int				InitialCapacity
	)
	{
		super(InitialCapacity);
	}
	
	public T push
	(
		final T					Obj
	)
	{
		add(Obj);
		return Obj;
	}
	
	public T pop() {
		final int				Size = size();
		
		if (Size == 0)
			throw new EmptyStackException();
		
		final int				LastIndex = Size - 1;
		final T					FromTop = get(LastIndex);
		
		remove(LastIndex);
		
		return FromTop;
	}
	
	public T peek() {
		final int				Size = size();
		
		if (Size == 0)
			throw new EmptyStackException();
		
		return get(Size - 1);
	}
	
	public boolean notEmpty() {
		return !isEmpty();
	}
}
