/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.util;


import java.util.Iterator;
import java.util.NoSuchElementException;


/**	A generic <code>Iterator</code> for arrays. 
 * 
 * 	@author Chris W. Johnson
 */
public final class ArrayIterator<Type> implements Iterator<Type> {

	private final Type[]		Array;
	private final int			Limit;	//	Exclusive
	private int					Index;


/**	Iterate over all of the elements of the supplied array.
 * 
 * @param Array					The array to iterate.
 */
	public ArrayIterator
	(
		final Type[]			Array
	)
	{
		this(Array, 0, Array.length);
	}

/**	Iterate over all of the elements of the supplied array.
 * 
 * @param Array					The array to iterate.
 * @param Offset				The index of the first element in the array to be included in the iteration.
 * @param Count					The number of elements in the array to be included in the iteration.
 */
	public ArrayIterator
	(
		final Type[]			Array,
		final int				Offset,
		final int				Count
	)
	{
		this.Array = Array;
		this.Index = Offset;
		this.Limit = Offset + Count;
	}

	@Override public boolean hasNext() {
		return Index < Limit;
	}

	@Override public Type next() {
		
		if (!hasNext())
			throw new NoSuchElementException();
		
		return Array[Index++];
	}

	@Override public void remove() { }
}
