/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.util;


/**	Represents an HTML entity corresponding to a particular Unicode code point, for example "&amp;amp;" represents the 
 *	ampersand character ('&amp;').
 *	
 *	@author Chris W. Johnson
 */
class HTMLEntity implements Comparable<HTMLEntity> {
	
//	Note that the markup is case-sensitive. See the entities "&b.Gammad;" and "&b.gammad;" for an example - one 
// 	represents the capitalized form of the glyph, the other represents the "small" form of the glyph.
	
	final String			Markup;
	final int				UnicodeCodePoint;
	
	
	HTMLEntity
	(
		final String		Markup,
		final int			UnicodeCodePoint
	)
	{
		this.Markup           = Markup;
		this.UnicodeCodePoint = UnicodeCodePoint;
	}

	@Override public String toString() {
		return Markup;
	}
	
	@Override public int hashCode() {
		return UnicodeCodePoint;
	}
	
	@Override public boolean equals
	(
		final Object			Other
	)
	{
		if (Other instanceof HTMLEntity == false)
			return false;
		
		final HTMLEntity		OtherEntity = (HTMLEntity) Other;
		
		return UnicodeCodePoint == OtherEntity.UnicodeCodePoint && Markup.equals(OtherEntity.Markup);
	}
	
	@Override public int compareTo
	(
		final HTMLEntity		Other
	)
	{
		int						Signum = UnicodeCodePoint - Other.UnicodeCodePoint;
		
		if (Signum == 0)
			Signum = Markup.compareTo(Other.Markup);
		
		return Signum;
	}
}
