/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2010 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.util;


import edu.utexas.its.eis.tools.qwicap.servlet.HTTPContentType;
import edu.utexas.ots.chrisj.util.Hexadecimal;

import java.io.*;
import java.net.URL;
import java.net.URLConnection;
import java.util.regex.Pattern;
import java.util.regex.Matcher;
import java.util.List;
import java.util.ArrayList;
import java.util.Arrays;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Apr 8, 2010
 * Time: 9:39:57 PM
 * To change this template use File | Settings | File Templates.
 */
class HTMLEntityListBuilder {

/*	This utility works, but it doesn't handle characters that require more than 16 bits. Also, there's redundancy 
	present in the list <http://www.w3.org/2003/entities/2007/w3centities-f.ent> that was present in the list included
	in the HTML 4 specification. That redundancy will make the decoder side of the HTMLEntityCodec more robust, but 
	might cause the encoder some confusion (or cause it to choose entity representations not known to all browsers.
	

	private static final Pattern	kEntityPattern = Pattern.compile("^<!ENTITY\\s+(\\S+)\\s+\"\\s*&#(x?[0-9a-fA-F]+);\"\\s*>\\s*<!--\\s*(.*?)\\s*-->.*");
	
	
	public static void main(String[] Args) throws Exception {
		final ArrayList<HTMLEntityWithComment>	AllEntitiesList = new ArrayList<HTMLEntityWithComment>(1000);
		
		if (Args.length == 0)
			Args = new String[] { "http://www.w3.org/2003/entities/2007/w3centities-f.ent" };
		
		for (final String Arg : Args) {

			System.out.printf("%n%nLoading entities from <%s>....%n", Arg);
			
			final URL				EntityDocURL  = new URL(Arg);
			final URLConnection 	EntityDocConn = EntityDocURL.openConnection();
			
			EntityDocConn.setAllowUserInteraction(false);
			EntityDocConn.setDoOutput(false);
			EntityDocConn.setDoInput(true);
			
			EntityDocConn.connect();
			
			final InputStream		InStream = EntityDocConn.getInputStream();
			
			try {
				System.out.printf("Docuemnt Content-Type is \"%s\".%n", EntityDocConn.getContentType());
				
				final HTTPContentType	ContentType = new HTTPContentType(EntityDocConn.getContentType());
				final InputStreamReader	InReader    = new InputStreamReader(InStream, ContentType.getCharacterSet());

				System.out.printf("Document character set is \"%s\".%n", ContentType.getCharacterSet());
				
				AllEntitiesList.addAll(processEntityList(InReader));
				
			} finally {
				InStream.close();
			}
		}

		System.out.printf("%n%n%n%n");
		
		final HTMLEntityWithComment[]	AllEntities = AllEntitiesList.toArray(new HTMLEntityWithComment[AllEntitiesList.size()]);
		
		Arrays.sort(AllEntities);

		System.out.printf("Comprehensible Entities: %,d%n%n%n%n", AllEntities.length);
		
		int						MaxMarkupLength      = 0;
		int						MaxCharCodeHexLength = 0;
								
		for (final HTMLEntityWithComment E : AllEntities) {
			final int				MarkupLength = E.Markup.length();
			final int				HexLength    = hexadecimalPlaces(E.UnicodeCodePoint);
			
			if (MaxMarkupLength < MarkupLength)
				MaxMarkupLength = MarkupLength;
			if (MaxCharCodeHexLength < HexLength)
				MaxCharCodeHexLength = HexLength;
		}
		
		HTMLEntityWithComment			PrevEntity         = null;
		int								OmittedEntityCount = 0;
		
		for (final HTMLEntityWithComment E : AllEntities) {
			
			if (PrevEntity != null && PrevEntity.compareTo(E) == 0) {
				
				OmittedEntityCount++;
				continue;
			}
			
			System.out.println(E.toSourceCode(MaxMarkupLength, MaxCharCodeHexLength));
			
			PrevEntity = E;
		}

		System.out.printf("%n%nOmitted Entities: %,d%n%n%n%n", OmittedEntityCount);
	}
	
	private static List<HTMLEntityWithComment> processEntityList
	(
		final Reader			In
	)
	throws IOException
	{
		final ArrayList<HTMLEntityWithComment>	Entities = new ArrayList<HTMLEntityWithComment>(200);
		final BufferedReader					LineIn   = new BufferedReader(In, 1024 * 1024);
		
		for (String Line = LineIn.readLine(); Line != null; Line = LineIn.readLine()) {
			final Matcher			Match = kEntityPattern.matcher(Line);
			
			if (Match.matches()) {
				final String				EntityName    = Match.group(1);
				final String				EntityCodeStr = Match.group(2);
				final String				Comment       = Match.group(3);
				final HTMLEntityWithComment	Entity;
					
				if (EntityCodeStr.startsWith("x"))
					Entity = new HTMLEntityWithComment(EntityName, Hexadecimal.toInt(EntityCodeStr.substring(1)), Comment);
				else
					Entity = new HTMLEntityWithComment(EntityName, Integer.parseInt(EntityCodeStr), Comment);
				
				Entities.add(Entity);
				
			} else
				System.out.printf("\t%s%n", Line);
		}
		
		return Entities;
	}
	
	
	private static class HTMLEntityWithComment extends HTMLEntity {
		
		private final String		Comment;
		
		
		HTMLEntityWithComment
		(
			final String		EntityName,
			final int			UnicodeCodePoint,
			final String		Comment
		)
		{
			super('&' + EntityName + ';', UnicodeCodePoint);
			this.Comment = Comment;
		}
		
		String toSourceCode
		(
			final int				MaxMarkupLength,
			final int				MaxCharCodeHexLength
		) 
		{
			final StringBuilder		Buff = new StringBuilder(64);
			
			Buff.append("new HTMLEntity(\"").append(Markup).append("\",");
			
			for (int Index = (MaxMarkupLength - Markup.length()) + 1; --Index >= 0; )
				Buff.append(' ');
			
			Buff.append("0x").append(Hexadecimal.fromInt(UnicodeCodePoint).substring(8 - MaxCharCodeHexLength)).append("),	//	").append(Comment);
			
			return Buff.toString();
		}
	}
	
	private static int hexadecimalPlaces
	(
		int						No
	)
	{
		if (No == 0)
			return 1;
		
		int						NibbleCount = 0;
		
		while (No > 0) {
			
			NibbleCount++;
			No >>>= 4;
		}
		
		return NibbleCount;
	}
	
//	private static int decimalPlaces
//	(
//		int						PositiveNo
//	)
//	{
//		int						DecimalLength = 1;
//		
//		while (PositiveNo > 0) {
//			
//			DecimalLength++;
//			PositiveNo /= 10;
//		}
//		
//		return DecimalLength;
//	}

*/
}
