/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2008 Chris W. Johnson. All rights reserved.
	
	Chris W. Johnson's copyright notice in this evolving version reflects his original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.its.eis.tools.qwicap.util;


import java.io.StringWriter;
import java.io.PrintWriter;


/**	This class is not part of Qwicap's public API, but could not be hidden due to limitations in Java's access control
 * 	scheme.
 * 
 * 	@author Chris W. Johnson
 */
public class StackTrace {

	
	public static String toString
	(
		final Throwable 		T
	)
	{
		return stackTracesToString(T);
	}
	
	public static String toString
	(
		final Thread			T	
	)
	{
		try {
			return stackTraceToString(T.getStackTrace());
		} catch (SecurityException e) {
			return "The Security Manager prevented access to the thread's stack trace. (" + e.getMessage() + ')';
		}
	}
	
	public static String toString
	(
		final StackTraceElement[] 	Trace
	)
	{
		return stackTraceToString(Trace);
	}

	private static String stackTracesToString
	(
		final Throwable			T
	)
	{
		final StringWriter		SW = new StringWriter(4096);
		final PrintWriter		PW = new PrintWriter(SW);

		writeStackTrace(null, T, PW);

		Throwable				Cause = T.getCause();

		while (Cause != null) {

			writeStackTrace("Caused by: ", Cause, PW);
			Cause = Cause.getCause();
		}

		return SW.toString();
	}

	private static String stackTraceToString
	(
		final StackTraceElement[]	ST
	)
	{
		final StringWriter		SW = new StringWriter(2048);
		final PrintWriter		PW = new PrintWriter(SW);

		writeStackTrace(ST, PW);

		return SW.toString();
	}

	private static void writeStackTrace
	(
		final String			PrefixStr,	//	Can be null.
		final Throwable			T,
		final PrintWriter		PW
	)
	{
		if (PrefixStr != null)
			PW.print(PrefixStr);

		PW.print(T.getClass().getName());
		PW.print(": ");
		PW.println(T.getMessage());

	//	Note that we write-out the entire stack trace. It is for this reason alone that this method and 
	//	stackTracesToString were created, rather than just using Throwable.printStackTrace.

		writeStackTrace(T.getStackTrace(), PW);
	}
	
	private static void writeStackTrace
	(
		final StackTraceElement[]	ST,
		final PrintWriter			PW
	)
	{
	//	Note that we write-out the entire stack trace. It is for this reason alone that this method and 
	//	stackTracesToString were created, rather than just using Throwable.printStackTrace.
		
		for (final StackTraceElement STE : ST) {
			
			PW.print('\t');
			PW.println(STE.toString());
		}
	}
}
