/*	QWICAP (Quick Web Interface for Conventional Applications) by Chris W. Johnson
	Copyright (C) 2003-2007 The University of Texas at Austin ("U.T. Austin"). All rights reserved.
	
	U.T. Austin's copyright notice in this evolving version reflects our original authorship. It does not assert 
	copyright over changes contributed by others.
	
	This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General 
	Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) 
	any later version.
	
	This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for 
	more details.
	
	You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to 
	the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package edu.utexas.ots.chrisj.util;


import junit.framework.TestCase;


/**
 * Created by IntelliJ IDEA.
 * User: chrisj
 * Date: Nov 16, 2005
 * Time: 2:44:20 PM
 * To change this template use File | Settings | File Templates.
 */
public class HexadecimalJUnit extends TestCase {
	
	public void testHexAndASCIIConversions() {
		final byte[]			Bytes = new byte[256];
		
		for (int Index = 0, Count = Bytes.length; Index < Count; Index++)
			Bytes[Index] = (byte) Index;
		
		final String			HexAndASCII = Hexadecimal.andASCIIFromBytes(Bytes, 0, Bytes.length, 16, "\t", " ", "    ", true, "\n");

		final String			MasterStr = 
		"\t00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F    ................\n" + 
		"\t10 11 12 13 14 15 16 17 18 19 1A 1B 1C 1D 1E 1F    ................\n" + 
		"\t20 21 22 23 24 25 26 27 28 29 2A 2B 2C 2D 2E 2F     !\"#$%&'()*+,-./\n" + 
		"\t30 31 32 33 34 35 36 37 38 39 3A 3B 3C 3D 3E 3F    0123456789:;<=>?\n" + 
		"\t40 41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F    @ABCDEFGHIJKLMNO\n" + 
		"\t50 51 52 53 54 55 56 57 58 59 5A 5B 5C 5D 5E 5F    PQRSTUVWXYZ[\\]^_\n" + 
		"\t60 61 62 63 64 65 66 67 68 69 6A 6B 6C 6D 6E 6F    `abcdefghijklmno\n" + 
		"\t70 71 72 73 74 75 76 77 78 79 7A 7B 7C 7D 7E 7F    pqrstuvwxyz{|}~.\n" + 
		"\t80 81 82 83 84 85 86 87 88 89 8A 8B 8C 8D 8E 8F    ................\n" + 
		"\t90 91 92 93 94 95 96 97 98 99 9A 9B 9C 9D 9E 9F    ................\n" + 
		"\tA0 A1 A2 A3 A4 A5 A6 A7 A8 A9 AA AB AC AD AE AF    ................\n" + 
		"\tB0 B1 B2 B3 B4 B5 B6 B7 B8 B9 BA BB BC BD BE BF    ................\n" + 
		"\tC0 C1 C2 C3 C4 C5 C6 C7 C8 C9 CA CB CC CD CE CF    ................\n" + 
		"\tD0 D1 D2 D3 D4 D5 D6 D7 D8 D9 DA DB DC DD DE DF    ................\n" + 
		"\tE0 E1 E2 E3 E4 E5 E6 E7 E8 E9 EA EB EC ED EE EF    ................\n" + 
		"\tF0 F1 F2 F3 F4 F5 F6 F7 F8 F9 FA FB FC FD FE FF    ................\n";
		
		assertTrue(HexAndASCII.equals(MasterStr));
	}
	
	public void testDoubleConversions() {
		final double			Ref    = Double.MAX_VALUE;
		
		assertTrue("Double conversions are broken.", Hexadecimal.toDouble(Hexadecimal.fromDouble(Ref)) == Ref);
	}
	
	public void testFloatConversions() {
		final float				Ref    = Float.MAX_VALUE;
		
		assertTrue("Double conversions are broken.", Hexadecimal.toFloat(Hexadecimal.fromFloat(Ref)) == Ref);
	}
	
	public void testLongConversions() {
		final long				Ref   = 0x0123456789ABCDEFL;
		final String			RefStr = "0123456789ABCDEF";
		
		assertTrue("Conversion of longs to hex is broken.", RefStr.equals(Hexadecimal.fromLong(Ref)));
		
		assertTrue("Conversion of hex to longs is broken.", Hexadecimal.toLong(RefStr) == Ref);
		
		assertTrue("Conversion of negative values is broken.", "FFFFFFFFFFFFFFFF".equals(Hexadecimal.fromLong(-1L)));
	}
	
	public void testIntConversions() {
		final int				Ref   = 0x014589CD;
		final String			RefStr = "014589CD";
		
		assertTrue("Conversion of ints to hex is broken.", RefStr.equals(Hexadecimal.fromInt(Ref)));
		
		assertTrue("Conversion of hex to ints is broken.", Hexadecimal.toInt(RefStr) == Ref);
		
		assertTrue("Conversion of negative values is broken.", "FFFFFFFF".equals(Hexadecimal.fromInt(-1)));
	}
	
	public void testShortConversions() {
		final short				Ref   = 0x45CD;
		final String			RefStr = "45CD";
		
		assertTrue("Conversion of shorts to hex is broken.", RefStr.equals(Hexadecimal.fromShort(Ref)));
		
		assertTrue("Conversion of hex to shorts is broken.", Hexadecimal.toShort(RefStr) == Ref);
		
		assertTrue("Conversion of negative values is broken.", "FFFF".equals(Hexadecimal.fromShort((short) -1)));
	}
	
	public void testCharConversions() {
		final char				Ref   = 0x45CD;
		final String			RefStr = "45CD";
		
		assertTrue("Conversion of chars to hex is broken.", RefStr.equals(Hexadecimal.fromChar(Ref)));
		
		assertTrue("Conversion of hex to chars is broken.", Hexadecimal.toChar(RefStr) == Ref);
		
		assertTrue("Conversion of negative values is broken.", "FFFF".equals(Hexadecimal.fromChar((char) -1)));
	}
	
	public void testByteConversions() {
		final byte				Ref   = 0x4D;
		final String			RefStr = "4D";
		
		assertTrue("Conversion of bytes to hex is broken.", RefStr.equals(Hexadecimal.fromByte(Ref)));
		
		assertTrue("Conversion of hex to bytes is broken.", Hexadecimal.toByte(RefStr) == Ref);
		
		assertTrue("Conversion of negative values is broken.", "FF".equals(Hexadecimal.fromByte((byte) -1)));
	}
	
	public void testNibbleConversions() {
		final char[]			Nibbles = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
		
		for (int Nibble = 0; Nibble < 16; Nibble++)
			assertTrue("Conversion of nibbles (" + Nibble + ") to hex is broken.", Nibbles[Nibble] == Hexadecimal.fromNibble(Nibble)); 
		
		for (int Nibble = 0; Nibble < 16; Nibble++)
			assertTrue("Conversion of hex to nibbles (" + Nibble + ") is broken.", Nibble == Hexadecimal.toNibble(Nibbles[Nibble]));
	}
	
	public void testByteArrayConversions1() {
		final byte[]			Ref    = { 0x01, 0x23, 0x45, 0x67, (byte) 0x89, (byte) 0xAB, (byte) 0xCD, (byte) 0xEF };
		final String			RefStr = "0123456789ABCDEF";
		
		assertTrue("Conversion of byte arrays to hex is broken.", RefStr.equals(Hexadecimal.fromBytes(Ref)));
		
		final byte[]			Bytes = Hexadecimal.toBytes(RefStr);
		
		assertTrue("Conversion of a String to a byte array has returned the wrong number of bytes.", Bytes.length == Ref.length);
		
		for (int Index = 0, Count = Bytes.length; Index < Count; Index++)
			assertTrue("Conversion of a hexdecimal String to a byte array is broken.", Ref[Index] == Bytes[Index]);
	}
	
	public void testByteArrayConversions2() {
		final byte[]			Ref    = { 0x01, 0x23, 0x45, 0x67, (byte) 0x89, (byte) 0xAB, (byte) 0xCD, (byte) 0xEF };
		final String			RefStr = "01 2 3     45 6 7	8 \n\t9AB CDE F  ";	//	Whitespace in this string should be ignored.
		final byte[]			Bytes  = Hexadecimal.toBytes(RefStr, true);
		
		assertTrue("Conversion of a String to a byte array has returned the wrong number of bytes.", Bytes.length == Ref.length);
		
		for (int Index = 0, Count = Bytes.length; Index < Count; Index++)
			assertTrue("Conversion of a hexdecimal String to a byte array is broken.", Ref[Index] == Bytes[Index]);
	}
	
	public void testByteArrayConversions3() {
		final byte[]			Ref = { 0x01, 0x23, 0x45, 0x67, (byte) 0x89, (byte) 0xAB, (byte) 0xCD, (byte) 0xEF };
		
	//	Note that the leading zero ('0') character in RefStr has been omitted, but, because the string contains an odd
	//	number of hexadecimal digits, the high-nibble of the first byte should be set to zero automatically, and this
	//	string should still produce an array of bytes matching Ref.
		
		final String			RefStr = "1 2 3     45 6 7	8 \n\t9AB CDE F";	//	Whitespace should be ignored.
		final byte[]			Bytes  = Hexadecimal.toBytes(RefStr, true);
		
		assertTrue("Conversion of a String to a byte array has returned the wrong number of bytes.", Bytes.length == Ref.length);
		
		for (int Index = 0, Count = Bytes.length; Index < Count; Index++)
			assertTrue("Conversion of a hexdecimal String to a byte array is broken: " + Hexadecimal.fromBytes(Bytes), Ref[Index] == Bytes[Index]);
	}
}
